// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "document_controller.h"

#include "foundation/utility.h"
#include "foundation/undo_manager.h"
#include "foundation/macros.h"
#include "foundation/command_callback.h"

#include "uml/uml.h"

#include "reflective/class.h"
#include "reflective/context.h"
#include "reflective/attribute.h"
#include "reflective/iterator.h"
#include "reflective/context.h"
#include "reflective/changes.h"


#include "python/python.h"
#include "python/toolkit/document.h"
#include "python/reflective/context.h"
#include <sstream>
using namespace std;


//================================================================================

DocumentController::DocumentController(const char * package, Toolkit::Document * document) {
  Python::Module module = Python::Module("author");
  PyObject * args = Py_BuildValue("(sO)", package, Python::document_reference_new(document));
  mObject = Python::Callable(module, "factory").call(args);
  Py_XDECREF(args);
}

DocumentController::~DocumentController() {
  Py_XDECREF(mObject);  
}

void 
DocumentController::set_context(REF::Context * context) {
  PyObject * ref_context = Python::context_reference_new(context);
  PyObject * result = PyObject_CallMethod(mObject, "set_context", "(O)", ref_context);
  Py_DECREF(ref_context);
  if (result == NULL) {
    ERROR("Error returned by 'set_context'");
    if (PyErr_Occurred())
      PyErr_Print();
  }
  else {
    Py_DECREF(result);
  }
}  

