#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <libgnome/gnome-program.h>

#include <libgnomeui/gnome-ui-init.h>
#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-about.h>

#include <gtk/gtkmain.h>
#include <gtk/gtkhpaned.h>
#include <gtk/gtkvpaned.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkmessagedialog.h>

#include "foundation/macros.h"
#include "foundation/command_callback.h"
#include "foundation/undo_manager.h"

#include "uml/uml.h" // testing

#include "app/common/kit.h"

#include "gtk/menu.h"
#include "gtk/toolbar.h"
#include "gtk/statusbar.h"
#include "gtk/settings.h"
#include "gtk/image.h"
#include "gtk/factory.h"
#include "gtk/tree.h"
#include "gtk/document.h"
#include "gtk/log.h"

//#include "pygobject.h"
#include "python/python.h"

#include <iostream>
#include <cstdlib>
using namespace std;

#ifdef DEBUG
int trace_level = 2;
#endif

extern "C" {
    int FT_Get_Next_Char;
    int FT_Get_First_Char;
}

namespace {

    const string DISCLAIMER =
        "Thank you for trying version " VERSION " of Dachshund!\n"
        "\n"
        "This is a developer's  release and cannot\n"
        "be used to do real work.\n"
        "\n"
        "You can find out more about Dachshund by visiting\n"
        "the website: dachshund.sourceforge.net";

    //
    // options
    //

    int show_version_flag = 0;

    const poptOption OPTIONS[] = {
	{
	    "version",
	    'v',
	    POPT_ARG_NONE,
	    &show_version_flag,
	    0,
	    "Display version information",
	    0
	},

#ifdef DEBUG
	{
	    "trace",
	    't',
	    POPT_ARG_INT,
	    &trace_level,
	    0,
	    "Set debug level",
	    "[0-10]"
	},
#endif

	POPT_TABLEEND
    };

    //
    // callbacks
    //

    void
    about_destroy_callback(GtkWidget * widget, GtkWidget ** about) {
	TRACE(10, "");
	*about = 0;
    }

    gint
    main_delete_event_callback(GtkWidget * widget, GdkEvent * event, gpointer data) {
	TRACE(10, "");
	Kit * kit = Kit::instance();

	return kit->can_exit() ? FALSE : TRUE;
    }

    void
    main_destroy_callback(GtkWidget * widget, gpointer data) {
	TRACE(10, "");
	gtk_main_quit();
    }

    //
    // helpers
    //

    void
    show_version() {
	cout << "Dachshund " << VERSION << endl;
    }

    void
    show_about() {
	const char * authors[] = {
	    "David Bryant <daveb@acres.com.au>",
	    "Greg Horrigan <gregh@acres.com.au>",
	    "David Bentham <mr_dave@users.sourceforge.net>",
	    0
	};

	static GtkWidget * about = 0;

	if(!about) {
	    about = gnome_about_new(PACKAGE,
				    VERSION,
				    "Copyright 2000-2002 David Bryant",
				    "A UML Visual Modeling Tool",
				    authors,
				    0,
				    0,
				    0);

	    g_signal_connect(G_OBJECT(about), "destroy",
			     G_CALLBACK(about_destroy_callback), &about);
	    gtk_widget_show(about);
	}
	else {
	    gdk_window_raise(about->window);
	}
    }

    gboolean
    show_disclaimer(void * data) {
        GtkWindow * parent = static_cast<GtkWindow *>(data);

        GtkWidget * widget = gtk_message_dialog_new(parent,
                                                    GTK_DIALOG_MODAL,
                                                    GTK_MESSAGE_WARNING,
                                                    GTK_BUTTONS_OK,
                                                    DISCLAIMER.c_str());

        gtk_dialog_run(GTK_DIALOG(widget));
        gtk_widget_destroy(widget);

        return FALSE;
    }

    void
    do_exit() {
	Kit * kit = Kit::instance();

	if(kit->can_exit()) {
	    gtk_main_quit();
	}
    }

} // namespace {anonymous}


int
main(int argc, char * argv[]) {
    TRACE(10, "entered main(argc, argv)");

    GnomeProgram * gnome_program = gnome_program_init(PACKAGE, VERSION,
						      LIBGNOMEUI_MODULE,
						      argc, argv,
						      GNOME_PARAM_POPT_TABLE, OPTIONS,
						      GNOME_PROGRAM_STANDARD_PROPERTIES,
						      GNOME_PARAM_HUMAN_READABLE_NAME, "Dachshund",
						      0);

    if(show_version_flag) {
	show_version();
	return 0;
    }



    // Initialise the python interpreter
    Python::Interpreter::instance();


#if 0
    GValue context_as_value = { 0 };

    g_object_get_property(G_OBJECT(gnome_program),
			  GNOME_PARAM_POPT_CONTEXT,
			  g_value_init(&context_as_value, G_TYPE_POINTER));

    poptContext popt_context = static_cast<poptContext>(g_value_get_pointer(&context_as_value));

    poptFreeContext(popt_context);		// doesn't the destructor for GnomeProgram do it ?
#endif


    GtkWidget * app = gnome_app_new(PACKAGE, 0);

    GTK::Image::set_widget(app);		// a widget to create stock items against
    GTK::Text::set_widget(app);			// a widget to create text layouts against

    g_signal_connect(G_OBJECT(app), "delete_event", G_CALLBACK(&main_delete_event_callback), 0);
    g_signal_connect(G_OBJECT(app), "destroy", G_CALLBACK(&main_destroy_callback), 0);

    //

    Command * command_about = command_callback(&show_about);
    command_about->set_instructions("About Dachshund");

    Command * command_exit = command_callback(&do_exit);
    command_exit->set_instructions("Exit Dachshund");

    //

    GTK::Settings * _settings = new GTK::Settings();
    GTK::Settings::instance(_settings);

    //

    GTK::Factory * _factory = new GTK::Factory();
    GTK::Factory::instance(_factory);

    //

    GTK::Factory2 * _factory2 = new GTK::Factory2();
    GTK::Factory2::instance(_factory2);

    //

    GTK::Menu * _menubar = new GTK::Menu(true);
    GtkWidget * menubar = _menubar->get_base_widget();

    //

    GTK::Toolbar * _toolbar = new GTK::Toolbar();
    GtkWidget * toolbar = _toolbar->get_base_widget();

    //

    GTK::Statusbar * _statusbar = new GTK::Statusbar();
    GtkWidget * statusbar = _statusbar->get_base_widget();

    //

    GTK::Tree * _tree = new GTK::Tree();
    GtkWidget * tree = _tree->get_base_widget();
    gtk_widget_set_size_request(tree, 150, 200);

    //

    GTK::Document * _document = new GTK::Document();
    GtkWidget * document = _document->get_base_widget();
    gtk_widget_set_size_request (document, 400, 400);

    //

    GTK::Log * _log = new GTK::Log();
    GtkWidget * log = _log->get_base_widget();
    //gtk_widget_set_size_request(log, 180, 100);

    //

    KitWidgets widgets = {
	_menubar,
	_toolbar,
	_statusbar,
	_tree,
	_document,
	_log
    };

    KitCommands commands = {
	command_about,
	command_exit
    };



    TRACE(10, "Instantiating Kit");

    Kit::instantiate(widgets, commands);

    Kit * kit = Kit::instance();

    //

    GtkWidget * hpaned = gtk_hpaned_new();

    GtkWidget * vpaned = gtk_vpaned_new();

    GtkWidget * hbox = gtk_hbox_new(FALSE, 0);

    gtk_paned_pack1(GTK_PANED(vpaned), tree, TRUE, TRUE);
    gtk_widget_show(tree);

    gtk_paned_pack2(GTK_PANED(vpaned), log, TRUE, TRUE);
    gtk_widget_show(log);

    gtk_paned_pack1(GTK_PANED(hpaned), vpaned, FALSE, FALSE);
    gtk_widget_show(vpaned);

    GtkWidget * scrolled = gtk_scrolled_window_new(0, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
                                   GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);

    gtk_paned_pack2(GTK_PANED(hpaned), scrolled, TRUE, TRUE);

    gtk_container_add(GTK_CONTAINER(scrolled), document);

    gtk_widget_show(hbox);
    gtk_widget_show(hpaned);
    gtk_widget_show(scrolled);
    gtk_widget_show(document);

    //

    gnome_app_set_menus(GNOME_APP(app), GTK_MENU_BAR(menubar));
    gtk_widget_show(menubar);

    gnome_app_set_toolbar(GNOME_APP(app), GTK_TOOLBAR(toolbar));
    gtk_widget_show(toolbar);

    gnome_app_set_contents(GNOME_APP(app), hpaned);
    gtk_widget_show(hpaned);

    gnome_app_set_statusbar(GNOME_APP(app), statusbar);
    gtk_widget_show(statusbar);

    //

    gtk_widget_show(app);

    kit->log("Initialisation complete");

    if(!getenv("DACHSHUND_LOVER")) {
        gtk_idle_add(&show_disclaimer, app);
    }

    TRACE(10, "Entering gtk event loop");

    gtk_main();

    TRACE(10, "Left gtk event loop");


    GTK::Factory2::shutdown();
    GTK::Factory::shutdown();
    GTK::Settings::shutdown();

    Kit::shutdown();

    UndoManager::shutdown();

    g_object_unref(G_OBJECT(gnome_program));

    Object::show_remaining();

    TRACE(10, "Terminating");

    return EXIT_SUCCESS;
}
