// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "diff_stream.h"

#include "foundation/macros.h"

#include <cstdio>
#include <cerrno>
#include <exception>
#include <vector>
#include <iterator>
using namespace std;

#include <unistd.h>

namespace Common {

    const string DiffStream::SUFFIX = ".new";
    const ios::openmode DiffStream::MODE = ios::out | ios::trunc;

    DiffStream::DiffStream(const string & name) : ofstream((name + SUFFIX).c_str(), MODE) {
	mName = name;

	vector<char> buffer(128);

	while(!getcwd(&buffer.front(), buffer.size())) {
	    ASSERT(errno == ERANGE);
	    buffer.resize(buffer.size() * 2);
	}

	mPath = &buffer.front();
    }

    DiffStream::~DiffStream() {
	close();

	if(!uncaught_exception()) {
	    vector<char> buffer(128);

	    while(!getcwd(&buffer.front(), buffer.size())) {
		ASSERT(errno == ERANGE);
		buffer.resize(buffer.size() * 2);
	    }

	    ASSERT(!chdir(mPath.c_str()));

	    try {
		ifstream ifs1((mName + SUFFIX).c_str(), ios::in);
		ifstream ifs2(mName.c_str(), ios::in);

		ifs1.flags(ifs1.flags() & ~ios::skipws);
		ifs2.flags(ifs2.flags() & ~ios::skipws);

		ASSERT(ifs1);

		bool overwrite = true;
		string action = "Updating";

		if(ifs2) {
		    istream_iterator<char> a(ifs1);
		    istream_iterator<char> b(ifs2);
		    istream_iterator<char> c;

		    while(a != c && b != c && *a++ == *b++) {
		    }

		    if(a == c && b == c) {
			// the files are the same
			overwrite = false;
		    }
		}
		else {
		    action = "Creating";
		}

		if(overwrite) {
		    cout << action << ": " << mName << endl;
		    ASSERT(!rename((mName + SUFFIX).c_str(), mName.c_str()));
		}
		else {
		    ASSERT(!remove((mName + SUFFIX).c_str()));
		}

		chdir(&buffer.front());		// restore the previous working directory
	    }
	    catch(...) {
		chdir(&buffer.front());		// restore the previous working directory

		throw;
	    }
	}
    }

} // namespace Common
