// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef COMMON_TYPE_CODE_HANDLER_H
#define COMMON_TYPE_CODE_HANDLER_H

#include "xml/sax_handler.h"

#include "foundation/hash_functions.h"
#include "foundation/hash.h"

#include <set>
using namespace std;

namespace Common {

    using namespace XML;		// FIXME not sure if we want this

    class TypeCodeHandler : public XML::SAXHandler {
    public:
	TypeCodeHandler();
	~TypeCodeHandler();

	void warning(XML::SAXParser * parser, const string & msg);
	void error(XML::SAXParser * parser, const string & msg);
	void fatal(XML::SAXParser * parser, const string & msg);

	void comment(XML::SAXParser * parser, const string & comment);
	void start_document(XML::SAXParser * parser);
	void end_document(XML::SAXParser * parser);
	void start_element(XML::SAXParser * parser,
			   const string & name, const XML::Properties & properties);
	void end_element(XML::SAXParser * parser, const string & name);
	void characters(XML::SAXParser * parser, const string & data);
	void reference(XML::SAXParser * parser, const string & name);
	void cdata_block(XML::SAXParser * parser, const string & data);

    private:
	void xmi__corba_type_code__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_type_code__end(SAXParser * parser);

	void xmi__corba_tc_enum__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_enum__end(SAXParser * parser);

	void xmi__corba_tc_enum_label__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_enum_label__end(SAXParser * parser);

	void xmi__corba_tc_long__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_long__end(SAXParser * parser);

	void xmi__corba_tc_string__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_string__end(SAXParser * parser);

	void xmi__corba_tc_boolean__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_boolean__end(SAXParser * parser);

	void xmi__corba_tc_struct__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_struct__end(SAXParser * parser);

	void xmi__corba_tc_field__start(SAXParser * parser, const Properties & properties);
	void xmi__corba_tc_field__end(SAXParser * parser);

	typedef void (TypeCodeHandler::*StartHandler)(SAXParser *, const Properties &);
	typedef void (TypeCodeHandler::*EndHandler)(SAXParser *);
	typedef pair<StartHandler, EndHandler> HandlerPair;

	string mCharacters;

	hash_map<string, HandlerPair> mHandlerMap;

	set<string> mUnhandledElements;
    };

} // namespace Common

#endif // COMMON_TYPE_CODE_HANDLER_H
