// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "xmi_handler.h"

#include "xml/sax_parser.h"

#include "foundation/macros.h"

#include "dumb_handler.h"

namespace Common {

    XMIHandler::XMIHandler(SAXHandler * content_handler) : XML::SAXHandler(),
        mContentHandler(content_handler),
	mContent(0) {
	ASSERT(mContentHandler);
	mContentHandler->ref();

	// populate the map

	mHandlerMap["XMI"] =
	    make_pair(&XMIHandler::xmi__start,
		      &XMIHandler::xmi__end);

	mHandlerMap["XMI.header"] =
	    make_pair(&XMIHandler::xmi__header__start,
		      &XMIHandler::xmi__header__end);

	mHandlerMap["XMI.documentation"] =
	    make_pair(&XMIHandler::xmi__documentation__start,
		      &XMIHandler::xmi__documentation__end);

	mHandlerMap["XMI.owner"] =
	    make_pair(&XMIHandler::xmi__owner__start,
		      &XMIHandler::xmi__owner__end);

	mHandlerMap["XMI.contact"] =
	    make_pair(&XMIHandler::xmi__contact__start,
		      &XMIHandler::xmi__contact__end);

	mHandlerMap["XMI.longDescription"] =
	    make_pair(&XMIHandler::xmi__long_description__start,
		      &XMIHandler::xmi__long_description__end);

	mHandlerMap["XMI.shortDescription"] =
	    make_pair(&XMIHandler::xmi__short_description__start,
		      &XMIHandler::xmi__short_description__end);

	mHandlerMap["XMI.exporter"] =
	    make_pair(&XMIHandler::xmi__exporter__start,
		      &XMIHandler::xmi__exporter__end);

	mHandlerMap["XMI.exporterVersion"] =
	    make_pair(&XMIHandler::xmi__exporter_version__start,
		      &XMIHandler::xmi__exporter_version__end);

	mHandlerMap["XMI.exporterID"] =
	    make_pair(&XMIHandler::xmi__exporter_id__start,
		      &XMIHandler::xmi__exporter_id__end);

	mHandlerMap["XMI.notice"] =
	    make_pair(&XMIHandler::xmi__notice__start,
		      &XMIHandler::xmi__notice__end);

	mHandlerMap["XMI.model"] =
	    make_pair(&XMIHandler::xmi__model__start,
		      &XMIHandler::xmi__model__end);

	mHandlerMap["XMI.metamodel"] =
	    make_pair(&XMIHandler::xmi__metamodel__start,
		      &XMIHandler::xmi__metamodel__end);

	mHandlerMap["XMI.metametamodel"] =
	    make_pair(&XMIHandler::xmi__metametamodel__start,
		      &XMIHandler::xmi__metametamodel__end);

	mHandlerMap["XMI.import"] =
	    make_pair(&XMIHandler::xmi__import__start,
		      &XMIHandler::xmi__import__end);

	mHandlerMap["XMI.content"] =
	    make_pair(&XMIHandler::xmi__content__start,
		      &XMIHandler::xmi__content__end);

	mHandlerMap["XMI.extensions"] =
	    make_pair(&XMIHandler::xmi__extensions__start,
		      &XMIHandler::xmi__extensions__end);

	mHandlerMap["XMI.extension"] =
	    make_pair(&XMIHandler::xmi__extension__start,
		      &XMIHandler::xmi__extension__end);

	mHandlerMap["XMI.difference"] =
	    make_pair(&XMIHandler::xmi__difference__start,
		      &XMIHandler::xmi__difference__end);

	mHandlerMap["XMI.delete"] =
	    make_pair(&XMIHandler::xmi__delete__start,
		      &XMIHandler::xmi__delete__end);

	mHandlerMap["XMI.add"] =
	    make_pair(&XMIHandler::xmi__add__start,
		      &XMIHandler::xmi__add__end);

	mHandlerMap["XMI.replace"] =
	    make_pair(&XMIHandler::xmi__replace__start,
		      &XMIHandler::xmi__replace__end);

	mHandlerMap["XMI.reference"] =
	    make_pair(&XMIHandler::xmi__reference__start,
		      &XMIHandler::xmi__reference__end);

	mHandlerMap["XMI.any"] =
	    make_pair(&XMIHandler::xmi__any__start,
		      &XMIHandler::xmi__any__end);
    }

    XMIHandler::~XMIHandler() {
	if(mContent) {
	    mContent->unref();
	}

	mContentHandler->unref();
    }

    void
    XMIHandler::warning(SAXParser * parser, const string & msg) {
	WARNING("Unhandled warning: " << msg);
    }

    void
    XMIHandler::error(SAXParser * parser, const string & msg) {
	WARNING("Unhandled error: " << msg);
    }

    void
    XMIHandler::fatal(SAXParser * parser, const string & msg) {
	WARNING("Unhandled fatal: " << msg);
    }

    void
    XMIHandler::comment(SAXParser * parser, const string & comment) {
    }

    void
    XMIHandler::start_document(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	    if(mContent) {
		mContent->unref();
		mContent = 0;
	    }
	}
    }

    void
    XMIHandler::end_document(SAXParser * parser) {
    }

    void
    XMIHandler::start_element(SAXParser * parser, const string & name,
				const Properties & properties) {
	TRACE(5, "Start: " << name);

	mCharacters.resize(0);

	hash_map<string, HandlerPair>::iterator iter = mHandlerMap.find(name);

	if(iter == mHandlerMap.end()) {
	    if(mUnhandledElements.insert(name).second) {
		WARNING("Unhandled element: " << name);
	    }

	    // use a dumb handler so that we don't try to parse
	    // the contents of the tag

	    XML::SAXHandler * handler = new Common::DumbHandler();
	    parser->use(handler);
	    handler->unref();
	}
	else {
	    HandlerPair handler_pair = iter->second;
	    (this->*handler_pair.first)(parser, properties);
	}
    }

    void
    XMIHandler::end_element(SAXParser * parser, const string & name) {
	hash_map<string, HandlerPair>::iterator iter = mHandlerMap.find(name);

	if(iter == mHandlerMap.end()) {
	}
	else {
	    HandlerPair handler_pair = iter->second;
	    (this->*handler_pair.second)(parser);
	}

	TRACE(5, "End: " << name);
    }

    void
    XMIHandler::characters(SAXParser * parser, const string & data) {
	mCharacters += data;
    }

    void
    XMIHandler::reference(SAXParser * parser, const string & name) {
	WARNING("Unhandled Reference: " << name);
    }

    void
    XMIHandler::cdata_block(SAXParser * parser, const string & data) {
	WARNING("Unhandled CDATA: " << data);
    }

    //
    //
    //


    void
    XMIHandler::xmi__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	    if(properties.get_required_property("xmi.version") != "1.1") {
		ERROR("Wrong version of XMI");
		throw;	// FIXME
	    }
	}
	else {
	}
    }

    void
    XMIHandler::xmi__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__header__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__header__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__documentation__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__documentation__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__owner__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__owner__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__contact__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__contact__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__long_description__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__long_description__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__short_description__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__short_description__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter_version__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter_version__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter_id__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__exporter_id__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__notice__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__notice__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__model__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__model__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__metamodel__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__metamodel__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__metametamodel__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__metametamodel__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__import__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__import__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__content__start(SAXParser * parser, const Properties & properties) {
	parser->use(mContentHandler);

	if(parser->get_pass() == 1) {
	}
	else {
	    //ASSERT(parser->mStack2.empty());
	    parser->push();
	}
    }

    void
    XMIHandler::xmi__content__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	    //ASSERT(parser->mStack2.size() == 1);
	    ASSERT(parser->top().size() == 1);

	    mContent = parser->top().front();
	    ASSERT(mContent);

	    // we don't unref the content because we are going to hang on to it

	    parser->pop();
	}
    }

    void
    XMIHandler::xmi__extensions__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__extensions__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__extension__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	    const string & extender = properties.get_required_property("extender");

	    if(extender == "dachshund") {
		XML::SAXHandler * handler = new Common::DumbHandler();
		parser->use(handler);
		handler->unref();
	    }
	    else {
		// unknown extender
		XML::SAXHandler * handler = new Common::DumbHandler();
		parser->use(handler);
		handler->unref();
	    }
	}
	else {
	    const string & extender = properties.get_required_property("extender");

	    if(extender == "dachshund") {
		XML::SAXHandler * handler = new Common::DumbHandler();
		parser->use(handler);
		handler->unref();
	    }
	    else {
		// unknown extender
		XML::SAXHandler * handler = new Common::DumbHandler();
		parser->use(handler);
		handler->unref();
	    }
	}
    }

    void
    XMIHandler::xmi__extension__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__difference__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__difference__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__delete__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__delete__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__add__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__add__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__replace__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__replace__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__reference__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__reference__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__any__start(SAXParser * parser, const Properties & properties) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

    void
    XMIHandler::xmi__any__end(SAXParser * parser) {
	if(parser->get_pass() == 1) {
	}
	else {
	}
    }

} // namespace Common
