// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_ellipse.h"

#include "selection_resize.h"

#include "drawing/drawable.h"
#include "drawing/rectangle.h"

namespace Diagram {

    FigEllipse::FigEllipse() : Fig() {
    }

    FigEllipse::~FigEllipse() {
    }

    void
    FigEllipse::paint(Drawing::Drawable * drawable) const {
	drawable->set_line_style(get_line_style());

	if(is_filled()) {
	    drawable->set_color(get_fill_color());
	    drawable->draw_oval(mBounds, true);
	}
	else {
	    drawable->set_color(get_line_color());
	    drawable->draw_oval(mBounds, false);
	}
    }

    Selection *
    FigEllipse::create_selection() {
	return new SelectionResize(this);
    }

    int
    FigEllipse::count_corners_contained(const Drawing::Rectangle & rect) const {
	int counter = 0;

	//
	// equation for circle with center (0,0):
	//    x^2 + y^2 = r^2
	//
	// equation for circle with center (x0,y0):
	//    (x - x0)^2 + (y - y0)^2 = r^2
	//
	// equation for ellipse with center (x0,y0) and dimensions (w,y):
	//    ( (x - x0) / (w/2) )^2 + ( (y - y0) / (h/2) )^2 = 1
	//
	// (x,y) is inside the ellipse if:
	//    ( (x - x0) / (w/2) )^2 + ( (y - y0) / (h/2) )^2 - 1 < 0
	//
	// (x,y) is outside the ellipse if:
	//    ( (x - x0) / (w/2) )^2 + ( (y - y0) / (h/2) )^2 - 1 > 0

	double x0 = get_bounds().get_cx();
	double y0 = get_bounds().get_cy();
	double w = get_bounds().get_w();
	double h = get_bounds().get_h();

	const Drawing::Point points[4] = {
	    Drawing::Point(rect.get_x(), rect.get_y()),
	    Drawing::Point(rect.get_xw(), rect.get_y()),
	    Drawing::Point(rect.get_xw(), rect.get_yh()),
	    Drawing::Point(rect.get_x(), rect.get_yh()),
	};

	for(int i = 0; i < 4; ++i) {
	    const Drawing::Point & point = points[i];

	    double a = (point.get_x() - x0) / (w / 2.0);
	    double b = (point.get_y() - y0) / (h / 2.0);

	    if(a * a + b * b - 1.0 < 0.0) {
		++counter;
	    }
	}

	return counter;
    }

} // namespace Diagram
