// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_FIG_POLY_H
#define DIAGRAM_FIG_POLY_H

#include "fig.h"

#include <vector>
using namespace std;

namespace Diagram {

    class Handle;

    class FigPoly : public Fig {
    public:
	FigPoly(Object * owner = 0);
	~FigPoly();

	//
	//
	//

	void relocate(const Drawing::Point & locate);
	void resize(const Drawing::Vector & size);

	virtual void set_points(const vector<Drawing::Point> & points);
	virtual void add_point(const Drawing::Point & point);
	virtual void set_point(Handle & handle, const Drawing::Point & point);
	virtual void remove_point(int index);
	virtual void insert_point(int index, const Drawing::Point & point);

	void cleanup();

	Selection * create_selection();

	//
	//
	//

	const vector<Drawing::Point> & get_points() const { return mPoints; }

	void set_end_points(const Drawing::Point & point_start, const Drawing::Point & point_end);

	bool hit(const Drawing::Rectangle & rect) const;

    protected:
	void append_two_points();
	void prepend_two_points();

	virtual void update_bounds_to_points();

	vector<Drawing::Point> mPoints;

    private:
	static const double MINIMUM_ANGLE = 0.11;
    };

} // namespace Diagram

#endif // DIAGRAM_FIG_POLY_H
