// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "layer.h"

#include "fig.h"
#include "editor.h"

#include "drawing/rectangle.h"
#include "drawing/drawable.h"

#include "foundation/functional.h"
#include "foundation/macros.h"

#include <algorithm>
#include <functional>
using namespace std;

namespace Diagram {

    Layer::Layer() : Object(),
	mHidden(false) {
    }

    Layer::~Layer() {
    }

    void
    Layer::set_hidden(bool hidden) {
	if(mHidden != hidden) {
	    mHidden = hidden;
	}
    }

    void
    Layer::set_layer_name(const string & layer_name) {
	mLayerName = layer_name;
    }

    void
    Layer::send_to_back(Fig * fig) {
    }

    void
    Layer::send_backward(Fig * fig) {
    }

    void
    Layer::send_behind(Fig * f1, Fig * f2) {
    }

    void
    Layer::bring_to_front(Fig * fig) {
    }

    void
    Layer::bring_forward(Fig * fig) {
    }

    void
    Layer::bring_in_front_of(Fig * f1, Fig * f2) {
    }

    Fig *
    Layer::hit(const Drawing::Rectangle & rect) {
	return 0;
    }

    Fig *
    Layer::hit_not(const Drawing::Rectangle & r, Fig * disregard) {
	return 0;
    }

    void
    Layer::paint(Drawing::Drawable * drawable) const {
    }

    void
    Layer::print(Drawing::Drawable * drawable) {
	paint(drawable);
    }

    void
    Layer::damage(Fig * fig) {
	for_each(mEditors.begin(), mEditors.end(),
		 bind2nd(mem_fun((void (Editor::*)(const Fig *))&Editor::damage), fig));
    }

    void
    Layer::damage(const Drawing::Rectangle & rect) {
	for_each(mEditors.begin(), mEditors.end(),
	bind2nd_vr(mem_fun((void (Editor::*)(const Drawing::Rectangle &))&Editor::damage), rect));
    }

    Fig *
    Layer::get_presentation_for(Object * object) {
	return 0;
    }

    void
    Layer::add(Fig * fig) {
    }

    void
    Layer::remove(Fig * fig) {
    }

    void
    Layer::add_editor(Editor * editor) {
	mEditors.push_front(editor);
    }

    void
    Layer::remove_editor(Editor * editor) {
	list<Editor *>::iterator iter = find(mEditors.begin(), mEditors.end(), editor);

	if(iter == mEditors.end()) {
	    ERROR("Editor not found");
	}
	else {
	    mEditors.erase(iter);
	}
    }

    void
    Layer::clear() {
    }

    const list<Fig *> &
    Layer::get_figs() {
	static const list<Fig *> dummy;
	return dummy;
    }

} // namespace Diagram
