// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_LAYER_H
#define DIAGRAM_LAYER_H

#include "foundation/object.h"

#include <list>
#include <string>
using namespace std;

namespace Drawing {

    class Rectangle;
    class Drawable;

} // namespace Drawing

namespace Diagram {

    class Editor;
    class Fig;

    class Layer : public Object {
    public:
	void set_hidden(bool hidden);
	bool is_hidden() const { return mHidden; }

	void set_layer_name(const string & layer_name);
	const string & get_layer_name() const { return mLayerName; }

	virtual Fig * get_presentation_for(Object * object);

	virtual void add(Fig * fig);
	virtual void remove(Fig * fig);
	virtual void clear();
	virtual Fig * hit(const Drawing::Rectangle & rect);
	virtual Fig * hit_not(const Drawing::Rectangle & rect, Fig * disregard);

	virtual const list<Fig *> & get_figs();

	void add_editor(Editor * editor);
	void remove_editor(Editor * editor);

	virtual void send_to_back(Fig * fig);
	virtual void send_backward(Fig * fig);
	virtual void send_behind(Fig * f1, Fig * f2); // f1 goes behind f2
	virtual void bring_forward(Fig * fig);
	virtual void bring_to_front(Fig * fig);
	virtual void bring_in_front_of(Fig * f1, Fig * f2); // f1 goes in front of f2

	virtual void paint(Drawing::Drawable * drawable) const;
	virtual void print(Drawing::Drawable * drawable);

	void damage(Fig * fig);
	void damage(const Drawing::Rectangle & rect);

    protected:
	Layer();
	~Layer();

    private:
	bool mHidden;
	string mLayerName;
	list<Editor *> mEditors;
    };

} // namespace Diagram

#endif // DIAGRAM_LAYER_H
