// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "layer_grid.h"

#include "drawing/drawable.h"
#include "drawing/rectangle.h"
#include "drawing/point.h"
#include "drawing/vector.h"

namespace Diagram {

    LayerGrid::LayerGrid() : LayerGuide(),
	mPaintLines(true),
	mSpacing(DEFAULT_SPACING) {
    }

    LayerGrid::LayerGrid(int spacing) : LayerGuide(),
	mPaintLines(true),
	mSpacing(spacing) {
    }

    LayerGrid::~LayerGrid() {
    }

    void
    LayerGrid::snap(Drawing::Point & point) {
	point.set_x((point.get_x() + mSpacing / 2) / mSpacing * mSpacing);
	point.set_y((point.get_y() + mSpacing / 2) / mSpacing * mSpacing);
    }

    void
    LayerGrid::paint(Drawing::Drawable * drawable) const {
	const Drawing::Rectangle & clip = drawable->get_clip();

	drawable->set_line_style(Drawing::LS_SOLID);

	if(mPaintLines) {
	    drawable->set_color("#dddddd");

	    if(clip) {
		for(int i = clip.get_y() - (clip.get_y() % mSpacing);
		    i <= clip.get_yh();
		    i += mSpacing) {
		    drawable->draw_line(Drawing::Point(clip.get_x(), i),
					Drawing::Point(clip.get_xw(), i));
		}

		for(int j = clip.get_x() - (clip.get_x() % mSpacing);
		    j <= clip.get_xw();
		    j += mSpacing) {
		    drawable->draw_line(Drawing::Point(j, clip.get_y()),
					Drawing::Point(j, clip.get_yh()));
		}
	    }
	    else {
		const Drawing::Vector & size = drawable->get_size();

		for(int i = 0; i < size.get_y(); i += mSpacing) {
		    drawable->draw_line(Drawing::Point(0, i),
					Drawing::Point(size.get_x() - 1, i));
		}

		for(int j = 0; j < size.get_x(); j += mSpacing) {
		    drawable->draw_line(Drawing::Point(j, 0),
					Drawing::Point(j, size.get_y() - 1));
		}
	    }
	}
	else {
	    drawable->set_color("#333333");

	    if(clip) {
		for(int i = clip.get_y() - (clip.get_y() % mSpacing);
		    i <= clip.get_yh();
		    i += mSpacing) {
		    for(int j = clip.get_x() - (clip.get_x() % mSpacing);
			j <= clip.get_xw();
			j += mSpacing) {
			drawable->draw_point(Drawing::Point(j, i));
		    }
		}
	    }
	    else {
		const Drawing::Vector & size = drawable->get_size();

		for(int i = 0; i < size.get_y(); i += mSpacing) {
		    for(int j = 0; j < size.get_x(); j += mSpacing) {
			drawable->draw_point(Drawing::Point(j, i));
		    }
		}
	    }
	}
    }

} // namespace Diagram
