// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "mode_create.h"

#include "toolkit/mouse_event.h"

#include "fig.h"
#include "editor.h"
#include "selection_manager.h"

#include "foundation/command.h"
#include "foundation/macros.h"

//
//
//
// FIXME the ModeCreate classes and subclasses need refactoring
//
//

namespace Diagram {

    Drawing::Vector ModeCreate::mDefaultSize(65, 65);	// FIXME how to correlate with the grid

    ModeCreate::ModeCreate() : Mode(),
	mNewItem(0) {
	mCursor = Toolkit::C_HAND;
    }

    ModeCreate::~ModeCreate() {
    }

    bool
    ModeCreate::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	start(mouse_event);

	Editor * editor = Editor::current();

	ASSERT(editor);

	mAnchor = mouse_event->get_point();

	editor->snap(mAnchor);

	mNewItem = create_new_item(mouse_event, mAnchor);
	update_new_item(mouse_event);

	if(!mNewItem) {
	    finish(mouse_event);
	}

	return true;
    }

    bool
    ModeCreate::mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
	// resize the item as the user drags the mouse

	if(mNewItem) {
	    Editor * editor = Editor::current();

	    ASSERT(editor);

	    update_new_item(mouse_event);

#if 0
	    editor->scroll_to_show(mouse_event->get_point());
#endif
	}

	return true;
    }

    bool
    ModeCreate::mouse_released(const Toolkit::MouseEvent * mouse_event) {
	if(mNewItem) {
	    Editor * editor = Editor::current();

	    ASSERT(editor);

	    update_new_item(mouse_event);

	    // FIXME the problem is that if the item is a node or edge then
	    // it is already added by the perspective layer...
	    editor->add(mNewItem);

	    SelectionManager * selection_manager = SelectionManager::instance();

	    ASSERT(selection_manager);

	    selection_manager->damage();
	    selection_manager->set(mNewItem);
	    selection_manager->damage();

	    mDefaultSize.set(mNewItem->get_bounds().get_w(), mNewItem->get_bounds().get_h());

	    mNewItem = 0;
	}

	finish(mouse_event);

	return true;
    }

    void
    ModeCreate::paint(Drawing::Drawable * drawable) const {
	if(mNewItem) {
	    mNewItem->paint(drawable);
	}
    }

    void
    ModeCreate::cancel() {
	Mode::cancel();

	if(mNewItem) {
	    mNewItem->unref();
	    mNewItem = 0;
	}
    }

    void
    ModeCreate::update_new_item(const Toolkit::MouseEvent * mouse_event) {
	if(mNewItem) {
	    Editor * editor = Editor::current();

	    ASSERT(editor);

	    damage();

	    Drawing::Point snapped_point = mouse_event->get_point();

	    editor->snap(snapped_point);

	    if(mAnchor == snapped_point) {
		mNewItem->create_drag(mAnchor,
				      mouse_event->get_point() + mDefaultSize -Drawing::Vector(1,1),
				      snapped_point + mDefaultSize - Drawing::Vector(1, 1));
	    }
	    else {
		mNewItem->create_drag(mAnchor, mouse_event->get_point(), snapped_point);
	    }

	    mBounds = mNewItem->get_bounds();

	    damage();
	}
    }

} // namespace Diagram
