// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "mode_create_fig_node.h"

#include "editor.h"

#include "foundation/macros.h"
#include "foundation/utility.h"
#include "foundation/undo_manager.h"

#include "toolkit/mouse_event.h"
#include "toolkit/factory.h"
#include "toolkit/message_box.h"

#include "graph/model.h"
#include "graph/exception.h"

namespace Diagram {

    ModeCreateFigNode::ModeCreateFigNode() : ModeCreate() {
    }

    ModeCreateFigNode::~ModeCreateFigNode() {
    }

    Fig *
    ModeCreateFigNode::create_new_item(const Toolkit::MouseEvent * mouse_event,
				       const Drawing::Point & point) {
	// FIXME it is obvious that ModeCreateFigNode should not be a subclass
	// of ModeCreate... refactor time...

	return 0;
    }

    bool
    ModeCreateFigNode::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	start(mouse_event);

	//

	begin_transaction();

	Editor * editor = Editor::current();

	ASSERT(editor);

	//

	Drawing::Point snapped_point = mouse_event->get_point();

	editor->snap(snapped_point);

	Object * node = create_new_node(snapped_point);

	// add the new node to the graph model

	Graph::Model * graph_model = editor->get_graph_model();

	ASSERT(graph_model);

	try {
	    graph_model->add_node(node);	// FIXME put the parent parameter
	}
	catch(Graph::Exception & ex) {
	    end_transaction();

	    UndoManager * undo_manager = UndoManager::instance();

	    undo_manager->undo();
	    undo_manager->clear_redo();

	    node->unref();

            Toolkit::Factory * factory = Toolkit::Factory::instance();

            Toolkit::MessageBox * message_box =
                factory->create_message_box(ex.get_details(), Toolkit::MBK_ERROR,
                                            mouse_event->get_widget());

            message_box->go();
            message_box->unref();

            ModeCreate::finish(mouse_event);

            return true;
	}

	node->unref();

	end_transaction();

	return true;
    }

    bool
    ModeCreateFigNode::mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    ModeCreateFigNode::mouse_released(const Toolkit::MouseEvent * mouse_event) {
        finish(mouse_event);

	return true;
    }

    void
    ModeCreateFigNode::finish(const Toolkit::MouseEvent * mouse_event) {
	ModeCreate::finish(mouse_event);
    }

} // namespace Diagram
