// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "mode_popup.h"

#include "foundation/macros.h"
#include "foundation/command_callback.h"
#include "toolkit/factory.h"
#include "toolkit/menu.h"
#include "toolkit/mouse_event.h"

#include "editor.h"
#include "selection_manager.h"
#include "fig.h"
#include "mode_modify.h"
#include "selection.h"
#include "layer.h"

namespace Diagram {

    ModePopup::ModePopup() : Mode() {
    }

    ModePopup::~ModePopup() {
    }

    string
    ModePopup::get_instructions() const {
	return "Select an action from the menu";
    }

    Toolkit::Menu *
    ModePopup::create_zoom_menu() {
	Toolkit::Menu * menu = Toolkit::Factory::instance()->create_menu();

	menu->add_push(0, "25%");
	menu->add_push(0, "50%");
	menu->add_push(0, "100%");
	menu->add_push(0, "200%");
	menu->add_push(0, "400%");

	return menu;
    }

    Toolkit::Menu *
    ModePopup::create_ordering_menu() {
	SelectionManager * selection_manager = SelectionManager::instance();
	ASSERT(selection_manager->size() == 1);

	Selection * selection = selection_manager->get_selections().front();
	ASSERT(selection);

	Fig * fig = selection->get_content();
	ASSERT(fig);

	Layer * layer = fig->get_layer();
	ASSERT(layer);

	Toolkit::Menu * menu = Toolkit::Factory::instance()->create_menu();

	menu->add_push(command_callback(layer, fig, &Layer::send_to_back), "Send to Back");
	menu->add_push(command_callback(layer, fig, &Layer::send_backward), "Send Backward");
	menu->add_push(command_callback(layer, fig, &Layer::bring_forward), "Bring Forward");
	menu->add_push(command_callback(layer, fig, &Layer::bring_to_front), "Bring to Front");

	return menu;
    }

    bool
    ModePopup::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	if(mouse_event->get_button() == 3) {
	    start(mouse_event);

	    SelectionManager * selection_manager = SelectionManager::instance();

	    Toolkit::Menu * menu = Toolkit::Factory::instance()->create_menu();

	    menu->add_submenu(create_zoom_menu(), "Zoom");

	    if(selection_manager->empty()) {
	    }
	    else {
		//menu->add_separator();

		//menu->add_push(0, "Cut");
		//menu->add_push(0, "Copy");
		//menu->add_push(0, "Paste");

		if(selection_manager->size() == 1) {
		    menu->add_separator();
		    menu->add_submenu(create_ordering_menu(), "Ordering");
		}
		else {
		}
	    }

	    menu->popup(mouse_event);

	    // FIXME maybe need to enter event loop

	    //menu->unref();

	    finish(mouse_event);

	    return true;
	}
	else {
	    return false;
	}
    }

} // namespace Diagram
