// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "palette_manager.h"

#include "toolkit/palette.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include "mode.h"
#include "editor.h"
#include "mode_change_event.h"
#include "command_set_mode.h"

namespace Diagram {

    namespace {

	bool
	is_mode(Command * command, void * data) {
	    Mode * mode = static_cast<Mode *>(data);
	    CommandSetMode * command_set_mode = known_cast<CommandSetMode *>(command);

	    return command_set_mode->get_mode() == mode;
	}

    } // namespace {anonymous}

    PaletteManager::PaletteManager(Editor * editor, Toolkit::Palette * palette) :
    ModeChangeListener(), mEditor(editor), mPalette(palette) {
	mEditor->add(static_cast<ModeChangeListener *>(this));
    }

    PaletteManager::~PaletteManager() {
	if(mEditor) {
	    mEditor->remove(static_cast<ModeChangeListener *>(this));
	}
    }

    void
    PaletteManager::mode_changed(const ModeChangeEvent * mode_change_event) {
	Command * command = mPalette->find_if(&is_mode, mode_change_event->get_mode());

	if(command) {
	    mPalette->set_current(command);
	}
    }

} // namespace Diagram
