// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "selection.h"

#include "drawing/point.h"
#include "drawing/drawable.h"
#include "toolkit/mouse_event.h"
#include "foundation/macros.h"

#include "fig.h"
#include "editor.h"
#include "handle.h"

namespace Diagram {

    const string Selection::DEFAULT_LINE_COLOR = "#000055";
    const string Selection::DEFAULT_FILL_COLOR = "#0000ff";
    const int Selection::HAND_SIZE = 7;
    const int Selection::BORDER_WIDTH = 4;

    Selection::Selection(Fig * fig) :
	mContent(fig),
        mLineColor(DEFAULT_LINE_COLOR),
	mFillColor(DEFAULT_FILL_COLOR) {
	ASSERT(mContent);
    }

    Selection::~Selection() {
    }

    //

    bool
    Selection::mouse_moved(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Selection::mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Selection::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Selection::mouse_released(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Selection::mouse_exited(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Selection::mouse_entered(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    //

    bool
    Selection::key_pressed(const Toolkit::KeyEvent * key_event) {
	return false;
    }

    bool
    Selection::key_released(const Toolkit::KeyEvent * key_event) {
	return false;
    }

    //

    bool
    Selection::contains(const Drawing::Point & point) const {
	return mContent->contains(point);
    }

    void
    Selection::set_content(Fig * content) {
	mContent = content;
    }

    void
    Selection::damage() {
	Editor * editor = Editor::current();

	if(editor) {
	    editor->damage(this);
	}
    }

    bool
    Selection::hit(const Drawing::Rectangle & rect) const {
	if(mContent->hit(rect)) {
	    return true;
	}
	else {
	    Handle handle;

	    hit_handle(rect, handle);

	    return bool(handle);
	}
    }

    bool
    Selection::intersects(const Drawing::Rectangle & rect) const {
	return mBounds & rect;
    }

    void
    Selection::translate(const Drawing::Vector & offset) {
	mContent->translate(offset);
	update_bounds_to_content();
    }

    void
    Selection::cleanup() {
	if(mContent) {
	    mContent->cleanup();
	}
    }

    void
    Selection::paint_rectangle(const Drawing::Point & point, Drawing::Drawable * drawable) const {
	drawable->set_color(mFillColor);
	drawable->draw_rectangle(Drawing::Rectangle(point - Drawing::Vector(HAND_SIZE / 2,
									    HAND_SIZE / 2),
					   Drawing::Vector(HAND_SIZE, HAND_SIZE)), true);

	drawable->set_color(mLineColor);
	drawable->draw_rectangle(Drawing::Rectangle(point - Drawing::Vector(HAND_SIZE / 2,
									    HAND_SIZE / 2),
					   Drawing::Vector(HAND_SIZE, HAND_SIZE)), false);
    }

    void
    Selection::begin_interaction() {
	mContent->begin_interaction();
    }

    void
    Selection::end_interaction() {
	mContent->end_interaction();
    }

    void
    Selection::update_bounds_to_content() {
        mBounds = mContent->get_bounds();
        mBounds.translate(Drawing::Vector(-BORDER_WIDTH, -BORDER_WIDTH));
        mBounds.grow(2 * BORDER_WIDTH, 2 * BORDER_WIDTH);
    }

} // namespace Diagram
