// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "selection_edge.h"

#include "fig_node.h"
#include "layer_perspective.h"
#include "handle.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

namespace Diagram {

    SelectionEdge::SelectionEdge(FigEdge * fig_edge) : SelectionPoly(fig_edge),
        mCurrentNode(0),
	mPart(FigEdge::EDGE) {
    }

    SelectionEdge::~SelectionEdge() {
    }

    void
    SelectionEdge::drag_handle(const Drawing::Point & m, const Drawing::Point & an,
			       Handle & handle) {
	SelectionPoly::drag_handle(m, an, handle);

	FigEdge * fig_edge = known_cast<FigEdge *>(mContent);
	LayerPerspective * layer = known_cast<LayerPerspective *>(fig_edge->get_layer());

	if(handle.get_index() == 0) {
	    mPart = FigEdge::SOURCE;
	}
	else if(handle.get_index() == fig_edge->get_points().size() - 1) {
	    mPart = FigEdge::DESTINATION;
	}
	else {
	    mPart = FigEdge::EDGE;
	}

	if(mPart != FigEdge::EDGE) {
	    Fig * current = layer->hit_not(Drawing::Rectangle(m, Drawing::Vector(1, 1)), fig_edge);

	    FigNode * current_node = 0;

	    if(current) {
		current_node = dynamic_cast<FigNode *>(current);
	    }

	    if(mCurrentNode != current_node) {
		if(mCurrentNode) {
		    mCurrentNode->set_highlight(false);
		}

		mCurrentNode = current_node;

		if(mCurrentNode) {
		    mCurrentNode->set_highlight(true);
		}
	    }
	}
    }

    void
    SelectionEdge::cleanup() {
	damage();

	FigEdge * fig_edge = known_cast<FigEdge *>(mContent);

	if(mCurrentNode) {
	    mCurrentNode->set_highlight(false);

	    //LayerPerspective * layer = known_cast<LayerPerspective *>(fig_edge->get_layer());

	    // TODO update the graph model

	    fig_edge->damage();

	    bool changed = false;

	    ASSERT(mPart != FigEdge::EDGE);	// according to the logic in drag_handle this
	    					// is not possible

	    if(mPart == FigEdge::SOURCE && fig_edge->get_source_fig() != mCurrentNode) {
		// don't cause a transaction unless the source fig is actually different
		if(fig_edge->get_destination_fig() != mCurrentNode ||
		   fig_edge->get_points().size() > 3) {
		    // only let them make a loop if there are more than 3 points
		    fig_edge->set_source_fig(mCurrentNode);
		    changed = true;
		}
	    }
	    else if(mPart == FigEdge::DESTINATION &&
		// don't cause a transaction unless the source fig is actually different
		    fig_edge->get_destination_fig() != mCurrentNode) {
		if(fig_edge->get_source_fig() != mCurrentNode ||
		   fig_edge->get_points().size() > 3) {
		    // only let them make a loop if there are more than 3 points
		    fig_edge->set_destination_fig(mCurrentNode);
		    changed = true;
		}
	    }

	    // FIXME although we prevent the user from making a loop with three points
	    // or less, we don't stop the user from "removing" these points by
	    // forming straight lines with the edge (see FigPoly::cleanup). This
	    // Could be fixed by adding a virtual method called can_remove_point()
	    // to FigPoly. FigEdge would override it and disallow points to
	    // be removed if it was a loop and had four or less points.

	    if(!changed) {
		// We might get here if an end of an edge has been dragged from a
		// node but then it is dropped over the same node.
		// We just clean it up so that it is neat again.
		//
		// We might be here because the user tried to make a loop but there
		// weren't enough points in the edge
		fig_edge->cleanup();
	    }

	    mCurrentNode = NULL;
	    mPart = FigEdge::EDGE;
	}
	else {
	    fig_edge->damage();
	    fig_edge->cleanup();
	    fig_edge->damage();
	}

	update_bounds_to_content();

	damage();
    }

} // namespace Diagram
