// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_SELECTION_MANAGER_H
#define DIAGRAM_SELECTION_MANAGER_H

#include "foundation/object.h"
#include "toolkit/mouse_listener.h"
#include "toolkit/key_listener.h"

#include <list>
#include <vector>
using namespace std;

namespace Drawing {

class Rectangle;
class Drawable;
class Vector;
class Point;

} // namespace Drawing

namespace Toolkit {

    class MouseEvent;

} // namespace Toolkit

namespace Diagram {

    class Selection;
    class Fig;
    class Handle;

    class SelectionManager : public Object,
			     public Toolkit::MouseListener, public Toolkit::KeyListener {
    public:
	static SelectionManager * instance();
	static void shutdown();

	void add(Fig * fig);
	void remove(Fig * fig);
	void set(Fig * fig);
	void toggle(Fig * fig);
	bool contains(Fig * fig) const;
	void clear();
	bool empty() const { return mSelections.empty(); }
	int size() const { return mSelections.size(); }

	const list<Selection *> & get_selections() { return mSelections; }
	void set_selections(const list<Selection *> selections);

	void get_bounds(Drawing::Rectangle & rect) const;
	void get_contents_bounds(Drawing::Rectangle & rect) const;

	void translate(const Drawing::Vector & offset);
	void drag_handle(const Drawing::Point & m, const Drawing::Point & an, Handle & handle);

	static Selection * make_selection_for(Fig * fig);

	void damage();

	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);

	bool key_pressed(const Toolkit::KeyEvent * key_event);
	bool key_released(const Toolkit::KeyEvent * key_event);

	Selection * find_selection_at(const Drawing::Point & point);
	Selection * find_selection_for(Fig * fig);

	bool hit(const Drawing::Rectangle & rect) const;
	void hit_handle(const Drawing::Rectangle & rect, Handle & handle) const;

	void paint(Drawing::Drawable * drawable) const;

	void cleanup();

	void begin_interaction();
	void end_interaction();

    protected:

    private:
	SelectionManager();
	~SelectionManager();

	static SelectionManager * mInstance;

	list<Selection *> mSelections;

	bool mCacheValid;		// cache of affected edges
	vector<Fig *> mAffectedEdges;

	bool mInInteraction;
    };

} // namespace Diagram

#endif // DIAGRAM_SELECTION_MANAGER_H
