// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "selection_poly.h"

#include "foundation/functional.h"
#include "foundation/utility.h"

#include "drawing/point.h"
#include "drawing/rectangle.h"
#include "drawing/segment.h"
#include "drawing/vector.h"
#include "drawing/drawable.h"

#include "fig_poly.h"
#include "handle.h"

#include <functional>
#include <algorithm>
using namespace std;

namespace Diagram {

    SelectionPoly::SelectionPoly(FigPoly * fig_poly) : Selection(fig_poly),
	mHandleIndex(-1) {

	update_bounds_to_content();
    }

    SelectionPoly::~SelectionPoly() {
    }

    void
    SelectionPoly::drag_handle(const Drawing::Point & m,
			       const Drawing::Point & an, Handle & handle) {
	FigPoly * fig_poly = known_cast<FigPoly *>(mContent);

	fig_poly->set_point(handle, m);

	update_bounds_to_content();
    }

    void
    SelectionPoly::hit_handle(const Drawing::Rectangle & rect, Handle & handle) const {
	FigPoly * fig_poly = known_cast<FigPoly *>(mContent);

	const vector<Drawing::Point> & points = fig_poly->get_points();

	vector<Drawing::Point>::const_iterator iter =
	    find_if(points.begin(), points.end(),
		    bind1st_rr(mem_fun_ref(&Drawing::Rectangle::contains_point), rect));

	if(iter == points.end()) {
	    handle.set_index(-1);

	    FigPoly * fig_poly = dynamic_cast<FigPoly *>(mContent);

	    if(fig_poly) {
		for(unsigned int i = 0; i < fig_poly->get_points().size() - 1; ++i) {
		    const Drawing::Point & p1 = fig_poly->get_points()[i];
		    const Drawing::Point & p2 = fig_poly->get_points()[i + 1];

		    Drawing::Point new_point;

		    if(rect.get_intercept(Drawing::Segment(p1, p2), new_point)) {
			handle.set_index(i + 1);
			handle.set_exists(false);
			handle.set_instructions("Create new point");
		    }
		}
	    }
	}
	else {
	    handle.set_index(iter - points.begin());
	    handle.set_instructions("Move point");
	}
    }

    void
    SelectionPoly::paint(Drawing::Drawable * drawable) const {
	FigPoly * fig_poly = known_cast<FigPoly *>(mContent);

	const vector<Drawing::Point> & points = fig_poly->get_points();

	for(vector<Drawing::Point>::const_iterator iter = points.begin();
	    iter != points.end();
	    ++iter) {
	    const Drawing::Point & point = *iter;
	    paint_rectangle(point, drawable);
	}
    }

    void
    SelectionPoly::update_bounds_to_content() {
	mBounds = mContent->get_bounds();
	mBounds.translate(Drawing::Vector(-HAND_SIZE / 2, -HAND_SIZE / 2));
	mBounds.grow(HAND_SIZE + 2, HAND_SIZE + 2);
    }

} // namespace Diagram
