// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "drawing_support.h"

#include "drawable.h"
#include "point.h"

#include "foundation/macros.h"

#include <cmath>
using namespace std;

namespace Drawing {

    void
    draw_arrow(Drawable * drawable, const Point & p1, const Point & p2, int size,
	       bool filled, bool back) {
	const double SPREAD = 18.0 * M_PI / 180.0;

	double angle = atan2((double)p2.get_y() - p1.get_y(), (double)p2.get_x() - p1.get_x());

	static vector<Point> points(3);		// FIXME is the static of any benifit ???

	points[0].set_x(p2.get_x() - (int)rint(double(size) * cos(angle - SPREAD)));
	points[0].set_y(p2.get_y() - (int)rint(double(size) * sin(angle - SPREAD)));

	points[1] = p2;

	points[2].set_x(p2.get_x() - (int)rint(double(size) * cos(angle + SPREAD)));
	points[2].set_y(p2.get_y() - (int)rint(double(size) * sin(angle + SPREAD)));

	if(filled) {
	    drawable->draw_polygon(points, true);
	}
	else {
	    if(back) {
		drawable->draw_polygon(points, false);
	    }
	    else {
		drawable->draw_lines(points);
	    }
	}
    }

    void
    draw_diamond(Drawable * drawable, const Point & p1, const Point & p2, int size, bool filled) {
	const double SPREAD = 22.0 * M_PI / 180.0;

	double angle = atan2((double)p2.get_y() - p1.get_y(), (double)p2.get_x() - p1.get_x());

	static vector<Point> points(4);		// FIXME as above


	points[0] = p2;

	points[1].set_x(p2.get_x() - (int)rint(double(size) / 2.0 * cos(angle - SPREAD)));
	points[1].set_y(p2.get_y() - (int)rint(double(size) / 2.0 * sin(angle - SPREAD)));

	points[2].set_x(p2.get_x() - (int)rint(double(size) * cos(angle) * cos(SPREAD)));
	points[2].set_y(p2.get_y() - (int)rint(double(size) * sin(angle) * cos(SPREAD)));

	points[3].set_x(p2.get_x() - (int)rint(double(size) / 2.0 * cos(angle + SPREAD)));
	points[3].set_y(p2.get_y() - (int)rint(double(size) / 2.0 * sin(angle + SPREAD)));

	drawable->draw_polygon(points, filled);
    }

} // namespace Drawing
