// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "point.h"

#include "foundation/macros.h"

#include "vector.h"

#include <iostream>
using namespace std;

namespace Drawing {

    Point::Point() :
	mX(0), mY(0) {
    }

    Point::Point(int x, int y) :
	mX(x), mY(y) {
    }

    Point &
    Point::operator += (const Vector & v) {
	mX += v.get_x();
	mY += v.get_y();

	return *this;
    }

    Point &
    Point::operator -= (const Vector & v) {
	mX -= v.get_x();
	mY -= v.get_y();

	return *this;
    }

    bool
    operator == (const Point & p1, const Point & p2) {
	return p1.get_x() == p2.get_x() && p1.get_y() == p2.get_y();
    }

    bool
    operator != (const Point & p1, const Point & p2) {
	return !(p1 == p2);
    }

    Point
    operator + (const Point & p, const Vector & v) {
	return Point(p.get_x() + v.get_x(), p.get_y() + v.get_y());
    }

    Point
    operator - (const Point & p, const Vector & v) {
	return Point(p.get_x() - v.get_x(), p.get_y() - v.get_y());
    }

    ostream &
    operator << (ostream & ost, const Point & point) {
	ost << "(" << point.get_x() << ", " << point.get_y() << ")";

	return ost;
    }

    istream &
    operator >> (istream & ist, Point & point) {
	char c;

        //                                    '('

	ist >> c;

        if(!ist.good()) {
            // probably just an eof
            return ist;
        }

        if(c != '(') {
            ERROR("Expected '('");
            ist.clear(ios::failbit);
            return ist;
        }

        //                                     x

	int x;

	ist >> x;

        if(!ist.good()) {
            ERROR("Expected an int");
            return ist;
        }

        //                                    ','

	ist >> c;

        if(!ist.good()) {
            ERROR("Expected a char");
            return ist;
        }

        if(c != ',') {
            ERROR("Expected ','");
            ist.clear(ios::failbit);
            return ist;
        }

        //                                     y

	int y;

	ist >> y;

        if(!ist.good()) {
            ERROR("Expected an int");
            return ist;
        }

        //                                    ')'

	ist >> c;

        if(!ist.good()) {
            ERROR("Expected a char");
            return ist;
        }

        if(c != ')') {
            ERROR("Expected ')'");
            ist.clear(ios::failbit);
            return ist;
        }

        // everything went correctly

	point.set(x, y);

	return ist;
    }

} // namespace Drawing
