// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "segment.h"

#include "vector.h"
#include "line.h"

#include <cmath>
#include <iostream>
using namespace std;

namespace Drawing {

    Segment::Segment(const Point & p1, const Point & p2) : mPoint1(p1), mPoint2(p2) {
    }

    double
    Segment::get_distance_to(const Point & p0) const {
	const Point & p1 = mPoint1;
	const Point & p2 = mPoint2;

	Vector v = p2 - p1;

	double distance0;
	double distance1;
	double distance2;

	distance0 = Line(p1, p2).get_distance_to(p0);

	distance1 = Line(p1, Point(p1.get_x() - v.get_y(),
				p1.get_y() + v.get_x())).get_distance_to(p0);

	distance2 = Line(p2, Point(p2.get_x() - v.get_y(),
				p2.get_y() + v.get_x())).get_distance_to(p0);

	double line_length = get_length();

	double distance;

	if(distance1 > line_length || distance2 > line_length) {
	    if(distance1 < distance2) {
		distance = sqrt(distance1 * distance1 + distance0 * distance0);
	    }
	    else {
		distance = sqrt(distance2 * distance2 + distance0 * distance0);
	    }
	}
	else {
	    distance = distance0;
	}

	return distance;
    }

    double
    Segment::get_length() const {
	return (mPoint2 - mPoint1).get_length();
    }

    bool
    Segment::get_intercept(const Segment & segment, Point & point) const {
	if(Line(mPoint1, mPoint2).get_intercept(Line(segment.mPoint1, segment.mPoint2), point)) {
	    int xmin1, xmax1;

	    if(mPoint1.get_x() < mPoint2.get_x()) {
		xmin1 = mPoint1.get_x();
		xmax1 = mPoint2.get_x();
	    }
	    else {
		xmin1 = mPoint2.get_x();
		xmax1 = mPoint1.get_x();
	    }

	    int ymin1, ymax1;

	    if(mPoint1.get_y() < mPoint2.get_y()) {
		ymin1 = mPoint1.get_y();
		ymax1 = mPoint2.get_y();
	    }
	    else {
		ymin1 = mPoint2.get_y();
		ymax1 = mPoint1.get_y();
	    }

	    int xmin2, xmax2;

	    if(segment.mPoint1.get_x() < segment.mPoint2.get_x()) {
		xmin2 = segment.mPoint1.get_x();
		xmax2 = segment.mPoint2.get_x();
	    }
	    else {
		xmin2 = segment.mPoint2.get_x();
		xmax2 = segment.mPoint1.get_x();
	    }

	    int ymin2, ymax2;

	    if(segment.mPoint1.get_y() < segment.mPoint2.get_y()) {
		ymin2 = segment.mPoint1.get_y();
		ymax2 = segment.mPoint2.get_y();
	    }
	    else {
		ymin2 = segment.mPoint2.get_y();
		ymax2 = segment.mPoint1.get_y();
	    }

	    return (point.get_x() >= xmin1 && point.get_x() <= xmax1 &&
		    point.get_y() >= ymin1 && point.get_y() <= ymax1 &&
		    point.get_x() >= xmin2 && point.get_x() <= xmax2 &&
		    point.get_y() >= ymin2 && point.get_y() <= ymax2);
	}
	else {
	    return false;
	}
    }

    ostream &
    operator << (ostream & ost, const Segment & segment) {
	const Point & p1 = segment.get_point1();
	const Point & p2 = segment.get_point2();

	ost << "(x1, y1, x2, y2): ("
	    << p1.get_x() << ", " << p1.get_y() << ", "
	    << p2.get_x() << ", " << p2.get_y() << ")";


	return ost;
    }

} // namespace Drawing
