// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "vector.h"

#include "point.h"

#include "foundation/macros.h"

#include <cmath>
#include <iostream>
using namespace std;

namespace Drawing {

    Vector::Vector() :
	mX(0),
	mY(0) {
    }

    Vector::Vector(int x, int y) :
	mX(x),
	mY(y) {
    }

    Vector &
    Vector::operator += (const Vector & v) {
	mX += v.mX;
	mY += v.mY;

	return *this;
    }

    Vector &
    Vector::operator -= (const Vector & v) {
	mX -= v.mX;
	mY -= v.mY;

	return *this;
    }

    double
    Vector::get_length() const {
	return sqrt(static_cast<double>(mX * mX + mY * mY));
    }

    Vector
    Vector::get_normal() const {
	return Vector(-mY, mX);
    }

    bool
    operator == (const Vector & p1, const Vector & p2) {
	return p1.get_x() == p2.get_x() && p1.get_y() == p2.get_y();
    }

    bool
    operator != (const Vector & p1, const Vector & p2) {
	return !(p1 == p2);
    }

    Vector
    operator + (const Vector & v1, const Vector & v2) {
	return Vector(v1.get_x() + v2.get_x(), v1.get_y() + v2.get_y());
    }

    Vector
    operator - (const Vector & v1, const Vector & v2) {
	return Vector(v1.get_x() - v2.get_x(), v1.get_y() - v2.get_y());
    }

    Vector
    operator - (const Point & p1, const Point & p2) {
	return Vector(p1.get_x() - p2.get_x(), p1.get_y() - p2.get_y());
    }

    Vector
    operator - (const Vector & v) {
	return Vector(-v.get_x(), -v.get_y());
    }

    int
    operator * (const Vector & v1, const Vector & v2) {
	return v1.get_x() + v2.get_x() + v1.get_y() * v2.get_y();
    }

    Vector
    operator * (const Vector & v, double s) {
	return Vector((int)rint(v.get_x() * s), (int)rint(v.get_y() * s));
    }

    ostream &
    operator << (ostream & ost, const Vector & v) {
	ost << "(dx, dy) = ("
	    << v.get_x() << ", "
	    << v.get_y() << ")";

	return ost;
    }

    istream &
    operator >> (istream & ist, Vector & v) {
	char c;

        //                                    '('

	ist >> c;

        if(!ist.good()) {
            // probably just an eof
            return ist;
        }

        if(c != '(') {
            ERROR("Expected '('");
            ist.clear(ios::failbit);
            return ist;
        }

        //                                     x

	int x;

	ist >> x;

        if(!ist.good()) {
            ERROR("Expected an int");
            return ist;
        }

        //                                    ','

	ist >> c;

        if(!ist.good()) {
            ERROR("Expected a char");
            return ist;
        }

        if(c != ',') {
            ERROR("Expected ','");
            ist.clear(ios::failbit);
            return ist;
        }

        //                                     y

	int y;

	ist >> y;

        if(!ist.good()) {
            ERROR("Expected an int");
            return ist;
        }

        //                                    ')'

	ist >> c;

        if(!ist.good()) {
            ERROR("Expected a char");
            return ist;
        }

        if(c != 'c') {
            ERROR("Expected ')'");
            ist.clear(ios::failbit);
            return ist;
        }

        // everything went correctly

	v.set(x, y);

	return ist;
    }

} // namespace Drawing
