// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <iostream>

#include <gtk/gtkmain.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkvbox.h>

#include "gtk/tree.h"
#include "gtk/toolbar.h"
#include "gtk/image.h"

#include "foundation/command_undo.h"
#include "foundation/command_redo.h"
#include "foundation/command_callback.h"
#include "foundation/exception.h"

#ifdef DEBUG
int trace_level = 0;
#endif

extern "C" {
    int FT_Get_Next_Char;
    int FT_Get_First_Char;
}

namespace {

    gint
    delete_event_callback(GtkWidget * widget, GdkEvent * event, gpointer data) {
	return FALSE;		// this will cause the destroy event to be emitted
    }

    void
    destroy_callback(GtkWidget * widget, gpointer data) {
	gtk_main_quit();
    }

} // namespace {anonymous}

int
main(int argc, char * argv[]) try {
    gtk_init(&argc, &argv);

    GtkWidget * toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    GTK::Image::set_widget(toplevel);

    g_signal_connect(G_OBJECT(toplevel), "delete_event", G_CALLBACK(&delete_event_callback), 0);
    g_signal_connect(G_OBJECT(toplevel), "destroy", G_CALLBACK(&destroy_callback), 0);

    GtkWidget * vbox = gtk_vbox_new(FALSE, 0);

    gtk_container_add(GTK_CONTAINER(toplevel), vbox);

    //

    GTK::Toolbar * _toolbar = new GTK::Toolbar();

    GtkWidget * toolbar = _toolbar->get_base_widget();

    {
	Command * command;
	GTK::Image * image;

	command = new CommandUndo();
	image = new GTK::Image("toolbar/undo.png");

	_toolbar->add_element(command, image);

	//

	command = new CommandRedo();
	image = new GTK::Image("toolbar/redo.png");

	_toolbar->add_element(command, image);

	//

	command = command_callback(&gtk_main_quit);
	image = new GTK::Image("toolbar/quit.png");

	_toolbar->add_element(command, image);
    }

    gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);

    gtk_widget_show(toolbar);

    //

    GTK::Tree * _browser = new GTK::Tree();

    GtkWidget * browser = _browser->get_base_widget();

    {
	GTK::Image * image;

	image = new GTK::Image("toolbar/undo.png");

	_browser->add(image /* dummy object */, image, "Hello", 0, 0);
    }

    gtk_box_pack_start(GTK_BOX(vbox), browser, TRUE, TRUE, 0);

    gtk_widget_show(browser);

    //

    gtk_widget_show(vbox);

    gtk_widget_show(toplevel);

    gtk_main();
}
catch(Exception & exception) {
    cerr << "Died in main with exception" << endl << exception << endl;
}
catch(...) {
    cerr << "Unknown exception in main" << endl;
}
