// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <iostream>
#include <string>

#include "foundation/macros.h"

#include <uml/foundation/core/class.h>
#include <uml/foundation/core/generalization.h>

using namespace UML;
using namespace std;

#ifdef DEBUG
int trace_level = 0;
#endif

//
// Dachshund Code Generator (general; what happens for C++ specifically???)
//
class DachshundCodeGenerator {
public:

	virtual int generate_class (UML::Classifier *) = 0;
};

// C++ DCG
class DCG_Cpp : public DachshundCodeGenerator {
public:

	int generate_class (UML::Classifier *);
};

// Java DCG
class DCG_Java : public DachshundCodeGenerator {
public:

	int generate_class (UML::Classifier *);
};

// generates a C++ class from a UML classifier
int DCG_Cpp::generate_class (UML::Classifier *uml_classifier) {

	cout << "\\\\\n\\\\ C++ class generated by DCG1 example\n\\\\" << endl;

	cout << "class <insert-name-here>";

	const list<Generalization *> & generalizations =
	    uml_classifier->get_generalization();

	if(!generalizations.empty()) {
	    cout << " : ";

	    list<Generalization *>::const_iterator iter = generalizations.begin();

	    while(iter != generalizations.end()) {
		cout << "public <other-name-here>";

		if(++iter != generalizations.end()) {
		    cout << ", ";
		}
	    }
	}

	cout << " {" << endl;

	cout << "};" << endl <<endl;

	return 0;
}

// generates a Java class from a UML classifier
int DCG_Java::generate_class (UML::Classifier *uml_classifier) {

	cout << "\\\\\n\\\\ Java class generated by DCG1 example\n\\\\" << endl;

	cout << "class <insert-name-here>";

	const list<Generalization *> & generalizations =
	    uml_classifier->get_generalization();

	if(!generalizations.empty()) {
	    cout << " : ";

	    ASSERT(generalizations.size() == 1);	// java doesn't support
							// multiple inheritance

	    list<Generalization *>::const_iterator iter = generalizations.begin();

	    while(iter != generalizations.end()) {
		cout << "extends <other-name-here>";

		if(++iter != generalizations.end()) {
		    cout << ", ";
		}
	    }
	}

	cout << "\n{" << endl;

	cout << "}" << endl <<endl;

	return 0;
}

int
main(int argc, char *argv[]) {
	DachshundCodeGenerator * dcg = 0;
	string lingo = "";
	bool dcg_ok = true;

	UML::Class * class1 = new UML::Class();
	UML::Class * class2 = new UML::Class();
	UML::Generalization * generalization1 = new UML::Generalization();

	if (argc>1)
		lingo = argv[1];

	// create UML model
	generalization1->set_parent(class1);
	generalization1->set_child(class2);

	class1->add_generalization(generalization1);
	class2->add_specialization(generalization1);

	// code generate...
	if (lingo == "java")
		dcg = new DCG_Java();
	else if (lingo == "c++")
		dcg = new DCG_Cpp();
	else
		dcg_ok = false;

	if (dcg_ok) {
		dcg->generate_class(class1);
		dcg->generate_class(class2);
	} else
		cout << "Usage = dcg1 c++ | java" << endl;

	// tidy up
    class1->unref();
    class2->unref();
	generalization1->unref();

    return 0;
}

