// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <gtk/gtkmain.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>

#include "gtk/canvas.h"
#include "gtk/palette.h"
#include "gtk/image.h"
#include "gtk/factory.h"

#include "foundation/exception.h"

#include "diagram/editor.h"
#include "diagram/layer_diagram.h"
#include "diagram/palette_manager.h"
#include "diagram/mode_select.h"
#include "diagram/mode_create_fig.h"
#include "diagram/command_set_mode.h"

#include "diagram/fig_rect.h"
#include "diagram/fig_ellipse.h"

#ifdef DEBUG
int trace_level = 0;
#endif

extern "C" {
    int FT_Get_Next_Char;
    int FT_Get_First_Char;
}

namespace {

    gint
    delete_event_callback(GtkWidget * widget, GdkEvent * event, gpointer data) {
	return FALSE;		// this will cause the destroy event to be emitted
    }

    void
    destroy_callback(GtkWidget * widget, gpointer data) {
	gtk_main_quit();
    }

} // namespace {anonymous}

int
main(int argc, char * argv[]) {
    gtk_init(&argc, &argv);

    //
    // create the windows and hook 'em up
    //

    GtkWidget * p_toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    GtkWidget * d1_toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    GtkWidget * d2_toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    GTK::Image::set_widget(p_toplevel);
    GTK::Text::set_widget(p_toplevel);

    g_signal_connect(G_OBJECT(p_toplevel), "delete_event", G_CALLBACK(&delete_event_callback), 0);
    g_signal_connect(G_OBJECT(p_toplevel), "destroy", G_CALLBACK(&destroy_callback), 0);

    g_signal_connect(G_OBJECT(d1_toplevel), "delete_event", G_CALLBACK(&delete_event_callback), 0);
    g_signal_connect(G_OBJECT(d1_toplevel), "destroy", G_CALLBACK(&destroy_callback), 0);

    g_signal_connect(G_OBJECT(d2_toplevel), "delete_event", G_CALLBACK(&delete_event_callback), 0);
    g_signal_connect(G_OBJECT(d2_toplevel), "destroy", G_CALLBACK(&destroy_callback), 0);

    //
    // palette window
    //

    GTK::Palette * _palette = new GTK::Palette();
    GtkWidget * palette = _palette->get_base_widget();
    gtk_container_add(GTK_CONTAINER(p_toplevel), palette);
    gtk_widget_show(palette);

    //
    // the layer common to both editors
    //

    Diagram::LayerDiagram * layer = new Diagram::LayerDiagram();

    //
    // diagram1 window
    //

    GTK::Canvas * _canvas1 = new GTK::Canvas(1025, 1025);
    GtkWidget * canvas1 = _canvas1->get_base_widget();
    gtk_container_add(GTK_CONTAINER(d1_toplevel), canvas1);
    gtk_widget_show(canvas1);

    Diagram::Editor * editor1 = new Diagram::Editor(_canvas1);
    editor1->add(layer);
    editor1->set_active(true);

    //
    // diagram2 window
    //

    GTK::Canvas * _canvas2 = new GTK::Canvas(1025, 1025);
    GtkWidget * canvas2 = _canvas2->get_base_widget();
    gtk_container_add(GTK_CONTAINER(d2_toplevel), canvas2);
    gtk_widget_show(canvas2);

    Diagram::Editor * editor2 = new Diagram::Editor(_canvas2);
    editor2->add(layer);
    editor2->set_active(true);

    //
    // palette tools
    //

    Diagram::Mode * mode;
    Command * command;
    Drawing::Image * image;

    mode = new Diagram::ModeSelect();
    command = new Diagram::CommandSetMode(mode, "Select and manipulate");
    mode->unref();
    image = new GTK::Image("palette/cursor.png");

    _palette->add_element(command, image);
    command->execute();         // use the default mode
    command->unref();

    //

    mode = new Diagram::ModeCreateFig<Diagram::FigRect>();
    command = new Diagram::CommandSetMode(mode, "Create rectangle");
    mode->unref();
    image = new GTK::Image("palette/rectangle.png");

    _palette->add_element(command, image);
    command->unref();

    //

    mode = new Diagram::ModeCreateFig<Diagram::FigEllipse>();
    command = new Diagram::CommandSetMode(mode, "Create ellipse");
    mode->unref();
    image = new GTK::Image("palette/ellipse.png");

    _palette->add_element(command, image);
    command->unref();

    //
    // go
    //

    gtk_widget_show(d2_toplevel);
    gtk_widget_show(d1_toplevel);
    gtk_widget_show(p_toplevel);

    gtk_main();

    //
    // cleanup
    //

    // TODO

    return 0;
}
