// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "composite_command.h"

#include "macros.h"

#include <algorithm>
#include <functional>
using namespace std;

CompositeCommand::CompositeCommand() : Command(),
    mClosed(false) {
}

CompositeCommand::~CompositeCommand() {
    for_each(mCommands.begin(), mCommands.end(), mem_fun(&Object::unref));
}

bool
CompositeCommand::is_empty() const {
    return mCommands.empty();
}

void
CompositeCommand::add(Command * command) {
    RETURN_IF_FAIL(!mClosed);

    command->ref();
    mCommands.push_back(command);
}

void
CompositeCommand::execute() {
    close();
    for_each(mCommands.begin(), mCommands.end(), mem_fun(&Command::execute));
}

void
CompositeCommand::unexecute() {
    close();
    for_each(mCommands.rbegin(), mCommands.rend(), mem_fun(&Command::unexecute));
}
