// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_FUNCTIONAL_H
#define FOUNDATION_FUNCTIONAL_H

#include <functional>
using namespace std;

// one day C++ will be fixed so that a reference to a reference is
// treated as a reference, but until that day, this file allows us
// to go on...

template<class Operation>
class binder2nd_vr : public unary_function<typename Operation::first_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    typename Operation::second_argument_type mArgument2;

public:
    binder2nd_vr(const Operation & operation,
		 typename Operation::second_argument_type argument2) :
    mOperation(operation), mArgument2(argument2) { }

    typename Operation::result_type
    operator() (const typename Operation::first_argument_type & argument1) const {
	return mOperation(argument1, mArgument2);
    }
};

template<class Operation, class T>
inline binder2nd_vr<Operation>
bind2nd_vr(const Operation & operation, const T & argument2) {
    return binder2nd_vr<Operation>(operation, typename Operation::second_argument_type(argument2));
}



template<class Operation>
class binder2nd_rr : public unary_function<typename Operation::first_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    typename Operation::second_argument_type mArgument2;

public:
    binder2nd_rr(const Operation & operation,
		 typename Operation::second_argument_type argument2) :
    mOperation(operation), mArgument2(argument2) { }

    typename Operation::result_type
    operator() (typename Operation::first_argument_type argument1) const {
	return mOperation(argument1, mArgument2);
    }
};

template<class Operation, class T>
inline binder2nd_rr<Operation>
bind2nd_rr(const Operation & operation, const T & argument2) {
    return binder2nd_rr<Operation>(operation, typename Operation::second_argument_type(argument2));
}



template<class Operation>
class binder2nd_rv : public unary_function<typename Operation::first_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    const typename Operation::second_argument_type & mArgument2;

public:
    binder2nd_rv(const Operation & operation,
		 const typename Operation::second_argument_type & argument2) :
    mOperation(operation), mArgument2(argument2) { }

    typename Operation::result_type
    operator() (typename Operation::first_argument_type argument1) const {
	return mOperation(argument1, mArgument2);
    }
};

template<class Operation, class T>
inline binder2nd_rv<Operation>
bind2nd_rv(const Operation & operation, const T & argument2) {
    return binder2nd_rv<Operation>(operation, typename Operation::second_argument_type(argument2));
}

///


template<class Operation>
class binder1st_vr : public unary_function<typename Operation::second_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    const typename Operation::first_argument_type & mArgument1;

public:
    binder1st_vr(const Operation & operation,
		 const typename Operation::first_argument_type & argument1) :
    mOperation(operation), mArgument1(argument1) { }

    typename Operation::result_type
    operator() (typename Operation::second_argument_type argument2) const {
	return mOperation(mArgument1, argument2);
    }
};

template<class Operation, class T>
inline binder1st_vr<Operation>
bind1st_vr(const Operation & operation, const T & argument1) {
    return binder1st_vr<Operation>(operation, typename Operation::first_argument_type(argument1));
}



template<class Operation>
class binder1st_rr : public unary_function<typename Operation::second_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    typename Operation::first_argument_type mArgument1;

public:
    binder1st_rr(const Operation & operation,
		 typename Operation::first_argument_type argument1) :
    mOperation(operation), mArgument1(argument1) { }

    typename Operation::result_type
    operator() (typename Operation::second_argument_type argument2) const {
	return mOperation(mArgument1, argument2);
    }
};

template<class Operation, class T>
inline binder1st_rr<Operation>
bind1st_rr(const Operation & operation, const T & argument1) {
    return binder1st_rr<Operation>(operation, typename Operation::first_argument_type(argument1));
}




template<class Operation>
class binder1st_rv : public unary_function<typename Operation::second_argument_type,
					   typename Operation::result_type> {
private:
    Operation mOperation;
    typename Operation::first_argument_type mArgument1;

public:
    binder1st_rv(const Operation & operation,
		 typename Operation::first_argument_type argument1) :
    mOperation(operation), mArgument1(argument1) { }

    typename Operation::result_type
    operator() (const typename Operation::second_argument_type & argument2) const {
	return mOperation(mArgument1, argument2);
    }
};

template<class Operation, class T>
inline binder1st_rv<Operation>
bind1st_rv(const Operation & operation, const T & argument1) {
    return binder1st_rv<Operation>(operation, typename Operation::first_argument_type(argument1));
}

#endif // FOUNDATION_FUNCTIONAL_H
