// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_HANDLE_H
#define FOUNDATION_HANDLE_H

#error "Don't use this class, we haven't hit rock bottom yet!"

#include "macros.h"

#include <typeinfo>
using namespace std;

template<class T>
class Handle {
public:
    Handle() : mT(0) { }
    Handle(const Handle & handle) : mT(handle.mT) { if(mT) mT->ref(); }
    template<class U>
    Handle(const Handle<U> & handle) : mT(handle.mT) { if(mT) mT->ref(); }
    Handle(T * t) : mT(t) { }
    ~Handle() { if(mT) mT->unref(); }

    T & operator = (const Handle & handle) {
	if(mT != handle.mT) {
	    if(mT) {
		mT->unref();
	    }

	    mT = handle.mT;

	    if(mT) {
		mT->ref();
	    }
	}

	return *this;
    }

    T & operator = (T * t) {
	if(mT != t) {
	    if(mT) {
		mT->unref();
	    }

	    mT = t;
	}

	return *this;
    }

    T & operator * () const { return *mT; }
    T * operator -> () const { return mT; }

    operator bool () const { return mT != 0; }
    bool operator ! () const { return mT == 0; }

    template<class U>
    operator Handle<U> () const { if(mT) mT->ref(); return Handle<U>(mT); }

    template<class U>
    bool is() const { ASSERT(mT); return typeid(T) == typeid(U); }

    template<class U>
    bool is_a() const { ASSERT(mT); return dynamic_cast<U *>(mT); }

    template<class U>
    Handle<U> down_cast() const {
	ASSERT(mT); mT->ref();
	return Handle<U>(mT);
    }

    template<class U>
    Handle<U> up_cast() const {
	ASSERT(mT);
	U * u = dynamic_cast<U *>(mT);
	ASSERT(u);
	u->ref();
	return Handle<U>(u);
    }

protected:

private:
    T * mT;
};

#endif // FOUNDATION_HANDLE_H
