// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_UTILITY_H
#define FOUNDATION_UTILITY_H

#include <typeinfo>
using namespace std;

#include "macros.h"

template<class T>
void delete_ptr(T * t) {
    delete t;
}

template<class T>
void delete_array(T * t) {
    delete[] t;
}

template<class T>
void delete_1st(const T & t) {
    delete t.first;
}

template<class T>
void delete_2nd(const T & t) {
    delete t.second;
}

template<class T>
void delete_both(const T & t) {
    delete t.first;
    delete t.second;
}

template<class NewType, class OldType>
NewType & death_cast(OldType & input) {
    return *static_cast<NewType *>(const_cast<void *>(static_cast<const void *>(&input)));
}

template<class NewType, class OldType>
NewType known_cast(OldType input) {
    if(!input) {
	FATAL("Trying to cast from null pointer");
    }

    NewType output = dynamic_cast<NewType>(input);

    if(!output) {
	FATAL("Error casting from old type: " << typeid(OldType).name() <<
	      " to new type: " << typeid(NewType).name() <<
	      ", actual type is: " << typeid(*input).name());
    }

    return output;
}

template<class Input, class Output>
struct Caster {
    Output * operator()(Input * input) const {
	Output * output = known_cast<Output *>(input);
	return output;
    }
};

template<class Type>
struct Dereferencer {
    Type & operator()(Type * type) const {
	return *type;
    }

    const Type & operator()(const Type * type) const {
	return *type;
    }
};

#endif // FOUNDATION_UTILITY_H
