// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "foundation/macros.h"
#include "foundation/utility.h"
#include "foundation/undo_manager.h"

#include "xml/sax_parser.h"

#include "common/xmi_handler.h"

#include "mof/model_handler.h"
#include "mof/package.h"
#include "mof/model.h"

#include "root_package.h"

#include <iostream>
#include <string>
using namespace std;

#include <unistd.h>

#ifdef DEBUG
int trace_level = 3;
#endif

int
main(int argc, char * argv[]) {
    string output_dir = ".";
    string include_dir = "";
    string model_name = "";

    for(int i = 1; i < argc; ++i) {
	if(argv[i] == string("--help")) {
	    cout
		<< "Usage:" << endl
		<< "    generator "
		<< "[--outputdir \"outputdir\"] "
		<< "[--includedir \"includedir\"] "
		<< "[--modelname \"modelname\"]"
		<< endl;
	    return 0;
	}
	else if(argv[i] == string("--outputdir")) {
	    ASSERT(++i != argc);
	    output_dir = argv[i];
	}
	else if(argv[i] == string("--includedir")) {
	    ASSERT(++i != argc);
	    include_dir = argv[i];
	}
	else if(argv[i] == string("--modelname")) {
	    ASSERT(++i != argc);
	    model_name = argv[i];
	}
	else {
	    WARNING("Unrecognised argument: " << argv[i]);
	}
    }

    if(isatty(STDIN_FILENO)) {
	cout << "Reading from standard input..." << endl;
    }

    //

    MOF::SAXHandler * content_handler = new MOF::ModelHandler();

    Common::XMIHandler * xmi_handler = new Common::XMIHandler(content_handler);
    content_handler->unref();

    XML::SAXParser * parser = new XML::SAXParser(xmi_handler);

    try {
	cout << "Parsing XML input to build MOF representation of Metamodel" << endl;
	parser->parse(cin, 2);
	cout << "Finished parsing" << endl;
    }
    catch(...) {
	ERROR("An exception occured while reading");

	parser->unref();

	throw;
    }

    parser->unref();			// we are finished with the parser now

    MOF::Package * content = known_cast<MOF::Package *>(xmi_handler->get_content());
    content->ref();
    xmi_handler->unref();		// we are finished with the xmi_handler now

    MOF::Model * context = new MOF::Model();
    context->ref_set_root(content);

    if(!model_name.empty()) {
	MOF::Name * name = new MOF::Name();
	name->set_value(model_name);

	content->set_name(name);

	name->unref();
    }
    else {
	model_name = content->get_name()->get_value();
    }

    Generator::Package * package = new Generator::RootPackage(content, model_name);

    try {
	cout << "Generating C++ version of Metamodel" << endl;
	package->generate(output_dir, include_dir, 0);
	cout << "Finished generating" << endl;
    }
    catch(...) {
	ERROR("An exception occurred while generating");

	package->unref();

	content->unref();
	context->unref();

	throw;
    }

    package->unref();

    content->ref_delete();
    context->unref();
    content->unref();

    cout << "All done." << endl;

    /*
    UndoManager * undo_manager = UndoManager::instance();
    undo_manager->clear();

    Object::show_remaining();
    */

    return 0;
}
