// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GENERATOR_IO_H
#define GENERATOR_IO_H

#include "foundation/object.h"

#include <string>
#include <list>
using namespace std;

namespace MOF {

    class Package;
    class Class;
    class Reference;
    class AssociationEnd;
    class Attribute;
    class DataType;

} // namespace MOF

namespace Generator {

    class IO : public Object {
    public:
	IO(MOF::Package * root_package, const string & model_name);

	virtual void generate(const string & output_dir, const string & include_dir);

    protected:
	~IO();

	virtual void load_details();

	MOF::Package * mPackage;
	string mModelName;

	struct SuperTypeDetails {
	    MOF::Class * klass;
	    string name;
	    string role;
	    string variable;
	};

	enum Cardinality {
	    UNARY,
	    MULTI
	};

	enum Ordering {
	    UNORDERED,
	    ORDERED
	};

	struct ReferenceDetails {
	    MOF::Reference * reference;

	    MOF::AssociationEnd * this_end;
	    bool this_end_aggregate;
	    MOF::AssociationEnd * other_end;
	    bool other_end_navigable;
	    string other_end_name;
	    string other_end_role;
	    string other_end_variable;
	    MOF::Class * other_type;
	    string other_type_name;
	    string other_type_role;
	    Cardinality other_cardinality;
	    Ordering other_ordering;
	};

	struct AttributeDetails {
	    MOF::Attribute * attribute;
	    string name;
	    string role;
	    string variable;
	    string member;

	    MOF::DataType * type;
	    string type_name;
	};

	struct ClassDetails {
	    MOF::Class * klass;
	    string name;
	    string role;
	    string variable;
	    string qualified_name;
	    string qualified_path;

	    list<ReferenceDetails> references;
	    list<AttributeDetails> attributes;
	    list<SuperTypeDetails> supertypes;
	};

	struct DataTypeDetails {
	    MOF::DataType * type;
	    string name;
	    string role;
	    string variable;
	    string qualified_name;
	    string qualified_path;
	};

	list<ClassDetails> mAbstractClasses;
	list<ClassDetails> mConcreteClasses;
	list<ClassDetails> mAllClasses;

	list<DataTypeDetails> mDataTypeDetails;;

    private:
	bool mDetailsLoaded;
    };

} // namespace Generator

#endif // GENERATOR_IO_H
