// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "utility.h"

#include "common/characters.h"

#include "foundation/macros.h"

namespace Generator {

    using namespace Common;

    string
    get_prologue() {
	return
	    string("// generated by Dachshund V" VERSION "\n") +
	    string("//\n") +
	    string("// Dachshund,\n") +
	    string("// Copyright (C) 2001 David Bryant\n") +
	    string("//\n") +
	    string("// This program is free software; you can redistribute it and/or modify\n") +
	    string("// it under the terms of the GNU General Public License as published by\n") +
	    string("// the Free Software Foundation; either version 2 of the License, or\n") +
	    string("// (at your option) any later version.\n") +
	    string("//\n") +
	    string("// This program is distributed in the hope that it will be useful,\n") +
	    string("// but WITHOUT ANY WARRANTY; without even the implied warranty of\n") +
	    string("// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n") +
	    string("// GNU General Public License for more details.\n") +
	    string("//\n") +
	    string("// You should have received a copy of the GNU General Public License\n") +
	    string("// along with this program; if not, write to the Free Software\n") +
	    string("// Foundation, Inc., 59 Temple Place - ") +
	    string("Suite 330, Boston, MA 02111-1307, USA.\n");
    }

    string
    name_to_path(const string & name) {
	string path;

	char pc = '\0';			// previous character

	for(unsigned int i = 0; i < name.size(); ++i) {
	    char cc = name[i];		// current character

	    if(cc == '_') {
	    }
	    else if(is_upper(cc)) {
		if(pc != '\0' && pc != ':' && pc != '-' && !is_upper(pc)) {
		    path.push_back('_');
		}

		path.push_back(lower(cc));
	    }
	    else if(cc == ':') {
		path.push_back('/');
		++i;
	    }
	    else {
		path.push_back(cc);
	    }

	    pc = cc;
	}

	return path;
    }

    string
    path_to_guard(const string & path) {
	string guard;

	char pc = '\0';

	for(unsigned int i = 0; i < path.size(); ++i) {
	    char cc = path[i];

	    if(cc == '/') {
		guard.push_back('_');
	    }
	    else {
		guard.push_back(upper(cc));
	    }

	    pc = cc;
	}

	return guard + "_H";
    }

    string
    name_to_role(const string & name) {
	string ret;

	char pc = '\0';

	for(unsigned int i = 0; i < name.size(); ++i) {
	    const char cc = name[i];

	    if(is_upper(cc)) {
		if(i != 0 && is_lower(pc)) {
		    ret += '_';
		}

		ret += lower(cc);
	    }
	    else if(cc == '-') {
		ret += '_';
	    }
	    else {
		ret += cc;
	    }

	    pc = cc;
	}

	return ret;
    }

    string
    name_to_member(const string & name) {
	ASSERT(!name.empty());

	return string("m") + upper(name[0]) + name.substr(1);
    }

    string
    trim_path(const string & path, int levels) {
	string::size_type index = 0;

	while(levels-- > 0) {
	    index = path.find("::", index);

	    ASSERT(index != string::npos);

	    index += 2;
	}

	return path.substr(index);
    }

    string
    role_to_variable(const string & name) {
	static const char * reserved_words[] = {
	    "namespace",
	    "class",
	    "static",
	    "unsigned",
	    "bool",
	    "string",
	    "virtual"
	};

	static const int TABLE_SIZE = sizeof(reserved_words) / sizeof(reserved_words[0]);

	for(int i = 0; i < TABLE_SIZE; ++i) {
	    if(reserved_words[i] == name) {
		return reserved_words[i] + string("_");
	    }
	}

	// if we are here then the name argument wont cause a clash in C++,
	// so we hand it straight back to them

	return name;
    }

    string
    singular_to_plural(const string & singular) {
	string::size_type size = singular.size();

	if(size > 1) {
	    string end = singular.substr(size - 2, 2);

	    if(end == "sh" || end == "ch") {
		return singular + "es";
	    }
	}

	if(size > 0) {
	    string end = singular.substr(size - 1, 1);

	    if(end == "z" || end == "x" || end == "s") {
		return singular + "es";
	    }
	    else if(end == "y") {
		return singular.substr(0, size - 1) + "ies";
	    }
	}

	return singular + "s";
    }

} // namespace Generator
