// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "model.h"

#include "listener.h"
#include "event.h"

#include <algorithm>
#include <functional>
using namespace std;

namespace Graph {

    Model::Model() : Object() {
    }

    Model::~Model() {
    }

    void
    Model::add(Listener * graph_listener) {
	mListeners.push_front(graph_listener);
    }

    void
    Model::remove(Listener * graph_listener) {
	mListeners.remove(graph_listener);
    }

    //
    //
    //

    void
    Model::node_added(Object * node) {
	Event graph_event(this, node);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::node_added), &graph_event));
    }

    void
    Model::node_changed(Object * node) {
	Event graph_event(this, node);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::node_changed), &graph_event));
    }

    void
    Model::node_removed(Object * node) {
	Event graph_event(this, node);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::node_removed), &graph_event));
    }

    //

    void
    Model::edge_added(Object * edge) {
	Event graph_event(this, edge);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::edge_added), &graph_event));
    }

    void
    Model::edge_changed(Object * edge) {
	Event graph_event(this, edge);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::edge_changed), &graph_event));
    }

    void
    Model::edge_removed(Object * edge) {
	Event graph_event(this, edge);

	for_each(mListeners.begin(), mListeners.end(),
		 bind2nd(mem_fun(&Listener::edge_removed), &graph_event));
    }

} // namespace Graph
