// This code is essentially a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by The Regents of the University of California.

// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GRAPH_MODEL_H
#define GRAPH_MODEL_H

#include "foundation/object.h"

#include "exception.h"

#include <list>
#include <vector>
#include <typeinfo>
using namespace std;

namespace Graph {

    class Listener;

    class Model : public Object {
    public:
	Model();
	~Model();

	virtual void add_node(Object * node, Object * parent = 0) throw(Exception) = 0;
	virtual void remove_node(Object * node) throw(Exception) = 0;

	virtual void add_edge(Object * edge, Object * node1, Object * node2) throw(Exception) = 0;
	virtual void remove_edge(Object * edge) throw(Exception) = 0;

	void add(Listener * graph_listener);
	void remove(Listener * graph_listener);

    protected:
	void node_added(Object * node);
	void node_changed(Object * node);
	void node_removed(Object * node);

	void edge_added(Object * edge);
	void edge_changed(Object * edge);
	void edge_removed(Object * edge);

    private:
	list<Listener *> mListeners;
    };

} // namespace Graph

#endif // GRAPH_MODEL_H
