// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "box.h"

#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <functional>
#include <algorithm>
using namespace std;

namespace GTK {

    Box::Box(Toolkit::OrientationKind orientation) : Toolkit::Box(), Widget() {
        if(orientation == Toolkit::OK_HORIZONTAL) {
            mBaseWidget = gtk_hbox_new(FALSE, 2);
        }
        else if(orientation == Toolkit::OK_VERTICAL) {
            mBaseWidget = gtk_vbox_new(FALSE, 2);
        }
        else {
            ASSERT_NOT_REACHED();
        }
    }

    Box::~Box() {
        for_each(mContents.begin(), mContents.end(), mem_fun(&Object::unref));
    }

    void
    Box::add(Toolkit::Widget * widget, bool expand) {
        RETURN_IF_FAIL(widget);

        Widget * _widget = known_cast<Widget *>(widget);

        gtk_box_pack_start(GTK_BOX(mBaseWidget),
                           _widget->get_base_widget(),
                           expand, expand /* fill */, 0);

        gtk_widget_show(_widget->get_base_widget());

        widget->ref();
        mContents.push_back(_widget);
    }

} // namespace GTK
