// Dachshund,			      -*- mode: C++; c-basic-offset: 4; -*-
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "document.h"
#include "tag.h"
#include "mark.h"
#include "image.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include "toolkit/mouse_event.h"
#include "toolkit/key_event.h"
#include "toolkit/document_maintainer.h"

#include <gtk/gtksignal.h>
#include <gtk/gtktextview.h>
#include <gtk/gtktexttag.h>
#include <gtk/gtktextmark.h>
#include <gtk/gtkmain.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <gtk/gtkbutton.h>  // TEMP

//#include <algorithm>
//#include <functional>
//#include <iostream>
//#include <cstring>
using namespace std;
#include <stdio.h>



namespace {

    const char *
    temporary_mark_name() {
	static char name[128];
	static int serial = 0;

	// Return a unique name for a temporary mark
	sprintf(name, "Temp%06d", ++serial);
	return name;
    }

    /*
      const char *
      block_mark_name(Object * node) {
      static char name[128];
      static int serial = 0;

      // Return a unique name for a block mark
      sprintf(name, "Block%06d", ++serial);
      return name;
      }
    */

    void
    dump_tags(GtkTextIter & iter) {
	MESSAGE("DUMP TAGS");

	MESSAGE("\tToggled tags ON");
	GSList * list = gtk_text_iter_get_toggled_tags(& iter, 1);
	while (list) {
	    MESSAGE("\t\t" << GTK_TEXT_TAG(list->data)->name);
	    list = g_slist_next(list);
	}
	g_slist_free(list);

	MESSAGE("\tToggled tags OFF");
	list = gtk_text_iter_get_toggled_tags(& iter, 0);
	while (list) {
	    MESSAGE("\t\t" << GTK_TEXT_TAG(list->data)->name);
	    list = g_slist_next(list);
	}
	g_slist_free(list);

	MESSAGE("\tAll tags");
	list = gtk_text_iter_get_tags(& iter);
	while (list) {
	    MESSAGE("\t\t" << GTK_TEXT_TAG(list->data)->name);
	    list = g_slist_next(list);
	}
	g_slist_free(list);
    }

    void
    list_tags(GtkTextBuffer * buff) {

	MESSAGE("LIST TAG POSITIONS");
	GtkTextIter iter;
	gtk_text_buffer_get_start_iter(buff, & iter);
	do {
	    MESSAGE("\tBuffer position " << gtk_text_iter_get_offset(& iter));
	    MESSAGE("\t\tToggled tags ON");
	    GSList * list = gtk_text_iter_get_toggled_tags(& iter, 1);
	    while (list) {
		MESSAGE("\t\t\t" << GTK_TEXT_TAG(list->data)->name);
		list = g_slist_next(list);
	    }
	    g_slist_free(list);

	    MESSAGE("\t\tToggled tags OFF");
	    list = gtk_text_iter_get_toggled_tags(& iter, 0);
	    while (list) {
		MESSAGE("\t\t\t" << GTK_TEXT_TAG(list->data)->name);
		list = g_slist_next(list);
	    }
	    g_slist_free(list);

	    MESSAGE("\t\tMarks");
	    list = gtk_text_iter_get_marks(& iter);
	    while (list) {
		MESSAGE("\t\t\t" << gtk_text_mark_get_name(GTK_TEXT_MARK(list->data)));
		list = g_slist_next(list);
	    }
	    g_slist_free(list);

	    gtk_text_iter_forward_to_tag_toggle(& iter, NULL);
	} while (! gtk_text_iter_is_end(& iter));
    }
/*
    void
    list_marks(GtkTextBuffer * buff) {

	MESSAGE("LIST TAG POSITIONS");
	GtkTextIter iter;
	gtk_text_buffer_get_start_iter(buff, & iter);
	do {
	    MESSAGE("\tBuffer position " << gtk_text_iter_get_offset(& iter));
	    MESSAGE("\t\tMarks");
	    GSList * list = gtk_text_iter_get_marks(& iter);
	    while (list) {
		MESSAGE("\t\t\t" << gtk_text_mark_get_name(GTK_TEXT_MARK(list->data)));
		list = g_slist_next(list);
	    }
	    g_slist_free(list);
	} while (! gtk_text_iter_is_end(& iter));
    }
*/
    GtkTextTag * get_gtk_tag(Toolkit::Tag * toolkit_tag) {
	GTK::Tag * tag = dynamic_cast<GTK::Tag *>(toolkit_tag);
	assert(tag);
	return tag->get_gtktag();
    }

    GtkTextMark * get_gtk_mark(Toolkit::Mark * toolkit_mark) {
	GTK::Mark * mark = static_cast<GTK::Mark *>(toolkit_mark);
	return mark->get_gtkmark();
    }

    void locate_mark(Toolkit::Mark * toolkit_mark, GtkTextIter & iter) {
	GtkTextMark * mark = get_gtk_mark(toolkit_mark);
	GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mark);
	gtk_text_buffer_get_iter_at_mark(buffer, & iter, mark);
    }

    void update_mark(Toolkit::Mark * toolkit_mark, GtkTextIter & iter) {
	GtkTextMark * mark = get_gtk_mark(toolkit_mark);
	GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mark);
	gtk_text_buffer_move_mark(buffer, mark, & iter);
    }

    void locate_tag_end(Toolkit::Tag * toolkit_tag, GtkTextIter & end) {
	GtkTextTag * tag = get_gtk_tag(toolkit_tag);
	//dump_tags(end);
	assert(gtk_text_iter_begins_tag(& end, tag));
	assert(gtk_text_iter_forward_to_tag_toggle(& end, tag));
    }

}


namespace GTK {

    //================================================================================
    // Callbacks for Textview
    //================================================================================

    void static_textview_delete_from_cursor(GtkTextView * textview, GtkDeleteType arg1,
					    gint arg2, void * data) {
	TRACE(1, "Textview::delete_from_cursor");
    }
					     
    void static_textview_insert_at_cursor(GtkTextView * textview,
					  gchar * arg1, void * data) {
	TRACE(1, "Textview::insert_at_cursor");
    }
					     
    void static_textview_move_cursor(GtkTextView * textview, GtkMovementStep arg1,
				     gint arg2, gboolean arg3, void * data) {
	TRACE(1, "Textview::move_cursor");
	Document * document = static_cast<Document *>(data);
	document->changes_sync();
    }
					     

    //================================================================================
    // Callbacks for Textbuffer
    //================================================================================

    void static_textbuff_changed(GtkTextBuffer * textbuffer, void * data) {
	Document * document = static_cast<Document *>(data);
	document->changed_check();
    }


    //================================================================================
    // Callbacks for Tags
    //================================================================================



    Document::Document() : Toolkit::Document(), Widget()
	//,mDownButton(0), mTimeoutId(0), mTest(0) 
    {
	mBaseWidget = gtk_text_view_new();
        gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(mBaseWidget), GTK_WRAP_WORD);
	mTextBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mBaseWidget));


	// Create the working marks
	GtkTextIter iter;
	gtk_text_buffer_get_start_iter(mTextBuffer, & iter);
	mMarkRG = gtk_text_buffer_create_mark(mTextBuffer, "Working_RG", & iter, false);
	mMarkLG = gtk_text_buffer_create_mark(mTextBuffer, "Working_LG", & iter, true);


	// Out copy of standard marks
	// - insert
	mGtkTextMarkInsert = gtk_text_buffer_get_insert(mTextBuffer);
	mMarkInsert = new GTK::Mark("insert", mGtkTextMarkInsert);
	// - change
	mGtkTextMarkChangeBegin = gtk_text_buffer_create_mark(mTextBuffer, "change_begin", & iter, 1);
	mGtkTextMarkChangeEnd = gtk_text_buffer_create_mark(mTextBuffer, "change_end", & iter, 0);
	mMarkChangeBegin = new GTK::Mark("change_begin", mGtkTextMarkChangeBegin);
	mMarkChangeEnd = new GTK::Mark("change_end", mGtkTextMarkChangeEnd);
	// - selection
	GtkTextMark * selection = gtk_text_buffer_get_selection_bound(mTextBuffer);
	mMarkSelection = new GTK::Mark("selection", selection);


	// Local change flag
	mLocalChange = false;


	// Textview signals
	g_signal_connect(G_OBJECT(mBaseWidget), "delete-from-cursor",
			 G_CALLBACK(static_textview_delete_from_cursor), this);
	g_signal_connect_after(G_OBJECT(mBaseWidget), "insert_at-cursor",
			       G_CALLBACK(static_textview_insert_at_cursor), this);
	g_signal_connect_after(G_OBJECT(mBaseWidget), "move-cursor",
			       G_CALLBACK(static_textview_move_cursor), this);

	// Textbuffer signals
	g_signal_connect_after(G_OBJECT(mTextBuffer), "changed",
			       G_CALLBACK(static_textbuff_changed), this);

	// Tag signals

	/*
	  mHandlers[HID_EXPOSE] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "expose_event",
	  G_CALLBACK(&Textview::static_expose_event), this);

	  mHandlers[HID_CONFIGURE] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "configure_event",
	  G_CALLBACK(&Textview::static_configure_event), this);

	  mHandlers[HID_MOTION_NOTIFY] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "motion_notify_event",
	  G_CALLBACK(&Textview::static_motion_notify_event), this);

	  mHandlers[HID_ENTER_NOTIFY] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "enter_notify_event",
	  G_CALLBACK(&Textview::static_enter_notify_event), this);

	  mHandlers[HID_LEAVE_NOTIFY] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "leave_notify_event",
	  G_CALLBACK(&Textview::static_leave_notify_event), this);

	  mHandlers[HID_BUTTON_PRESS] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "button_press_event",
	  G_CALLBACK(&Textview::static_button_press_event), this);

	  mHandlers[HID_BUTTON_RELEASE] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "button_release_event",
	  G_CALLBACK(&Textview::static_button_release_event), this);

	  mHandlers[HID_KEY_PRESS] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "key_press_event",
	  G_CALLBACK(&Textview::static_key_press_event), this);
    
	  mHandlers[HID_KEY_RELEASE] =
	  g_signal_connect_after(G_OBJECT(mCanvas), "key_release_event",
	  G_CALLBACK(&Textview::static_key_release_event), this);
    
	  //
    
	  gtk_widget_show(mCanvas);
	*/
	post_init();
    }

    Document::~Document() {
	/*if(mBaseWidget) {
	  for(int i = 0; i < HID_LAST; ++i) {
	  g_signal_handler_disconnect(G_OBJECT(mCanvas), mHandlers[i]);
	  }
	  }*/
    }

    //================================================================================

    class Document::MarkTag {
	// Class used to store Mark and associated Tag on mInsertStack
    public:
	GtkTextMark *	mark;
	GtkTextMark *	block_mark;
	Toolkit::Tag *	tag;
	MarkTag(GtkTextMark * m, GtkTextMark * bm, Toolkit::Tag * t) 
	    : mark(m), block_mark(bm), tag(t) { }
    };




    //================================================================================
    // Toolkit::Document methods
    //================================================================================


    Toolkit::Mark *
    Document::get_insert_mark() {
	mMarkInsert->ref();
	return mMarkInsert;
    }

    Toolkit::Mark *
    Document::get_change_mark() {
	mMarkChangeBegin->ref();
	return mMarkChangeBegin;
    }

    Toolkit::Mark *
    Document::get_selection_mark() {
	mMarkSelection->ref();
	return mMarkSelection;
    }


    //================================================================================

    void
    Document::insert_block_begin(Toolkit::Mark * toolkit_location, 
				 Toolkit::Mark * toolkit_block_mark, Toolkit::Tag * block) {
	LocalChange mask(this);
	GTK::Mark * block_mark = static_cast<GTK::Mark *>(toolkit_block_mark);

	// Get the insert location
	GtkTextIter iter;
	locate_mark(toolkit_location, iter);

	// Create a temp marker to identify the block
	GtkTextMark * mark = allocate_temp_mark(&iter);

	// Save the mark in the stack
	mInsertStack.push_back(new MarkTag(mark, block_mark->get_gtkmark(), block));
    }


    void	
    Document::insert_block_end(Toolkit::Mark * location, Toolkit::Tag * block) {
	LocalChange mask(this);

	MarkTag * item = mInsertStack.back();
	assert(item); // FIXUP
	assert(item->tag == block); // FIXUP
	mInsertStack.pop_back();

	// Validate iterators to bound the block
	GtkTextIter iter_begin, iter_end;
	gtk_text_buffer_get_iter_at_mark(mTextBuffer, &iter_begin, item->mark);
	locate_mark(location, iter_end);
	GtkTextTag * tag = get_gtk_tag(block);
	gtk_text_buffer_apply_tag(mTextBuffer, tag, &iter_begin, &iter_end);

	// Now position the block_mark at the start of the block
	gtk_text_buffer_move_mark(mTextBuffer, item->block_mark, &iter_begin);
	release_temp_mark(item->mark);
	delete item;
	//MESSAGE("Block " << block->get_name() << " applied over " << gtk_text_iter_get_offset(& iter_begin) << ' ' << gtk_text_iter_get_offset(& iter_end));
    }

    //----------------------------------------

    void
    Document::insert_field_begin(Toolkit::Mark * location, Toolkit::Tag * field) {
	LocalChange mask(this);

	// Get the insert location
	GtkTextIter iter;
	locate_mark(location, iter);

	// Create a temp marker to identify the block
	GtkTextMark * mark = allocate_temp_mark(&iter);

	// Save the mark in the stack
	mInsertStack.push_back(new MarkTag(mark, NULL, field));
    }


    void	
    Document::insert_field_end(Toolkit::Mark * location, Toolkit::Tag * field) {
	LocalChange mask(this);

	MarkTag * item = mInsertStack.back();
	assert(item); // FIXUP
	assert(item->tag == field); // FIXUP
	mInsertStack.pop_back();

	// Validate iterators to bound the block
	GtkTextIter iter_begin, iter_end;
	gtk_text_buffer_get_iter_at_mark(mTextBuffer, &iter_begin, item->mark);
	locate_mark(location, iter_end);
	GtkTextTag * tag = get_gtk_tag(field);
	gtk_text_buffer_apply_tag(mTextBuffer, tag, &iter_begin, &iter_end);
	release_temp_mark(item->mark);
	delete item;
	//MESSAGE("Field " << field->get_name() << " applied over " << gtk_text_iter_get_offset(& iter_begin) << ' ' << gtk_text_iter_get_offset(& iter_end));
    }

    //----------------------------------------

    void
    Document::insert_field_text(Toolkit::Mark * location, Toolkit::Tag * field, 
				const char * text) {
	LocalChange mask(this);

	GtkTextIter iter;
	locate_mark(location, iter);
	GtkTextTag * tag = get_gtk_tag(field);
	gtk_text_buffer_insert_with_tags(mTextBuffer, &iter, text, -1, tag, 0);
	update_mark(location, iter);
    }


    void
    Document::insert_text(Toolkit::Mark * location, const char * text) {
	LocalChange mask(this);

	GtkTextIter iter;
	locate_mark(location, iter);
	gtk_text_buffer_insert(mTextBuffer, &iter, text, -1);
	update_mark(location, iter);
    }


    void
    Document::insert_widget(Toolkit::Mark * location, Toolkit::Widget * toolkit_widget) {
	LocalChange mask(this);

	GtkTextIter iter;
	locate_mark(location, iter);

	GtkTextChildAnchor * anchor = gtk_text_buffer_create_child_anchor(mTextBuffer, &iter);
	GtkWidget * widget = dynamic_cast<GTK::Widget *>(toolkit_widget)->get_base_widget();

	gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(mBaseWidget), widget, anchor);
	//gtk_widget_show(widget);
	update_mark(location, iter);
    }


    void
    Document::insert_image(Toolkit::Mark * location, Drawing::Image * toolkit_image) {
	LocalChange mask(this);

	GtkTextIter iter;
	locate_mark(location, iter);

	GdkPixbuf * image = dynamic_cast<GTK::Image *>(toolkit_image)->get_pixbuf();
	gtk_text_buffer_insert_pixbuf(mTextBuffer, & iter, image);
	update_mark(location, iter);
    }

    //================================================================================




    void
    Document::get_field_text(Toolkit::Mark * mark, Toolkit::Tag * field, string & result) {

	// Get the bounds of the field
	GtkTextIter iter_begin;
	locate_mark(mark, iter_begin);
	GtkTextIter iter_end = iter_begin;
	locate_tag_end(field, iter_end);

	// Finally, extract the text
	gchar * s = gtk_text_iter_get_text(& iter_begin, & iter_end);
	result = s;
	g_free(s);
    }
    
    void	
    Document::replace_field_text(Toolkit::Mark * location, Toolkit::Tag * field, 
				 const char * new_text) {
	LocalChange mask(this);

	// Get the bounds of the field
	GtkTextIter iter_begin;
	locate_mark(location, iter_begin);
	GtkTextIter iter_end = iter_begin;
	locate_tag_end(field, iter_end);

	// Extract the existing text to see if it is different
	gchar * s = gtk_text_iter_get_text(& iter_begin, & iter_end);
	if (strcmp(s, new_text) != 0) {
	    // Mark the old text, and insert the new
	    GtkTextTag * tag = get_gtk_tag(field);
	    gtk_text_buffer_move_mark(mTextBuffer, mMarkRG, & iter_begin);
	    gtk_text_buffer_move_mark(mTextBuffer, mMarkLG, & iter_end);
	    gtk_text_buffer_insert_with_tags(mTextBuffer, & iter_end, new_text, -1, tag, 0);

	    // Revalidate our iterators, and delete the old text
	    gtk_text_buffer_get_iter_at_mark(mTextBuffer, & iter_begin, mMarkRG);
	    gtk_text_buffer_get_iter_at_mark(mTextBuffer, & iter_end, mMarkLG);
	    gtk_text_buffer_delete(mTextBuffer, & iter_begin, & iter_end);
	}
	g_free(s);
    }

    void	
    Document::replace_field_image(Toolkit::Mark * location, Toolkit::Tag * field, 
				  Drawing::Image * image) {
	LocalChange mask(this);
    
	NYI();
    }

    void	
    Document::replace_field_widget(Toolkit::Mark * location, Toolkit::Tag * field, 
				   Toolkit::Widget * widget) {
	LocalChange mask(this);

	NYI();
    }

    void
    Document::delete_field(Toolkit::Mark * location, Toolkit::Tag * field) {
	LocalChange mask(this);

	// Get the bounds of the field
	GtkTextIter iter_begin;
	locate_mark(location, iter_begin);
	GtkTextIter iter_end = iter_begin;
	locate_tag_end(field, iter_end);
	gtk_text_buffer_delete(mTextBuffer, & iter_begin, & iter_end);
    }

    void
    Document::delete_block(Toolkit::Mark * block_mark, Toolkit::Tag * block) {
	LocalChange mask(this);

	// Get the bounds of the field
	GtkTextIter iter_begin;
	locate_mark(block_mark, iter_begin);
	GtkTextIter iter_end = iter_begin;
	locate_tag_end(block, iter_end);
	gtk_text_buffer_delete(mTextBuffer, & iter_begin, & iter_end);

	GtkTextMark * mark = get_gtk_mark(block_mark);
	gtk_text_buffer_delete_mark(mTextBuffer, mark);
    }



    //================================================================================

    void	
    Document::clear() {
	LocalChange mask(this);

	// Clear the buffer
	GtkTextIter iter_start, iter_end;
	gtk_text_buffer_get_bounds(mTextBuffer, & iter_start, & iter_end);
	gtk_text_buffer_delete(mTextBuffer, & iter_start, & iter_end);

	// Check if there is a block mark at the start of the field
	gtk_text_buffer_get_bounds(mTextBuffer, & iter_start, & iter_end);
	GSList * mark_list = gtk_text_iter_get_marks(& iter_start);
	while (mark_list) {
			
	    // See if there is a Toolkit::Mark attached to the mark
	    GtkTextMark * mark = GTK_TEXT_MARK(mark_list->data);
	    Toolkit::Mark * tkm = static_cast<Toolkit::Mark *>
		(g_object_get_data(G_OBJECT(mark), "this"));
	    if (tkm && tkm->is_block_mark()) {
		//MESSAGE("\tClear Mark " << gtk_text_mark_get_name(mark));
		gtk_text_buffer_delete_mark(mTextBuffer, mark);
	    }
	    mark_list = g_slist_next(mark_list);
	}
	g_slist_free(mark_list);
    }


    //================================================================================
    //================================================================================

    /*
      void verify_layout(Toolkit::Mark * location, const list<Toolkit::Tag *> & layout) {

      // The first tag in the layout is the block
      list<Toolkit::Tag *>::const_iterator iter_layout = layout.begin();
      TRACE(2, "Verify layout of Block " << (* iter_layout)->get_name());
      GtkTextIter block_begin;
      locate_mark(location, block_begin);
      GtkTextIter block_end = block_begin;
      locate_tag_end(* iter_layout++, block_end);

      // Step through each tag, till we find the one we want
      // Each tag *must* start where the other leaves off
      // otherwise we assume that it is not present
      GtkTextIter field_begin = block_begin;
      GtkTextIter field_end;
      while (iter_layout != layout.end()) {
      
      // Test if it toggles on here
      GtkTextTag * tag = get_gtk_tag(* iter_layout);
      if (gtk_text_iter_begins_tag(& field_begin, tag)) {

      // Now locate the end of the field
      TRACE(2, "  Field " << (* iter_layout)->get_name() << " is present");
      field_end = field_begin;
      assert(gtk_text_iter_forward_to_tag_toggle(& field_end, tag));
      }
      else {

      // Assume tag is not present
      TRACE(2, "  Field " << (* iter_layout)->get_name() << " is missing");
      field_end = field_begin;	
      }

      // End of current field will be start of next
      field_begin = field_end;
      iter_layout++;
      }

      // Now check that we are at the end of the block
      if (gtk_text_iter_compare(& field_end, & block_end) == 0) {
      TRACE(2, "Verify complete");
      }
      else {
      TRACE(2, "Verify failed (" << gtk_text_iter_get_offset(& field_end)
      << " ," << gtk_text_iter_get_offset(& block_end) << ")");
      }
      }
    */

    //================================================================================
    //================================================================================

    GtkTextMark *
    Document::allocate_temp_mark(GtkTextIter * iter) {

	if (mTemporaryMarks.empty()) {

	    // Allocate a new mark
	    const char * name = temporary_mark_name();
	    GtkTextMark * gtkMark = gtk_text_buffer_create_mark(mTextBuffer, name, iter, 1);
	    return gtkMark;
	}
	else {
      
	    // Grab a used on out of the rank
	    GtkTextMark * gtkMark = mTemporaryMarks.front();
	    mTemporaryMarks.pop_front();
	    gtk_text_buffer_move_mark(mTextBuffer, gtkMark, iter);
	    return gtkMark;
	}
    }


    void
    Document::release_temp_mark(GtkTextMark * mark) {

	// Return the used mark for reuse
	mTemporaryMarks.push_back(mark);
    }


    //================================================================================

    /*
    // This method is called if a change occurs within any Tag sensitive to changes
    virtual void changed(Object * object,
    Toolkit::Mark * block_mark, Toolkit::Tag * block, 
    Toolkit::Mark * field_mark, Toolkit::Tag * field) = 0;

    // This method is called to sync changes made in document with model
    // It is called just once to apply the changes when cursor moves out of changed area
    virtual void apply_change(Object * object,
    Toolkit::Mark * block_mark, Toolkit::Tag * block, 
    Toolkit::Mark * field_mark, Toolkit::Tag * field) = 0;
    */

    void
    Document::changed_check() {

	Toolkit::DocumentMaintainer * maintainer = get_maintainer();
	if (maintainer && ! mLocalChange) {
	    TRACE(1, "Textbuffer::changed");

	    // There is a minor problem, that when characters are typed where
	    // the cursor sits at the beginning of the line, those characters
	    // fall into the crack between tags.  Obviously the gravity of
	    // start and end tags is not the same.
	    GtkTextIter iter_i;
	    gtk_text_buffer_get_iter_at_mark(mTextBuffer, & iter_i, mGtkTextMarkInsert);
	    if (gtk_text_iter_begins_tag(& iter_i, NULL)) {

		// There is a problem, the insert mark sits at the start of a tag
		// and start tags and block markers are right gravity.
		// The fix is to simply move them....
		GtkTextIter iter_l = iter_i;
		gtk_text_iter_backward_to_tag_toggle(& iter_l, NULL);
		//MESSAGE("relocating to " << gtk_text_iter_get_offset(& iter_l));

		// Get a list of the tags to be relocated
		GSList * tag_list_on = gtk_text_iter_get_toggled_tags(& iter_i, 1);
		while (tag_list_on) {
		    GtkTextTag * tag = GTK_TEXT_TAG(tag_list_on->data);
		    //MESSAGE("\tRelocating Tag " << tag->name);
		    gtk_text_buffer_apply_tag(mTextBuffer, tag, & iter_l, & iter_i);
		    tag_list_on = g_slist_next(tag_list_on);
		}
		g_slist_free(tag_list_on);

		GSList * mark_list = gtk_text_iter_get_marks(& iter_i);
		while (mark_list) {
		    GtkTextMark * mark = GTK_TEXT_MARK(mark_list->data);
		    //MESSAGE("\tRelocating Mark" << gtk_text_mark_get_name(mark_list);
		    Toolkit::Mark * tkm = static_cast<Toolkit::Mark *>
			    (g_object_get_data(G_OBJECT(mark_list->data), "this"));
		    if (tkm && tkm->is_block_mark()) {
			//MESSAGE("Found block mark " << tkm->get_name());
			gtk_text_buffer_move_mark(mTextBuffer, mark, & iter_l);
		    }
		    mark_list = g_slist_next(mark_list);
		}
		g_slist_free(mark_list);
	    }


	    // Check if we have already localisaed the change
	    if (mHaveChange) {


	    }
	    else {

		// Find the most restrictive enclosing tag, that must be the field
		// - first find the start, this should be any tag at or before the change
		GtkTextIter iter_begin;
		gtk_text_buffer_get_iter_at_mark(mTextBuffer, & iter_begin, 
						 mGtkTextMarkInsert);
		//MESSAGE("Insert point at " << gtk_text_iter_get_offset(& iter_begin));

		// Check if we are at the start of a tag
		if (! gtk_text_iter_begins_tag(& iter_begin, NULL)) {
		    // Move back to where there is a tag begin
		    gtk_text_iter_backward_to_tag_toggle(& iter_begin, NULL);
		}
		//dump_tags(iter_begin);
		//list_tags(mTextBuffer);

		// - for check each start tag, to find nearest end
		Toolkit::Tag * field = 0;
		GtkTextIter iter_end;
		gtk_text_buffer_get_end_iter(mTextBuffer, & iter_end);
		GSList * tag_list_on = gtk_text_iter_get_toggled_tags(& iter_begin, 1);
		//MESSAGE("Field begin at " << gtk_text_iter_get_offset(& iter_begin));
		while (tag_list_on) {
		    //MESSAGE("\tTAG " << GTK_TEXT_TAG(tag_list_on->data)->name);
		    Toolkit::Tag * tkt = static_cast<Toolkit::Tag *>
			(g_object_get_data(G_OBJECT(tag_list_on->data), "this"));
		    if (tkt && tkt->is_field()) {
			//MESSAGE("  Found field tag " << tkt->get_name());
			GtkTextIter iter = iter_begin;
			gtk_text_iter_forward_to_tag_toggle(
			    & iter, GTK_TEXT_TAG(tag_list_on->data));
			//MESSAGE("  Possible end at " << gtk_text_iter_get_offset(& iter));
			if (gtk_text_iter_compare(& iter, & iter_end) <= 0) {
			    //MESSAGE("  I like that");
			    iter_end = iter;
			    field = tkt;
			}
		    }
		    tag_list_on = g_slist_next(tag_list_on);
		}
		g_slist_free(tag_list_on);


		// Some sanity checking
		if (field == 0) {
		    ERROR("Could not locate field at change position");
		    return;
		}


		gtk_text_buffer_move_mark(mTextBuffer, mGtkTextMarkChangeBegin, & iter_begin);
		gtk_text_buffer_move_mark(mTextBuffer, mGtkTextMarkChangeEnd, & iter_end);
		//MESSAGE("Change field " << field->get_name() << " is " << gtk_text_iter_get_offset(& iter_begin) << ' ' << gtk_text_iter_get_offset(& iter_end));


		// Highlight the changed area - to show how clever we is
		gtk_text_buffer_apply_tag_by_name(mTextBuffer, "Changed-OK", 
						  & iter_begin, & iter_end);


		// Search back for the block mark of our field
		GtkTextIter iter_block = iter_begin;
		Toolkit::Tag * block = 0;
		Toolkit::Mark * block_mark = 0;
		int level = field->get_level();
		for (;;) {
		    //MESSAGE("Checking for mark at " << gtk_text_iter_get_offset(& iter_block));
	  
		    // Check if there is a block mark at the start of the field
		    GSList * mark_list = gtk_text_iter_get_marks(& iter_block);
		    while (mark_list) {
			//MESSAGE("\tMARK " << gtk_text_mark_get_name(GTK_TEXT_MARK(mark_list->data)));
			// See if there is a Toolkit::Mark attached to the mark
			Toolkit::Mark * tkm = static_cast<Toolkit::Mark *>
			    (g_object_get_data(G_OBJECT(mark_list->data), "this"));
			if (tkm && tkm->is_block_mark() &&
			    (tkm->get_associated_level() == level)) {
			    //MESSAGE("Found block mark " << tkm->get_name());
			    
			    block_mark = tkm;
			    break;
			}
			mark_list = g_slist_next(mark_list);
		    }
		    g_slist_free(mark_list);

		    // If we found the block mark, identify the block tag
		    if (block_mark) {
			//MESSAGE("Block mark found at " << gtk_text_iter_get_offset(& iter_block));

			// Verify that the block associated with the mark is in fact applied
			GTK::Tag * tkt = dynamic_cast<GTK::Tag *>
			    (block_mark->get_associated_block());
			if (tkt) {
			    GtkTextTag * tag = tkt->get_gtktag();
			    if (gtk_text_iter_has_tag(& iter_block, tag))
				block = tkt;
			}
			break;
		    }

		    // Seek back to the next place tags are toggled on, 
		    // but skip any owned blocks
		    // - for check each end tag, to find furtherest start
		    GtkTextIter iter_begin = iter_block;
		    GSList * tag_list_off = gtk_text_iter_get_toggled_tags(& iter_block, 0);
		    while (tag_list_off) {
			//MESSAGE("\tTAG " << GTK_TEXT_TAG(tag_list_off->data)->name);
			GtkTextIter iter = iter_block;
			gtk_text_iter_backward_to_tag_toggle(& iter, 
							     GTK_TEXT_TAG(tag_list_off->data));
			if (gtk_text_iter_compare(& iter, & iter_begin) < 0)
			    iter_begin = iter;
			tag_list_off = g_slist_next(tag_list_off);
		    }
		    g_slist_free(tag_list_off);

		    // If we did not find a new place, terminate -- we are lost
		    if (gtk_text_iter_compare(& iter_block, & iter_begin) == 0) {
			ERROR("Could not seek to find the start of block");
			return;
		    }
		    iter_block = iter_begin;
		}

		// Sanity checking
		if (block == 0 || block_mark == 0) {
		    ERROR("Could not locate block at change position");
		    return;
		}

		// If all is well, accept the change
		mHaveChange = true;
		mChangeField = field;
		mChangeBlock = block;
		mMarkBlock = block_mark;		
	    }

	    // Test if the field or the block is sensitive to change
	    if (mHaveChange && mChangeField && mChangeBlock &&
		(mChangeField->is_sensitive_change() || mChangeBlock->is_sensitive_change())) {

		// Now call the maintainer
		maintainer->changed(mMarkBlock->get_associated_object(),
				    mMarkBlock, mChangeBlock,
				    mMarkChangeBegin, mMarkChangeEnd, mChangeField);
	    }
	}
    }

    void
    Document::changes_sync() {

	Toolkit::DocumentMaintainer * maintainer = get_maintainer();
	if (maintainer) {

	    // If there are changes present
	    if (mHaveChange) {

		// Test that we are still within the change area
		GtkTextIter insert, change_begin, change_end;
		gtk_text_buffer_get_iter_at_mark(mTextBuffer, & insert, mGtkTextMarkInsert);
		gtk_text_buffer_get_iter_at_mark(mTextBuffer, & change_begin, 
						 mGtkTextMarkChangeBegin);
		gtk_text_buffer_get_iter_at_mark(mTextBuffer, & change_end,   
						 mGtkTextMarkChangeEnd);
		if (! gtk_text_iter_in_range(& insert, & change_begin, & change_end)) {


		    // Highlight the changed area - to show how clever we is
		    gtk_text_buffer_remove_tag_by_name(mTextBuffer, "Changed-OK", 
						       & change_begin, & change_end);

		    // Now call the maintainer
		    maintainer->apply_change(mMarkBlock->get_associated_object(),
					     mMarkBlock, mChangeBlock,
					     mMarkChangeBegin, mMarkChangeEnd, mChangeField);

		    // Clear the change tag
		    mHaveChange = false;
		    
		}
	    }
	}
    }


    //================================================================================

    /*
      bool
      Window::configure_event(const GdkEventConfigure * event) {
      TRACE(10, "");

      mSize.set(event->width >> mTest, event->height >> mTest);

      return true;
      }

      bool
      Window::motion_notify_event(const GdkEventMotion * event) {
      TRACE(10, "");

      bool dragged = mDownButton != 0;
      bool handled;

      if(dragged) {
      Toolkit::MouseEvent mouse_event(Toolkit::MET_DRAGGED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time);

      handled = mouse_dragged(&mouse_event);
      }
      else {
      Toolkit::MouseEvent mouse_event(Toolkit::MET_MOVED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time);

      handled = mouse_moved(&mouse_event);
      }

      return handled;
      }

      bool
      Window::enter_notify_event(const GdkEventCrossing * event) {
      TRACE(10, "");

      gtk_widget_grab_focus(mBaseWidget);		// FIXME TEMPORARY HACK

      bool dragged = mDownButton != 0;
      bool handled = false;

      if(mTimeoutId != 0) {
      gtk_timeout_remove(mTimeoutId);
      mTimeoutId = 0;
      }

      if(event->mode == GDK_CROSSING_NORMAL && !dragged) {
      Toolkit::MouseEvent mouse_event(Toolkit::MET_ENTERED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time);

      handled = mouse_entered(&mouse_event);
      }

      return handled;
      }

      bool
      Window::leave_notify_event(const GdkEventCrossing * event) {
      TRACE(10, "");

      bool dragged = mDownButton != 0;
      bool handled = false;

      if(dragged && mTimeoutId == 0) {
      #if 0
      // this is making things hard to debug for now (FIXME put it back in)
      mTimeoutId = gtk_timeout_add(SCROLL_TIMEOUT, &Window::static_timeout, this);
      #endif
      }

      switch(event->mode) {
      case GDK_CROSSING_NORMAL:
      if(!dragged) {
      Toolkit::MouseEvent mouse_event(Toolkit::MET_EXITED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time);

      handled = mouse_exited(&mouse_event);
      }
      break;
      case GDK_CROSSING_GRAB:
      // We might get here if a popup menu grabbed the cursor from us
      // We have to reset the mouse button
      mDownButton = 0;
      break;
      case GDK_CROSSING_UNGRAB:
      // we might get here if a popup menu returned the grab to us
      break;
      }

      return handled;
      }

      bool
      Window::button_press_event(const GdkEventButton * event) {
      TRACE(10, "");

      if(mDownButton == 0) {
      // FIXME I don't think grabs are necessary
      // because the cursor is grabbed anyway when the button 
      // is pressed
      //gtk_grab_add(mCanvas);

      mDownButton = event->button;

      Toolkit::MouseEvent mouse_event(Toolkit::MET_PRESSED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time,
      event->button,
      translate_presses(event->type));

      return mouse_pressed(&mouse_event);
      }
      else {
      return 0;
      }
      }

      bool
      Window::button_release_event(const GdkEventButton * event) {
      TRACE(10, "");

      if(mDownButton == event->button) {
      //gtk_grab_remove(mCanvas);

      if(mTimeoutId != 0) {
      gtk_timeout_remove(mTimeoutId);
      mTimeoutId = 0;
      }

      mDownButton = 0;

      Toolkit::MouseEvent mouse_event(Toolkit::MET_RELEASED,
      Drawing::Point(int(event->x + 0.5) >> mTest,
      int(event->y + 0.5) >> mTest),
      this,
      translate_modifiers(event->state),
      event->time,
      event->button,
      translate_presses(event->type));

      #if 0
      return mouse_released(&mouse_event);
      }
      else {
      return 0;
      }
      #else
      bool rval = mouse_released(&mouse_event);

      TRACE(10, "");

      return rval;
      }
      else {
      TRACE(10, "");
      return 0;
      }
      #endif
      }

      bool
      Textview::key_press_event(const GdkEventKey * event) {
      TRACE(10, "");

      Toolkit::KeyEvent key_event(Toolkit::KET_PRESSED,
      event->keyval,
      this,
      translate_modifiers(event->state),
      event->time);

      return key_pressed(&key_event);
      }

      bool
      Textview::key_release_event(const GdkEventKey * event) {
      TRACE(10, "");

      Toolkit::KeyEvent key_event(Toolkit::KET_RELEASED,
      event->keyval,
      this,
      translate_modifiers(event->state),
      event->time);

      return key_released(&key_event);
      }

      bool
      Textview::timeout() {
      GtkScrolledWindow * scrolled_window = GTK_SCROLLED_WINDOW(mBaseWidget);

      GtkAdjustment * hadjustment = gtk_scrolled_window_get_hadjustment(scrolled_window);
      GtkAdjustment * vadjustment = gtk_scrolled_window_get_vadjustment(scrolled_window);

      ASSERT(hadjustment);
      ASSERT(vadjustment);

      gdouble old_hvalue = gtk_adjustment_get_value(hadjustment);
      gdouble old_vvalue = gtk_adjustment_get_value(vadjustment);

      GtkWidget * viewport = GTK_BIN(mBaseWidget)->child;
      int w = viewport->allocation.width;
      int h = viewport->allocation.height;

      int x, y;
      GdkModifierType mask;

      gdk_window_get_pointer(viewport->window, &x, &y, &mask);

      int hoffset = x < 0 ? x / 3 : x > w ? (x - w) / 3 : 0;
      int voffset = y < 0 ? y / 3 : y > h ? (y - h) / 3 : 0;

      gdouble new_hvalue = CLAMP(old_hvalue + hoffset,
      hadjustment->lower, 
      hadjustment->upper - hadjustment->page_size);

      gdouble new_vvalue = CLAMP(old_vvalue + voffset,
      vadjustment->lower, 
      vadjustment->upper - vadjustment->page_size);

      bool need_event = false;

      if(old_hvalue != new_hvalue) {
      gtk_adjustment_set_value(hadjustment, new_hvalue);
      need_event = true;
      }

      if(old_vvalue != new_vvalue) {
      gtk_adjustment_set_value(vadjustment, new_vvalue);
      need_event = true;
      }

      if(need_event) {
      // create a dummy event for the apparent motion caused
      // by the viewport changing

      GdkEventMotion event;

      memset(&event, 0, sizeof(event));

      gdk_window_get_pointer(mCanvas->window, &x, &y, &mask);

      event.type = GDK_MOTION_NOTIFY;
      event.send_event = TRUE;
      event.window = mCanvas->window;
      event.x = x;
      event.y = y;
      event.time = 0;
      event.state = mask;
      event.is_hint = 0;

      motion_notify_event(&event);
      }

      return true;
      }

      gint
      Textview::static_expose_event(GtkWidget * widget, GdkEventExpose * event,
      Window * window) {
      return textview->expose_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_configure_event(GtkWidget * widget, GdkEventConfigure * event,
      Textview * textview) {
      return textview->configure_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_motion_notify_event(GtkWidget * widget, GdkEventMotion * event,
      Textview * textview) {
      return textview->motion_notify_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_enter_notify_event(GtkWidget * widget, GdkEventCrossing * event,
      Textview * textview) {
      return textview->enter_notify_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_leave_notify_event(GtkWidget * widget, GdkEventCrossing * event,
      Textview * textview) {
      return textview->leave_notify_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_button_press_event(GtkWidget * widget, GdkEventButton * event,
      Textview * textview) {
      return textview->button_press_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_button_release_event(GtkWidget * widget, GdkEventButton * event,
      Textview * textview) {
      return textview->button_release_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_key_press_event(GtkWidget * widget, GdkEventKey * event,
      Textview * textview) {
      return textview->key_press_event(event) ? TRUE : FALSE;
      }

      gint
      Textview::static_key_release_event(GtkWidget * widget, GdkEventKey * event,
      Textview * textview) {
      return textview->key_release_event(event) ? TRUE : FALSE;
      }

      gboolean
      Textview::static_timeout(gpointer data) {
      Textview * textview = static_cast<Textview *>(data);
      return textview->timeout();
      }
    */
} // namespace GTK
