// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "file_selection.h"

#include "foundation/macros.h"

#include <gtk/gtkmain.h>
#include <gtk/gtkfilesel.h>

#include <list>
using namespace std;

namespace GTK {

    FileSelection::FileSelection(const string & title,
				 const string & path) : Toolkit::FileSelection(), Widget() {
	mBaseWidget = gtk_file_selection_new(title.c_str());
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(mBaseWidget), path.c_str());

	post_init();
    }

    FileSelection::~FileSelection() {
    }

    Toolkit::ResponseKind
    FileSelection::go(string & result) {
	gtk_window_set_modal(GTK_WINDOW(mBaseWidget), TRUE);

	gint response = gtk_dialog_run(GTK_DIALOG(mBaseWidget));

	if(response == GTK_RESPONSE_OK) {
	    gchar ** selection = gtk_file_selection_get_selections(GTK_FILE_SELECTION(mBaseWidget));

	    list<string> selected_files;

	    for(int i = 0; selection[i]; ++i) {
		selected_files.push_back(selection[i]);
	    }

	    g_strfreev(selection);

	    //

	    ASSERT(!selected_files.empty());

	    result = selected_files.front();

	    return Toolkit::RK_OK;
	}
	else if(response == GTK_RESPONSE_CANCEL) {
	    return Toolkit::RK_CANCEL;
	}
	else {
	    ASSERT_NOT_REACHED();
	}
    }

} // namespace GTK
