// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GTK_MENU_H
#define GTK_MENU_H

#include "toolkit/menu.h"
#include "widget.h"

namespace GTK {

    class Menu : public Toolkit::Menu, public Widget {
    public:
	explicit Menu(bool bar = false);
	~Menu();

	void popup(const Toolkit::MouseEvent * mouse_event);

	void add_push(Command * command, const string & label, bool sensitive = true);
	void add_check(Command * command, const string & label, bool sensitive = true);
	void add_radio(Command * command, const string & label, Command * group,
		       bool sensitive = true);
	void add_submenu(Toolkit::Menu * submenu, const string & label);
	void add_separator();

	void set_sensitive(Command * command, bool sensitive);
	void set_state(Command * command, bool state);	// valid for radio and check buttons

    protected:
	GtkWidget * find(const Command * command);

    private:
	static void static_destroy_callback(GtkWidget * widget, Menu * menu);
	void destroy_callback(GtkWidget * widget);

	static void static_menu_callback(GtkWidget * widget, Menu * menu);
	void menu_callback(GtkWidget * widget);

	static void static_unref_command(gpointer data);
    };

} // namespace GTK

#endif // GTK_MENU_H
