// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GTK_TREE_H
#define GTK_TREE_H

#include "toolkit/tree.h"
#include "widget.h"

#include <glib/ghash.h>

#include <gtk/gtktreestore.h>
#include <gtk/gtktreeselection.h>
#include <gtk/gtkcellrenderertext.h>

namespace GTK {

    class Tree : public Toolkit::Tree, public Widget {
    public:
	Tree();
	~Tree();

	void add(Object * node, Drawing::Image * image, const string & label, bool expanded,
		 Object * parent, Object * position = 0);

	void remove(Object * node);
	void clear();

	Object * get_root();
	Object * get_parent(Object * node);

	bool contains(Object * node) const;

	void set_image(Object * node, Drawing::Image * image);
	void set_label(Object * node, const string & label);

	Object * point_to_node(const Drawing::Point & point);

    protected:
	//

	void hash_insert(Object * object, GtkTreeIter * iter);
	void hash_remove(Object * object);

	GtkTreeIter * object_to_iter(Object * object);
	Object * iter_to_object(GtkTreeIter * iter);

	//

	void selection_changed(GtkTreeSelection * selection);

	gboolean is_draggable(GdkDragContext * drag_context, GtkTreePath * path);
	gboolean is_droppable(GdkDragContext * drag_context,
			      GtkTreePath * path, GtkTreeViewDropPosition * position);

	void edited(GtkCellRendererText * text_renderer,
		    const char * path_string, const char * text);

	//

	enum {
	    PIXBUF_COLUMN,
	    TEXT_COLUMN,
	    NODE_COLUMN,
	    EDITABLE_COLUMN,
	    NUM_COLUMNS
	};

    private:
	bool motion_notify_event(const GdkEventMotion * event);
	bool enter_notify_event(const GdkEventCrossing * event);
	bool leave_notify_event(const GdkEventCrossing * event);
	bool button_press_event(const GdkEventButton * event);
	bool button_release_event(const GdkEventButton * event);

	static void hash_key_destroy(gpointer key);
	static void hash_value_destroy(gpointer value);

	static void static_selection_changed(GtkTreeSelection * selection,
					    Tree * tree);

	static gboolean static_is_draggable(GtkTreeView * tree_view,
					    GdkDragContext * drag_context,
					    GtkTreePath * path,
					    gpointer user_data);

	static gboolean static_is_droppable(GtkTreeView * tree_view,
					    GdkDragContext * drag_context,
					    GtkTreePath * path,
					    GtkTreeViewDropPosition * position,
					    gpointer user_data);

	static void static_edited(GtkCellRendererText * text_renderer,
				  const char * path, const char * text, Tree * tree);

	static gint static_motion_notify_event(GtkWidget * widget, GdkEventMotion * event,
					       Tree * tree);
	static gint static_enter_notify_event(GtkWidget * widget, GdkEventCrossing * event,
					      Tree * tree);
	static gint static_leave_notify_event(GtkWidget * widget, GdkEventCrossing * event,
					      Tree * tree);
	static gint static_button_press_event(GtkWidget * widget, GdkEventButton * event,
					      Tree * tree);
	static gint static_button_release_event(GtkWidget * widget, GdkEventButton * event,
						Tree * tree);

	GtkWidget * mTreeView;
	GtkTreeStore * mTreeStore;

	GHashTable * mHashTable;
    };

} // namespace GTK

#endif // GTK_TREE_H
