// Dachshund
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MODULES_H
#define MODULES_H

#ifdef MCG_REQUIRED

#include <set>
#include <string>

#include "uml/model_management/package.h"
#include "uml/foundation/core/class.h"

#include "model_code_generator.h"

using namespace std;
using namespace UML;

//
// Dachshund UML-Model Code Generator 
//

namespace DMCG {
	
	class ClassModule;

	class Module {
		private:
			MCG_Language mLanguage;
			MCG_Operating_System mOperatingSystem;

		public:
			MCG_Language get_language (void)	{ return mLanguage; }
			void set_language (MCG_Language new_language)	{ mLanguage = new_language; }

			MCG_Operating_System get_operating_system (void)	{ return mOperatingSystem; }
			void set_operating_system (MCG_Operating_System new_operating_system)	{ mOperatingSystem = new_operating_system; }
	};

	class PackageModule : virtual public Module {
		private:
			Package * mPackage;

			PackageModule * mParent;

			set<ClassModule *> mClasses;

		public:
			PackageModule();

			Package * get_package (void)	{ return mPackage; }
			void set_package (Package * new_package)	{ mPackage = new_package; }

			PackageModule * get_parent_package_module (void)	{ return mParent; }
			void set_parent_package_module (PackageModule * new_package)	{ mParent = new_package; }

			void add_class (ClassModule * new_class)	{ mClasses.insert (new_class); }
	};

	class FileModule : virtual public Module { 
		private:
			static string mTargetDirectory;

			string mModuleDirectory;

			set<Module *> mDependents;
			set<string> mExternalDependents;

		public:
			FileModule();

			static string get_target_directory (void) { return mTargetDirectory; }
			static void set_target_directory (string new_directory) { mTargetDirectory = new_directory; }

			string get_module_directory (void) { return mModuleDirectory; }
			void set_module_directory (string new_directory) { mModuleDirectory = new_directory; }
	};

	class ClassModule : virtual public FileModule {
		private:
			UML::Class * mClass;
			PackageModule * mPackageModule;

		public:
			ClassModule();

			UML::Class * get_class (void) { return mClass; }
			void set_class (UML::Class * new_class) { mClass = new_class; }

			PackageModule * get_package_module (void) { return mPackageModule; }
			void set_package_module (PackageModule * new_package_module) { mPackageModule = new_package_module; }
	};

	class ComponentModule : virtual public FileModule {
		private:
//			Component * mComponent;
			PackageModule * mPackageModule;

		public:
			ComponentModule();

//			Component * get_component (void) { return mComponent; }
//			void set_component (Component * new_component) { mComponent = new_component; }

			PackageModule * get_package_module (void) { return mPackageModule; }
			void set_package_module (PackageModule * new_package_module) { mPackageModule = new_package_module; }
		};

}	// namespace DMCG

#endif // MCG_REQUIRED

#endif // MODULES_H

