// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "class.h"

#include "data_type.h"
#include "attribute.h"
#include "reference.h"
#include "operation.h"
#include "exception.h"
#include "constraint.h"
#include "constant.h"
#include "tag.h"
#include "model.h"

#include "foundation/commands.h"
#include "reflective/changes.h"

#include <typeinfo>
using namespace std;

namespace MOF {

    const string Class::TYPE = "Class";

    //

    Class::Class() : Classifier(),
        mSingleton(0) {
    }

    Class::~Class() {
    }

    void
    Class::set_singleton(const Boolean * singleton) {
	begin_transaction();

	//

	Boolean * old_value = mSingleton ? new Boolean(*mSingleton) : 0;
	Boolean * new_value = singleton ? new Boolean(*singleton) : 0;

	//

	Command * command = command_flip_argument(this, &Class::set_singleton,
						  new_value, old_value);
	log_command(command);
	command->unref();

	// make the changes

	if(mSingleton) {
	    mSingleton->unref();
	}

	mSingleton = new_value ? new Boolean(*new_value) : 0;

	// changes made

	ChangeEvent * change = change_attribute_changed(this, "singleton", new_value, old_value);
	log_change(change);
	change->unref();

	//

	if(new_value) {
	    new_value->unref();
	}

	if(old_value) {
	    old_value->unref();
	}

	//

	end_transaction();
    }

    bool
    Class::ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    return class_name == TYPE || Classifier::ref_is_a_type_of(class_name);
	}
	else {
	    return false;
	}
    }

    const string &
    Class::ref_get_type() const {
	return TYPE;
    }

    void
    Class::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_class(this);

	    Classifier::ref_implant_impl(visited);
	}
    }

    void
    Class::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_class(this);

	    Classifier::ref_uproot_impl(visited);
	}
    }

} // namespace MOF
