// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "classifier.h"

#include "typed_element.h"
#include "model.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <algorithm>
#include <functional>
using namespace std;

namespace MOF {

    const string Classifier::TYPE = "Classifier";

    //

    Classifier::Classifier() : GeneralizableElement() {
    }

    Classifier::~Classifier() {
    }

    void
    Classifier::internal_add_typed_element(TypedElement * typed_element) throw() {
	typed_element->ref();
	mTypedElements.insert(typed_element);
    }

    void
    Classifier::internal_remove_typed_element(TypedElement * typed_element) throw() {
	mTypedElements.erase(typed_element);
	typed_element->unref();
    }

    void
    Classifier::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    Classifier * null_classifier = 0;
	    set<TypedElement *> typed_elements_copy = mTypedElements;	// have to make a copy

	    for_each(typed_elements_copy.begin(), typed_elements_copy.end(),
		     bind2nd(mem_fun(&TypedElement::set_type), null_classifier));

	    GeneralizableElement::ref_delete_impl(visited);
	}
    }

    void
    Classifier::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_classifier(this);

	    GeneralizableElement::ref_implant_impl(visited);
	}
    }

    bool
    Classifier::ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    return class_name == TYPE || GeneralizableElement::ref_is_a_type_of(class_name);
	}
	else {
	    return false;
	}
    }

    const string &
    Classifier::ref_get_type() const {
	return TYPE;
    }

    void
    Classifier::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_classifier(this);

	    GeneralizableElement::ref_uproot_impl(visited);
	}
    }

} // namespace MOF
