// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "constant.h"

#include "model.h"

#include "foundation/commands.h"
#include "reflective/changes.h"

namespace MOF {

    const string Constant::TYPE = "Constant";

    //

    Constant::Constant() : TypedElement(),
        mValue(0) {
    }

    Constant::~Constant() {
    }

    void
    Constant::set_value(LiteralType * value) {
	begin_transaction();

	Command * command = command_flip_argument(this, &Constant::set_value, value, mValue);
	log_command(command);
	command->unref();

	// make the changes

	if(mValue) {
	    mValue->unref();
	}

	mValue = value;

	if(mValue) {
	    mValue->ref();
	}

	//

#if 0
	ChangeEvent * change = new ChangeRoleChanged(this, "value", value, mValue);
	log_change(change);
	change->unref();
#endif

	end_transaction();
    }

    bool
    Constant::ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const {
	return class_name == TYPE || TypedElement::ref_is_a_type_of(class_name);
    }

    const string &
    Constant::ref_get_type() const {
	return TYPE;
    }

    void
    Constant::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_constant(this);

	    TypedElement::ref_implant_impl(visited);
	}
    }

    void
    Constant::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_constant(this);

	    TypedElement::ref_uproot_impl(visited);
	}
    }

} // namespace MOF
