// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_CONSTRAINT_H
#define MOF_CONSTRAINT_H

#include "model_element.h"

#include "name.h"
#include "evaluation_kind.h"

#include <set>
using namespace std;

namespace MOF {

    using namespace Common;

    class Model;

    //

    class Constraint : public ModelElement {
    public:
	Constraint();
	~Constraint();

	Object * get_expression() { return mExpression; }
	void set_expression(Object * expression);

	const Name * get_language() const { return mLanguage; }
	void set_language(const Name * language);

	const EvaluationKind * get_evaluation_policy() const { return mEvaluationPolicy; }
	void set_evaluation_policy(const EvaluationKind * evaluation_policy);

	const set<ModelElement *> & get_constrained_elements() { return mConstrainedElements; }
	void set_constrained_elements(const set<ModelElement *> & constrained_elements);
	void add_constrained_element(ModelElement * constrained_element);
	void remove_constrained_element(ModelElement * constrained_element);
	void modify_constrained_element(ModelElement * old_constrained_element,
					ModelElement * new_constrained_element);

	const vector<const Name *> & get_test() const;
	void set_test(const vector<const Name *> & test);

	// internal

	void internal_add_constrained_element(ModelElement * constrained_element) throw();
	void internal_remove_constrained_element(ModelElement * constrained_element) throw();

    protected:
	// reflective

	const string & ref_get_type() const;
	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();

	void ref_delete_impl(set<string> & visited);

    private:
	Object * mExpression;
	Name * mLanguage;
	EvaluationKind * mEvaluationPolicy;
	set<ModelElement *> mConstrainedElements;

	vector<Name *> mTest;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_CONSTRAINT_H
