// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_MODEL_H
#define MOF_MODEL_H

#include "reflective/context.h"

#include <set>
using namespace std;

namespace MOF {

    class AliasType;
    class AssociationEnd;
    class Association;
    class Attribute;
    class BehavioralFeature;
    class Class;
    class Classifier;
    class CollectionType;
    class Constant;
    class Constraint;
    class DataType;
    class EnumerationType;
    class Exception;
    class Feature;
    class GeneralizableElement;
    class Import;
    class ModelElement;
    class Namespace;
    class Operation;
    class Package;
    class Parameter;
    class PrimitiveType;
    class Reference;
    class StructuralFeature;
    class StructureField;
    class StructureType;
    class Tag;
    class TypedElement;

    class Model : public REF::Context {
    public:
	Model();

	REF::Class * ref_create_class(const string & class_name);
	REF::Attribute * ref_create_attribute(const string & attribute_name);

	//

	const set<AliasType *> & get_all_of_type_alias_type() {
	    return mAllOfTypeAliasType;
	}

	const set<ModelElement *> & get_all_of_type_model_element() {
	    return mAllOfTypeModelElement;
	}

	const set<BehavioralFeature *> & get_all_of_type_behavioral_feature() {
	    return mAllOfTypeBehavioralFeature;
	}

	const set<Classifier *> & get_all_of_type_classifier() {
	    return mAllOfTypeClassifier;
	}

	const set<Feature *> & get_all_of_type_feature() {
	    return mAllOfTypeFeature;
	}

	const set<TypedElement *> & get_all_of_type_typed_element() {
	    return mAllOfTypeTypedElement;
	}

	const set<EnumerationType *> & get_all_of_type_enumeration_type() {
	    return mAllOfTypeEnumerationType;
	}

	const set<StructureType *> & get_all_of_type_structure_type() {
	    return mAllOfTypeStructureType;
	}

	const set<StructureField *> & get_all_of_type_structure_field() {
	    return mAllOfTypeStructureField;
	}

	const set<PrimitiveType *> & get_all_of_type_primitive_type() {
	    return mAllOfTypePrimitiveType;
	}

	const set<AssociationEnd *> & get_all_of_type_association_end() {
	    return mAllOfTypeAssociationEnd;
	}

	const set<Association *> & get_all_of_type_association() {
	    return mAllOfTypeAssociation;
	}

	const set<GeneralizableElement *> & get_all_of_type_generalizable_element() {
	    return mAllOfTypeGeneralizableElement;
	}

	const set<Namespace *> & get_all_of_type_namespace() {
	    return mAllOfTypeNamespace;
	}

	const set<StructuralFeature *> & get_all_of_type_structural_feature() {
	    return mAllOfTypeStructuralFeature;
	}

	const set<Attribute *> & get_all_of_type_attribute() {
	    return mAllOfTypeAttribute;
	}

	const set<Class *> & get_all_of_type_class() {
	    return mAllOfTypeClass;
	}

	const set<Constant *> & get_all_of_type_constant() {
	    return mAllOfTypeConstant;
	}

	const set<Constraint *> & get_all_of_type_constraint() {
	    return mAllOfTypeConstraint;
	}

	const set<DataType *> & get_all_of_type_data_type() {
	    return mAllOfTypeDataType;
	}

	const set<Exception *> & get_all_of_type_exception() {
	    return mAllOfTypeException;
	}

	const set<Import *> & get_all_of_type_import() {
	    return mAllOfTypeImport;
	}

	const set<Operation *> & get_all_of_type_operation() {
	    return mAllOfTypeOperation;
	}

	const set<Package *> & get_all_of_type_package() {
	    return mAllOfTypePackage;
	}

	const set<Parameter *> & get_all_of_type_parameter() {
	    return mAllOfTypeParameter;
	}

	const set<Reference *> & get_all_of_type_reference() {
	    return mAllOfTypeReference;
	}

	const set<Tag *> & get_all_of_type_tag() {
	    return mAllOfTypeTag;
	}

	const set<CollectionType *> & get_all_of_collection_type() {
	    return mAllOfTypeCollectionType;
	}

	// internal

	void internal_add_alias_type(AliasType * element);
	void internal_add_behavioral_feature(BehavioralFeature * element);
	void internal_add_classifier(Classifier * element);
	void internal_add_class(Class * element);
	void internal_add_enumeration_type(EnumerationType * element);
	void internal_add_primitive_type(PrimitiveType * element);
	void internal_add_structure_type(StructureType * element);
	void internal_add_structure_field(StructureField * element);
	void internal_add_collection_type(CollectionType * element);
	void internal_add_feature(Feature * element);
	void internal_add_typed_element(TypedElement * element);
	void internal_add_association_end(AssociationEnd * element);
	void internal_add_association(Association * element);
	void internal_add_generalizable_element(GeneralizableElement * element);
	void internal_add_model_element(ModelElement * element);
	void internal_add_namespace(Namespace * element);
	void internal_add_structural_feature(StructuralFeature * element);
	void internal_add_attribute(Attribute * element);
	void internal_add_constant(Constant * element);
	void internal_add_constraint(Constraint * element);
	void internal_add_data_type(DataType * element);
	void internal_add_exception(Exception * element);
	void internal_add_import(Import * element);
	void internal_add_operation(Operation * element);
	void internal_add_package(Package * element);
	void internal_add_parameter(Parameter * element);
	void internal_add_reference(Reference * element);
	void internal_add_tag(Tag * element);

	void internal_remove_alias_type(AliasType * element);
	void internal_remove_behavioral_feature(BehavioralFeature * element);
	void internal_remove_classifier(Classifier * element);
	void internal_remove_class(Class * element);
	void internal_remove_feature(Feature * element);
	void internal_remove_typed_element(TypedElement * element);
	void internal_remove_association_end(AssociationEnd * element);
	void internal_remove_association(Association * element);
	void internal_remove_generalizable_element(GeneralizableElement * element);
	void internal_remove_model_element(ModelElement * element);
	void internal_remove_namespace(Namespace * element);
	void internal_remove_structural_feature(StructuralFeature * element);
	void internal_remove_attribute(Attribute * element);
	void internal_remove_constant(Constant * element);
	void internal_remove_constraint(Constraint * element);
	void internal_remove_data_type(DataType * element);
	void internal_remove_exception(Exception * element);
	void internal_remove_import(Import * element);
	void internal_remove_operation(Operation * element);
	void internal_remove_package(Package * element);
	void internal_remove_parameter(Parameter * element);
	void internal_remove_reference(Reference * element);
	void internal_remove_tag(Tag * element);
	void internal_remove_enumeration_type(EnumerationType * element);
	void internal_remove_primitive_type(PrimitiveType * element);
	void internal_remove_structure_type(StructureType * element);
	void internal_remove_structure_field(StructureField * element);
	void internal_remove_collection_type(CollectionType * element);

    protected:
	~Model();

    private:
	set<PrimitiveType *> mAllOfTypePrimitiveType;
	set<StructureType *> mAllOfTypeStructureType;
	set<StructureField *> mAllOfTypeStructureField;
	set<EnumerationType *> mAllOfTypeEnumerationType;
	set<CollectionType *> mAllOfTypeCollectionType;
	set<ModelElement *> mAllOfTypeModelElement;
	set<BehavioralFeature *> mAllOfTypeBehavioralFeature;
	set<Classifier *> mAllOfTypeClassifier;
	set<Feature *> mAllOfTypeFeature;
	set<TypedElement *> mAllOfTypeTypedElement;
	set<AssociationEnd *> mAllOfTypeAssociationEnd;
	set<Association *> mAllOfTypeAssociation;
	set<GeneralizableElement *> mAllOfTypeGeneralizableElement;
	set<Namespace *> mAllOfTypeNamespace;
	set<StructuralFeature *> mAllOfTypeStructuralFeature;
	set<Attribute *> mAllOfTypeAttribute;
	set<Class *> mAllOfTypeClass;
	set<Constant *> mAllOfTypeConstant;
	set<Constraint *> mAllOfTypeConstraint;
	set<DataType *> mAllOfTypeDataType;
	set<Exception *> mAllOfTypeException;
	set<Import *> mAllOfTypeImport;
	set<Operation *> mAllOfTypeOperation;
	set<Package *> mAllOfTypePackage;
	set<Parameter *> mAllOfTypeParameter;
	set<Reference *> mAllOfTypeReference;
	set<Tag *> mAllOfTypeTag;
	set<AliasType *> mAllOfTypeAliasType;
    };

} // namespace MOF

#endif // MOF_MODEL_H
