// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_MODEL_HANDLER_H
#define MOF_MODEL_HANDLER_H

#include "xml/sax_handler.h"

#include "foundation/hash_functions.h"
#include "foundation/hash.h"

#include <string>
#include <set>
using namespace std;

namespace MOF {

    class ModelElement;

    using namespace XML;		// FIXME not sure if we want this

    class ModelHandler : public SAXHandler {
    public:
	ModelHandler();
	~ModelHandler();

	// the sax interface

	void warning(SAXParser * parser, const string & msg);
	void error(SAXParser * parser, const string & msg);
	void fatal(SAXParser * parser, const string & msg);

	void comment(SAXParser * parser, const string & comment);
	void start_document(SAXParser * parser);
	void end_document(SAXParser * parser);
	void start_element(SAXParser * parser, const string & name, const Properties & properties);
	void end_element(SAXParser * parser, const string & name);
	void characters(SAXParser * parser, const string & data);
	void reference(SAXParser * parser, const string & name);
	void cdata_block(SAXParser * parser, const string & data);

    private:
	void mof__namespace__contents__start(SAXParser * parser, const Properties & properties);
	void mof__namespace__contents__end(SAXParser * parser);

	void mof__feature__visibility__start(SAXParser * parser, const Properties & properties);
	void mof__feature__visibility__end(SAXParser * parser);

	void mof__feature__scope__start(SAXParser * parser, const Properties & properties);
	void mof__feature__scope__end(SAXParser * parser);

	void mof__feature__start(SAXParser * parser, const Properties & properties);
	void mof__feature__end(SAXParser * parser);

	void mof__classifier__start(SAXParser * parser, const Properties & properties);
	void mof__classifier__end(SAXParser * parser);

	void mof__namespace__start(SAXParser * parser, const Properties & properties);
	void mof__namespace__end(SAXParser * parser);

	void mof__structural_feature__multiplicity__start(SAXParser * parser,
							  const Properties & properties);
	void mof__structural_feature__multiplicity__end(SAXParser * parser);

	void mof__structural_feature__is_changeable__start(SAXParser * parser,
							   const Properties & properties);
	void mof__structural_feature__is_changeable__end(SAXParser * parser);

	void mof__structural_feature__start(SAXParser * parser, const Properties & properties);
	void mof__structural_feature__end(SAXParser * parser);

	void mof__attribute__is_derived__start(SAXParser * parser,
						   const Properties & properties);
	void mof__attribute__is_derived__end(SAXParser * parser);

	void mof__attribute__start(SAXParser * parser, const Properties & properties);
	void mof__attribute__end(SAXParser * parser);

	void mof__operation__is_query__start(SAXParser * parser, const Properties & properties);
	void mof__operation__is_query__end(SAXParser * parser);

	void mof__operation__exceptions__start(SAXParser * parser, const Properties & properties);
	void mof__operation__exceptions__end(SAXParser * parser);

	void mof__operation__start(SAXParser * parser, const Properties & properties);
	void mof__operation__end(SAXParser * parser);

	void mof__class__is_singleton__start(SAXParser * parser, const Properties & properties);
	void mof__class__is_singleton__end(SAXParser * parser);

	void mof__class__start(SAXParser * parser, const Properties & properties);
	void mof__class__end(SAXParser * parser);

	void mof__model_element__name__start(SAXParser * parser, const Properties & properties);
	void mof__model_element__name__end(SAXParser * parser);

	void mof__model_element__annotation__start(SAXParser * parser,
						   const Properties & properties);
	void mof__model_element__annotation__end(SAXParser * parser);

	void mof__model_element__container__start(SAXParser * parser,
						  const Properties & properties);
	void mof__model_element__container__end(SAXParser * parser);

	void mof__model_element__constraints__start(SAXParser * parser,
						    const Properties & properties);
	void mof__model_element__constraints__end(SAXParser * parser);

	void mof__model_element__start(SAXParser * parser, const Properties & properties);
	void mof__model_element__end(SAXParser * parser);

	void mof__package__start(SAXParser * parser, const Properties & properties);
	void mof__package__end(SAXParser * parser);

	void mof__constraint__expression__start(SAXParser * parser, const Properties & properties);
	void mof__constraint__expression__end(SAXParser * parser);

	void mof__constraint__language__start(SAXParser * parser, const Properties & properties);
	void mof__constraint__language__end(SAXParser * parser);

	void mof__constraint__evaluation_policy__start(SAXParser * parser,
						       const Properties & properties);
	void mof__constraint__evaluation_policy__end(SAXParser * parser);

	void mof__constraint__constrained_elements__start(SAXParser * parser,
							  const Properties & properties);
	void mof__constraint__constrained_elements__end(SAXParser * parser);

	void mof__constraint__start(SAXParser * parser, const Properties & properties);
	void mof__constraint__end(SAXParser * parser);

	void mof__association__is_derived__start(SAXParser * parser, const Properties & properties);
	void mof__association__is_derived__end(SAXParser * parser);

	void mof__association__start(SAXParser * parser, const Properties & properties);
	void mof__association__end(SAXParser * parser);

	void mof__parameter__direction__start(SAXParser * parser, const Properties & properties);
	void mof__parameter__direction__end(SAXParser * parser);

	void mof__parameter__multiplicity__start(SAXParser * parser, const Properties & properties);
	void mof__parameter__multiplicity__end(SAXParser * parser);

	void mof__parameter__start(SAXParser * parser, const Properties & properties);
	void mof__parameter__end(SAXParser * parser);

	void mof__behavioral_feature__start(SAXParser * parser, const Properties & properties);
	void mof__behavioral_feature__end(SAXParser * parser);

	void mof__import__visibility__start(SAXParser * parser, const Properties & properties);
	void mof__import__visibility__end(SAXParser * parser);

	void mof__import__imported_namespace__start(SAXParser * parser,
						    const Properties & properties);
	void mof__import__imported_namespace__end(SAXParser * parser);

	void mof__import__start(SAXParser * parser, const Properties & properties);
	void mof__import__end(SAXParser * parser);

	void mof__generalizable_element__visibility__start(SAXParser * parser,
							   const Properties & properties);
	void mof__generalizable_element__visibility__end(SAXParser * parser);

	void mof__generalizable_element__is_abstract__start(SAXParser * parser,
							    const Properties & properties);
	void mof__generalizable_element__is_abstract__end(SAXParser * parser);

	void mof__generalizable_element__is_root__start(SAXParser * parser,
							const Properties & properties);
	void mof__generalizable_element__is_root__end(SAXParser * parser);

	void mof__generalizable_element__is_leaf__start(SAXParser * parser,
							const Properties & properties);
	void mof__generalizable_element__is_leaf__end(SAXParser * parser);

	void mof__generalizable_element__supertypes__start(SAXParser * parser,
							   const Properties & properties);
	void mof__generalizable_element__supertypes__end(SAXParser * parser);

	void mof__generalizable_element__start(SAXParser * parser, const Properties & properties);
	void mof__generalizable_element__end(SAXParser * parser);

	void mof__constant__const_value__start(SAXParser * parser, const Properties & properties);
	void mof__constant__const_value__end(SAXParser * parser);

	void mof__constant__start(SAXParser * parser, const Properties & properties);
	void mof__constant__end(SAXParser * parser);

	void mof__reference__referenced_end__start(SAXParser * parser,
						   const Properties & properties);
	void mof__reference__referenced_end__end(SAXParser * parser);

	void mof__reference__start(SAXParser * parser, const Properties & properties);
	void mof__reference__end(SAXParser * parser);

	void mof__mof_exception__start(SAXParser * parser, const Properties & properties);
	void mof__mof_exception__end(SAXParser * parser);

	void mof__data_type__type_code__start(SAXParser * parser, const Properties & properties);
	void mof__data_type__type_code__end(SAXParser * parser);

	void mof__data_type__start(SAXParser * parser, const Properties & properties);
	void mof__data_type__end(SAXParser * parser);

	void mof__association_end__multiplicity__start(SAXParser * parser,
						       const Properties & properties);
	void mof__association_end__multiplicity__end(SAXParser * parser);

	void mof__association_end__aggregation__start(SAXParser * parser,
						      const Properties & properties);
	void mof__association_end__aggregation__end(SAXParser * parser);

	void mof__association_end__is_navigable__start(SAXParser * parser,
						       const Properties & properties);
	void mof__association_end__is_navigable__end(SAXParser * parser);

	void mof__association_end__is_changeable__start(SAXParser * parser,
							const Properties & properties);
	void mof__association_end__is_changeable__end(SAXParser * parser);

	void mof__association_end__other_end__start(SAXParser * parser,
						    const Properties & properties);
	void mof__association_end__other_end__end(SAXParser * parser);

	void mof__association_end__start(SAXParser * parser, const Properties & properties);
	void mof__association_end__end(SAXParser * parser);

	void mof__typed_element__type__start(SAXParser * parser, const Properties & properties);
	void mof__typed_element__type__end(SAXParser * parser);

	void mof__typed_element__start(SAXParser * parser, const Properties & properties);
	void mof__typed_element__end(SAXParser * parser);

	void mof__multiplicity__lower__start(SAXParser * parser, const Properties & properties);
	void mof__multiplicity__lower__end(SAXParser * parser);

	void mof__multiplicity__upper__start(SAXParser * parser, const Properties & properties);
	void mof__multiplicity__upper__end(SAXParser * parser);

	void mof__multiplicity__is_ordered__start(SAXParser * parser,
						  const Properties & properties);
	void mof__multiplicity__is_ordered__end(SAXParser * parser);

	void mof__multiplicity__is_unique__start(SAXParser * parser, const Properties & properties);
	void mof__multiplicity__is_unique__end(SAXParser * parser);

	void mof__multiplicity__start(SAXParser * parser, const Properties & properties);
	void mof__multiplicity__end(SAXParser * parser);

	void mof__tag__tag_id__start(SAXParser * parser, const Properties & properties);
	void mof__tag__tag_id__end(SAXParser * parser);

	void mof__tag__values__start(SAXParser * parser, const Properties & properties);
	void mof__tag__values__end(SAXParser * parser);

	void mof__tag__elements__start(SAXParser * parser, const Properties & properties);
	void mof__tag__elements__end(SAXParser * parser);

	void mof__tag__start(SAXParser * parser, const Properties & properties);
	void mof__tag__end(SAXParser * parser);

	void mof__mof__start(SAXParser * parser, const Properties & properties);
	void mof__mof__end(SAXParser * parser);

	void mof__model__start(SAXParser * parser, const Properties & properties);
	void mof__model__end(SAXParser * parser);

	// data types

	void mof__alias_type__start(SAXParser * parser, const Properties & properties);
	void mof__alias_type__end(SAXParser * parser);

	void mof__primitive_type__start(SAXParser * parser, const Properties & properties);
	void mof__primitive_type__end(SAXParser * parser);

	void mof__structure_type__start(SAXParser * parser, const Properties & properties);
	void mof__structure_type__end(SAXParser * parser);

	void mof__collection_type__start(SAXParser * parser, const Properties & properties);
	void mof__collection_type__end(SAXParser * parser);

	void mof__enumeration_type__start(SAXParser * parser, const Properties & properties);
	void mof__enumeration_type__end(SAXParser * parser);

	void mof__enumeration_type__labels__start(SAXParser * parser, const Properties & properties);
	void mof__enumeration_type__labels__end(SAXParser * parser);

	void mof__structure_field__start(SAXParser * parser, const Properties & properties);
	void mof__structure_field__end(SAXParser * parser);

	//
	//
	//

	string mCharacters;

	typedef void (ModelHandler::*StartHandler)(SAXParser *, const Properties &);
	typedef void (ModelHandler::*EndHandler)(SAXParser *);
	typedef pair<StartHandler, EndHandler> HandlerPair;

	hash_map<string, HandlerPair> mHandlerMap;
	hash_map<string, Object *> mElementMap;

	set<string> mUnhandledElements;

	void store(const string & id, Object * object);
	Object * lookup(const string & id);
    };

} // namespace MOF

#endif // MOF_MODEL_HANDLER_H
