// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "multiplicity_type.h"

#include "foundation/macros.h"

#include <iostream>
using namespace std;

namespace MOF {

    const string MultiplicityType::TYPE = "MultiplicityType";

    MultiplicityType::MultiplicityType() : REF::Attribute(),
        mLower(new Integer()),
	mUpper(new Integer()),
	mOrdered(new Boolean()),
	mUnique(new Boolean()) {
    }

    MultiplicityType::MultiplicityType(const MultiplicityType & mt) : REF::Attribute(),
        mLower(new Integer(*mt.mLower)),
        mUpper(new Integer(*mt.mUpper)),
        mOrdered(new Boolean(*mt.mOrdered)),
        mUnique(new Boolean(*mt.mUnique)) {
    }

    MultiplicityType::~MultiplicityType() {
	mUnique->unref();
	mOrdered->unref();
	mUpper->unref();
	mLower->unref();
    }

    void
    MultiplicityType::ref_set_value(const string & value) {
	NYI();
    }

    void
    MultiplicityType::ref_get_value(string & value) const {
	value += "{ ";
	mLower->ref_get_value(value);
	value += ", ";
	mUpper->ref_get_value(value);
	value += ", ";
	mOrdered->ref_get_value(value);
	value += ", ";
	mUnique->ref_get_value(value);
	value += " };";
    }

    const string &
    MultiplicityType::ref_get_type() const {
	return TYPE;
    }

    void
    MultiplicityType::set_lower(const Integer * lower) {
	RETURN_IF_FAIL(lower);

	mLower->unref();
	mLower = new Integer(*lower);
    }

    void
    MultiplicityType::set_upper(const Integer * upper) {
	RETURN_IF_FAIL(upper);

	mUpper->unref();
	mUpper = new Integer(*upper);
    }

    void
    MultiplicityType::set_ordered(const Boolean * ordered) {
	RETURN_IF_FAIL(ordered);

	mOrdered->unref();
	mOrdered = new Boolean(*ordered);
    }

    void
    MultiplicityType::set_unique(const Boolean * unique) {
	RETURN_IF_FAIL(unique);

	mUnique->unref();
	mUnique = new Boolean(*unique);
    }

} // namespace MOF
