// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_NAMESPACE_H
#define MOF_NAMESPACE_H

#include "model_element.h"

#include <set>
#include <vector>
#include <list>
using namespace std;

namespace MOF {

    using namespace Common;

    class Model;
    class Import;

    //

    class Namespace : public virtual ModelElement {
    public:
	const list<ModelElement *> & get_contained_elements() { return mContainedElements; }
	void add_contained_element(ModelElement * contained_element, int index = -1);
	void remove_contained_element(ModelElement * contained_element);
	void set_contained_elements(const list<ModelElement *> & contained_elements);
	void modify_contained_element(ModelElement * old_contained_element,
				      ModelElement * new_contained_element);

#if 0
	ModelElement * lookup(const string & name);
	ModelElement * resolve_qualified_name(const vector<string> & names);
#endif
#if 0
	bool name_is_valid(const string & proposed_name) const;
#endif
#if 0
	void find_elements_by_type(set<ModelElement *> & model_elements, const type_info & type);
#endif

	// internal

	void internal_add_contained_element(ModelElement * contained_element) throw();
	void internal_remove_contained_element(ModelElement * contained_element) throw();

	void internal_add_importer(Import * importer) throw();
	void internal_remove_importer(Import * importer) throw();

	// internal reflective

	void ref_set_context_impl(set<string> & visited, REF::Context * context);

    protected:
	Namespace();
	~Namespace();

	// reflective

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();
	void ref_delete_impl(set<string> & visited);
	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;
	const string & ref_get_type() const;


    private:
	list<ModelElement *> mContainedElements;
	set<Import *> mImporters;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_NAMESPACE_H
