// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_OPERATION_H
#define MOF_OPERATION_H

#include "behavioral_feature.h"

#include "boolean.h"

#include <list>
using namespace std;

namespace MOF {

    class Exception;

    using namespace Common;

    const string OPERATION_CONTAINMENT_RULES =
	"org.omg.mof:constraint.model.operation.operation_containment_rules";
    const string OPERATIONS_HAVE_AT_MOST_ONE_RETURN =
	"org.omg.mof:constraint.model.operation.operations_have_at_most_one_return";
    const string OPERATION_EXCEPTIONS_MUST_BE_VISIBLE =
	"org.omg.mof:constraint.model.operation.operation_exceptions_must_be_visible";

    class Model;

    //

    class Operation : public BehavioralFeature {
    public:
	Operation();
	~Operation();

	void add_exception(Exception * exception, int index = -1);
	void remove_exception(Exception * exception);
	void set_exceptions(const list<Exception *> & exceptions);
	const list<Exception *> & get_exceptions() { return mExceptions; }

	const Boolean * is_query() const { return mQuery; }
	void set_query(const Boolean * query);

    protected:
	// internal

	// reflective

	void ref_delete_impl(set<string> & visited);

	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;
	const string & ref_get_type() const;

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();

    private:
	list<Exception *> mExceptions;
	Boolean * mQuery;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_OPERATION_H
