// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "writer.h"

#include "model.h"

#include "association.h"
#include "association_end.h"
#include "attribute.h"
#include "class.h"
#include "constant.h"
#include "constraint.h"
#include "data_type.h"
#include "exception.h"
#include "import.h"
#include "operation.h"
#include "package.h"
#include "parameter.h"
#include "reference.h"
#include "tag.h"
#include "alias_type.h"

#include "foundation/utility.h"
#include "foundation/macros.h"

#include <iostream>
#include <functional>
#include <algorithm>
using namespace std;

namespace MOF {

    ostream &
    operator << (ostream & ost, Writer & writer) {
	writer.write(ost);
	return ost;
    }

    Writer::Writer(ModelElement * root) : Object(),
	mRoot(root),
	mStream(0) {

	if(mRoot) {
	    mRoot->ref();
	}

	// initialise the table

	mMap[&typeid(Association)] = &Writer::a_association;
	mMap[&typeid(AssociationEnd)] = &Writer::a_association_end;
	mMap[&typeid(Attribute)] = &Writer::a_attribute;
	mMap[&typeid(Class)] = &Writer::a_class;
	mMap[&typeid(Constant)] = &Writer::a_constant;
	mMap[&typeid(Constraint)] = &Writer::a_constraint;
	mMap[&typeid(DataType)] = &Writer::a_data_type;
	mMap[&typeid(Exception)] = &Writer::a_exception;
	mMap[&typeid(Import)] = &Writer::a_import;
	mMap[&typeid(Operation)] = &Writer::a_operation;
	mMap[&typeid(Package)] = &Writer::a_package;
	mMap[&typeid(Parameter)] = &Writer::a_parameter;
	mMap[&typeid(Reference)] = &Writer::a_reference;
	mMap[&typeid(Tag)] = &Writer::a_tag;
	mMap[&typeid(AliasType)] = &Writer::a_alias_type;
    }

    Writer::~Writer() {
	if(mRoot) {
	    mRoot->unref();
	}
    }

    void
    Writer::write(ostream & ost) {
	if(!mRoot) {
	    cerr << "There is no spoon" << endl;
	    throw;	// FIXME
	}
	else {
	    mStream = new Stream(ost, "XMI", DTDK_SYSTEM, "xmi.dtd");

	    *mStream
		<< XmlComment("This file was generated by Dachshund V" VERSION)
		<< XmlBeginOpen("XMI")
		<< XmlAttribute("xmi.version", "1.1")
		<< XmlBeginClose();

	    *mStream
		<< XmlBeginOpen("XMI.header")
		<< XmlBeginClose();

	    // TODO the header

	    *mStream
		<< XmlEnd();

	    *mStream
		<< XmlBeginOpen("XMI.content")
		<< XmlBeginClose();

	    try {
		invoke_handler(mRoot, false);
	    }
	    catch(...) {
		delete mStream;
		throw;
	    }

	    *mStream
		<< XmlEnd();		// XMI.content

	    *mStream
		<< XmlEnd();		// XMI

	    delete mStream;
	}
    }

    void
    Writer::invoke_handler(ModelElement * model_element, bool reference) {
	map<TypeInfo, Method, Compare>::iterator iter = mMap.find(&typeid(*model_element));

	if(iter == mMap.end()) {
	    cerr << "No handler for this type" << endl;
	    throw;	// FIXME
	}
	else {
	    Method method = iter->second;
	    (this->*method)(model_element, reference);
	}
    }

    // start the entry, but uncasted

    void
    Writer::a_association(ModelElement * model_element, bool reference) {
	b_association(known_cast<Association *>(model_element), reference);
    }

    void
    Writer::a_association_end(ModelElement * model_element, bool reference) {
	b_association_end(known_cast<AssociationEnd *>(model_element), reference);
    }

    void
    Writer::a_attribute(ModelElement * model_element, bool reference) {
	b_attribute(known_cast<Attribute *>(model_element), reference);
    }

    void
    Writer::a_class(ModelElement * model_element, bool reference) {
	b_class(known_cast<Class *>(model_element), reference);
    }

    void
    Writer::a_constant(ModelElement * model_element, bool reference) {
	b_constant(known_cast<Constant *>(model_element), reference);
    }

    void
    Writer::a_constraint(ModelElement * model_element, bool reference) {
	b_constraint(known_cast<Constraint *>(model_element), reference);
    }

    void
    Writer::a_data_type(ModelElement * model_element, bool reference) {
	b_data_type(known_cast<DataType *>(model_element), reference);
    }

    void
    Writer::a_exception(ModelElement * model_element, bool reference) {
	b_exception(known_cast<Exception *>(model_element), reference);
    }

    void
    Writer::a_import(ModelElement * model_element, bool reference) {
	b_import(known_cast<Import *>(model_element), reference);
    }

    void
    Writer::a_operation(ModelElement * model_element, bool reference) {
	b_operation(known_cast<Operation *>(model_element), reference);
    }

    void
    Writer::a_package(ModelElement * model_element, bool reference) {
	b_package(known_cast<Package *>(model_element), reference);
    }

    void
    Writer::a_parameter(ModelElement * model_element, bool reference) {
	b_parameter(known_cast<Parameter *>(model_element), reference);
    }

    void
    Writer::a_reference(ModelElement * model_element, bool reference) {
	b_reference(known_cast<Reference *>(model_element), reference);
    }

    void
    Writer::a_tag(ModelElement * model_element, bool reference) {
	b_tag(known_cast<Tag *>(model_element), reference);
    }

    void
    Writer::a_alias_type(ModelElement * model_element, bool reference) {
	b_alias_type(known_cast<AliasType *>(model_element), reference);
    }


    // start the entry

    void
    Writer::b_association(Association * association, bool reference) {
	*mStream << XmlBeginOpen("MOF:Association");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", association->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", association->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_association(association);

	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_association_end(AssociationEnd * association_end, bool reference) {
	*mStream << XmlBeginOpen("MOF:AssociationEnd");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", association_end->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", association_end->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_association_end(association_end);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_attribute(Attribute * attribute, bool reference) {
	*mStream << XmlBeginOpen("MOF:MofAttribute");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", attribute->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", attribute->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_attribute(attribute);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_class(Class * klass, bool reference) {
	*mStream << XmlBeginOpen("MOF:Class");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", klass->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", klass->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_class(klass);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_constant(Constant * constant, bool reference) {
	*mStream << XmlBeginOpen("MOF:Constant");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", constant->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", constant->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_constant(constant);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_constraint(Constraint * constraint, bool reference) {
	*mStream << XmlBeginOpen("MOF:Constraint");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", constraint->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", constraint->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_constraint(constraint);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_data_type(DataType * data_type, bool reference) {
	*mStream << XmlBeginOpen("MOF:DataType");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", data_type->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", data_type->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_data_type(data_type);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_exception(Exception * exception, bool reference) {
	*mStream << XmlBeginOpen("MOF:MofException");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuid", exception->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", exception->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_exception(exception);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_import(Import * import, bool reference) {
	*mStream << XmlBeginOpen("MOF:Import");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuid", import->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", import->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_import(import);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_operation(Operation * operation, bool reference) {
	*mStream << XmlBeginOpen("MOF:Operation");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuid", operation->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", operation->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_operation(operation);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_package(Package * package, bool reference) {
	*mStream << XmlBeginOpen("MOF:Package");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", package->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", package->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_package(package);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_parameter(Parameter * parameter, bool reference) {
	*mStream << XmlBeginOpen("MOF:Parameter");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", parameter->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", parameter->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_parameter(parameter);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_reference(Reference * reference, bool ref) {
	*mStream << XmlBeginOpen("MOF:Reference");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuid", reference->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", reference->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_reference(reference);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_tag(Tag * tag, bool reference) {
	*mStream << XmlBeginOpen("MOF:Tag");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", tag->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", tag->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_tag(tag);
	}

	*mStream << XmlEnd();
    }

    void
    Writer::b_alias_type(AliasType * alias_type, bool reference) {
	*mStream << XmlBeginOpen("MOF:AliasType");

	if(reference) {
	    *mStream
		<< XmlAttribute("xmi.uuidref", alias_type->ref_get_uuid().get_string())
		<< XmlBeginClose();
	}
	else {
	    *mStream
		<< XmlAttribute("xmi.uuid", alias_type->ref_get_uuid().get_string())
		<< XmlBeginClose();

	    c_alias_type(alias_type);
	}

	*mStream << XmlEnd();
    }


    // fill in the guts (concrete types)

    void
    Writer::c_association(Association * association) {
	d_classifier(association);

	*mStream
	    << XmlBeginOpen("MOF:Association.isDerived")
	    //<< XmlAttribute("xmi.value", stringify(association->is_derived()))
	    << XmlBeginClose()
	    << XmlEnd();
    }

    void
    Writer::c_association_end(AssociationEnd * association_end) {
	d_typed_element(association_end);

	// TODO Multiplicity

	*mStream
	    << XmlBeginOpen("MOF:AssociationEnd.aggregation")
	    //<< XmlAttribute("xmi.value", stringify(association_end->get_aggregation()))
	    << XmlBeginClose()
	    << XmlEnd();
    }

    void
    Writer::c_attribute(Attribute * attribute) {
	d_structural_feature(attribute);
    }

    void
    Writer::c_class(Class * klass) {
	d_classifier(klass);
    }

    void
    Writer::c_constant(Constant * constant) {
	d_typed_element(constant);
    }

    void
    Writer::c_constraint(Constraint * constraint) {
	d_model_element(constraint);

	//
	// constrained elements
	//

	const set<ModelElement *> & constrained_elements = constraint->get_constrained_elements();

	if(!constrained_elements.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:Constraint.constrainedElements")
		<< XmlBeginClose();

	    for(set<ModelElement *>::const_iterator iter = constrained_elements.begin();
		iter != constrained_elements.end();
		++iter) {
		ModelElement * constrained_element = *iter;
		invoke_handler(constrained_element, true);
	    }

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::c_data_type(DataType * data_type) {
	d_classifier(data_type);
    }

    void
    Writer::c_exception(Exception * exception) {
	d_behavioral_feature(exception);
    }

    void
    Writer::c_import(Import * import) {
	d_model_element(import);

	//
	// imported
	//

	Namespace * imported = import->get_imported();

	if(imported) {
	    *mStream
		<< XmlBeginOpen("MOF:Import.imported")
		<< XmlBeginClose();

	    invoke_handler(imported, true);

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::c_operation(Operation * operation) {
	d_behavioral_feature(operation);

	//
	// exceptions
	//

	const list<Exception *> & exceptions = operation->get_exceptions();

	if(!exceptions.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:Operation.exceptions")
		<< XmlBeginClose();

	    for(list<Exception *>::const_iterator iter = exceptions.begin();
		iter != exceptions.end();
		++iter) {
		Exception * exception = *iter;
		invoke_handler(exception, true);
	    }

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::c_package(Package * package) {
	d_namespace(package);
    }

    void
    Writer::c_parameter(Parameter * parameter) {
	d_typed_element(parameter);
    }

    void
    Writer::c_reference(Reference * reference) {
	d_structural_feature(reference);

	//
	// referenced end
	//

	AssociationEnd * referenced_end = reference->get_referenced_end();

	if(referenced_end) {
	    *mStream
		<< XmlBeginOpen("MOF:Reference.referencedEnd")
		<< XmlBeginClose();

	    invoke_handler(referenced_end, true);

	    *mStream
		<< XmlEnd();
	}
	else {
	    cerr << "Should have referenced_end" << endl;
	}

	//
	// exposed end
	//

	AssociationEnd * exposed_end = reference->get_exposed_end();

	if(exposed_end) {
	    *mStream
		<< XmlBeginOpen("MOF:Reference.exposedEnd")
		<< XmlBeginClose();

	    invoke_handler(exposed_end, true);

	    *mStream
		<< XmlEnd();
	}
	else {
	    cerr << "Should have exposed_end" << endl;
	}
    }

    void
    Writer::c_tag(Tag * tag) {
	d_model_element(tag);

	//
	// elements
	//

	const set<ModelElement *> & elements = tag->get_models();

	if(!elements.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:Tag.elements")
		<< XmlBeginClose();

	    for(set<ModelElement *>::const_iterator iter = elements.begin();
		iter != elements.end();
		++iter) {
		ModelElement * element = *iter;

		invoke_handler(element, true);
	    }

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::c_alias_type(AliasType * alias_type) {
	d_typed_element(alias_type);
    }


    // fill in the guts (abstract types)

    void
    Writer::d_behavioral_feature(BehavioralFeature * behavioral_feature) {
	d_namespace(behavioral_feature);
	d_feature(behavioral_feature);
    }

    void
    Writer::d_classifier(Classifier * classifier) {
	d_generalizable_element(classifier);
    }

    void
    Writer::d_feature(Feature * feature) {
	*mStream
	    << XmlBeginOpen("MOF:Feature.visibility")
	    //<< XmlAttribute("xmi.value", stringify(feature->get_visibility()))
	    << XmlBeginClose()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:Feature.scope")
	    //<< XmlAttribute("xmi.value", stringify(feature->get_scope()))
	    << XmlBeginClose()
	    << XmlEnd();

	// don't descend further because the only classes that derive from us
	// also derive from another descendent of model element
    }

    void
    Writer::d_generalizable_element(GeneralizableElement * generalizable_element) {
	d_namespace(generalizable_element);

	*mStream
	    << XmlBeginOpen("MOF:GeneralizableElement.visibility")
	    //<< XmlAttribute("xmi.value", stringify(generalizable_element->get_visibility()))
	    << XmlBeginClose()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:GeneralizableElement.isAbstract")
	    //<< XmlAttribute("xmi.value", stringify(generalizable_element->is_abstract()))
	    << XmlBeginClose()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:GeneralizableElement.isRoot")
	    //<< XmlAttribute("xmi.value", stringify(generalizable_element->is_root()))
	    << XmlBeginClose()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:GeneralizableElement.isLeaf")
	    //<< XmlAttribute("xmi.value", stringify(generalizable_element->is_leaf()))
	    << XmlBeginClose()
	    << XmlEnd();

	//
	// supertypes
	//

	const list<GeneralizableElement *> & supertypes = generalizable_element->get_supertypes();

	if(!supertypes.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:GeneralizableElement.supertypes")
		<< XmlBeginClose();

	    for(list<GeneralizableElement *>::const_iterator iter = supertypes.begin();
		iter != supertypes.end();
		++iter) {
		GeneralizableElement * supertype = *iter;

		invoke_handler(supertype, true);
	    }

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::d_model_element(ModelElement * model_element) {
	*mStream
	    << XmlBeginOpen("MOF:ModelElement.name")
	    << XmlBeginClose()
	    << model_element->get_name()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:ModelElement.annotation")
	    << XmlBeginClose()
	    << model_element->get_annotation()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:ModelElement.isFrozen")
	    //<< XmlAttribute("xmi.value", stringify(model_element->is_frozen()))
	    << XmlBeginClose()
	    << XmlEnd();

	*mStream
	    << XmlBeginOpen("MOF:ModelElement.isVisible")
	    //<< XmlAttribute("xmi.value", stringify(model_element->is_visible()))
	    << XmlBeginClose()
	    << XmlEnd();

	//
	// providers
	//

	const set<ModelElement *> & providers = model_element->get_providers();

	if(!providers.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:ModelElement.providers")
		<< XmlBeginClose();

	    for(set<ModelElement *>::const_iterator iter = providers.begin();
		iter != providers.end();
		++iter) {
		ModelElement * provider = *iter;
		invoke_handler(provider, true);
	    }

	    *mStream
		<< XmlEnd();
	}

	//
	// tags
	//

	const list<Tag *> & tags = model_element->get_tags();

	if(!tags.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:ModelElement.tags")
		<< XmlBeginClose();

	    for(list<Tag *>::const_iterator iter = tags.begin(); iter != tags.end(); ++iter) {
		Tag * tag = *iter;
		invoke_handler(tag, true);
	    }

	    *mStream
		<< XmlEnd();
	}

	//
	// constraints
	//

	const set<Constraint *> & constraints = model_element->get_constraints();

	if(!constraints.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:ModelElement.constraints")
		<< XmlBeginClose();

	    for(set<Constraint *>::const_iterator iter = constraints.begin();
		iter != constraints.end();
		++iter) {
		Constraint * constraint = *iter;
		invoke_handler(constraint, true);
	    }

	    *mStream
		<< XmlEnd();
	}

	//
	// container
	//

	Namespace * container = model_element->get_container();

	if(container) {
	    *mStream
		<< XmlBeginOpen("MOF:ModelElement.container")
		<< XmlBeginClose();

	    invoke_handler(container, true);

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::d_namespace(Namespace * namspace) {
	d_model_element(namspace);

	//
	// contained elements
	//

	const list<ModelElement *> & contained_elements = namspace->get_contained_elements();

	if(!contained_elements.empty()) {
	    *mStream
		<< XmlBeginOpen("MOF:Namespace.contents")
		<< XmlBeginClose();

	    for(list<ModelElement *>::const_iterator iter = contained_elements.begin();
		iter != contained_elements.end();
		++iter) {
		ModelElement * contained_element = *iter;
		invoke_handler(contained_element, false);
	    }

	    *mStream
		<< XmlEnd();
	}
    }

    void
    Writer::d_structural_feature(StructuralFeature * structural_feature) {
	d_feature(structural_feature);
	d_typed_element(structural_feature);

	*mStream
	    << XmlBeginOpen("MOF:StructuralFeature.isChangeable")
	    //<< XmlAttribute("xmi.value", stringify(structural_feature->is_changeable()))
	    << XmlBeginClose()
	    << XmlEnd();
    }

    void
    Writer::d_typed_element(TypedElement * typed_element) {
	d_model_element(typed_element);

	//
	// type
	//

	Classifier * type = typed_element->get_type();

	if(type) {
	    *mStream
		<< XmlBeginOpen("MOF:TypedElement.types")
		<< XmlBeginClose();

	    invoke_handler(type, true);

	    *mStream
		<< XmlEnd();
	}
    }

} // namespace MOF
