// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "foundation/macros.h"
#include "foundation/undo_manager.h"

// Python Interfaces

#include "python/foundation/object.h"
#include "python/foundation/module.h"


namespace {

  PyObject *
  wrapped_begin_transaction(PyObject * self, PyObject * args) {

    if (!PyArg_ParseTuple(args, ":begin_transaction"))
      return NULL;
    UndoManager * undo_manager = UndoManager::instance();
    undo_manager->begin_transaction();
    return Py_BuildValue("");
  }

  PyObject *
  wrapped_end_transaction(PyObject * self, PyObject * args) {

    if (!PyArg_ParseTuple(args, ":begin_transaction"))
      return NULL;
    UndoManager * undo_manager = UndoManager::instance();
    undo_manager->end_transaction();
    return Py_BuildValue("");
  }

}


namespace Python {

    void foundation_initialise() {
        static struct PyMethodDef module_methods[] = {
            { "Object",   object_reference_new,   METH_VARARGS },
            { "begin_transaction", wrapped_begin_transaction, METH_VARARGS },
            { "end_transaction",   wrapped_end_transaction,   METH_VARARGS },

            { NULL, NULL }
        };

        TRACE(1, "Initialise Foundation interface...");
        Py_InitModule("Foundation", module_methods);
    }

} // namespace Python
