// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <map>

#include "foundation/object.h"
#include "foundation/macros.h"
#include "foundation/exception.h"

#include "toolkit/widget.h"
#include "toolkit/tree.h"
#include "toolkit/tree_maintainer.h"
#include "toolkit/mouse_listener.h"
#include "toolkit/mouse_event.h"

#include "drawing/point.h"
#include "drawing/image.h"

// Python interfaces

#include "python/common/convert.h"
#include "python/foundation/object.h"

#include "python/drawing/image.h"
#include "python/toolkit/mouse_event.h"
#include "python/toolkit/tree.h"


//================================================================================

namespace {
  using namespace Python;

  typedef map<Object *, PyObject *> ReferenceMap;
  ReferenceMap reference_table;

/*
  PyObject *
  wrapped_tree_begin_transaction(PyObject * self, PyObject * args) {
    //virtual void begin_transaction()
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * wrp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.begin_transaction"))
	return NULL;

      // Finally call the method
      wrp->mTree->begin_transaction();
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  PyObject *
  wrapped_tree_end_transaction(PyObject * self, PyObject * args) {
    //virtual void end_transaction()
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * wrp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.end_transaction"))
	return NULL;

      // Finally call the method
      wrp->mTree->end_transaction();
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }
*/

  //--------------------------------------------------------------------------------


  PyObject *
  wrapped_tree_show(PyObject * self, PyObject * args) {
    // Widget methods
    //virtual void show()
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.show"))
	return NULL;

      // Finally call the method
      rp->mTree->show();
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  PyObject *
  wrapped_tree_hide(PyObject * self, PyObject * args) {
    // Widget methods
    //virtual void hide()
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.hide"))
	return NULL;

      // Finally call the method
      rp->mTree->hide();
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  //--------------------------------------------------------------------------------


  PyObject *
  wrapped_tree_add(PyObject * self, PyObject * args) {
    //virtual void add(Object * node, Drawing::Image * image, const string & label,
    //                 bool expanded, Object * parent, Object * position = 0) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      PyObject * ref_image = 0;
      const char * label = 0;
      int expanded = 0;
      PyObject * ref_parent = 0;
      PyObject * ref_position = 0;
      if (!PyArg_ParseTuple(args, "OOsiO|O:Tree.add", & ref_node, & ref_image, 
			    & label, & expanded, & ref_parent, & ref_position))
	return NULL;

      Object * node = 0;
      Drawing::Image * image = 0;
      Object * parent = 0;
      Object * position = 0;
      if (!convert_to_object(node, ref_node) ||
	  !convert_to_image(image, ref_image) ||
	  !convert_to_object(parent, ref_parent) ||
	  (position && !convert_to_object(position, ref_position)))
	return NULL;

      //MESSAGE("Tree add for tree " << rp->mTree << " " << label);
      // Finally call the method
      rp->mTree->add(node, image, string(label), expanded, parent, position);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  PyObject *
  wrapped_tree_remove(PyObject * self, PyObject * args) {
    //virtual void remove(Object * node) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.remove", & ref_node))
	return NULL;

      Object * node = 0;
      if (!convert_to_object(node, ref_node))
	return NULL;

      // Finally call the method
      rp->mTree->remove(node);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  //--------------------------------------------------------------------------------

  PyObject *
  wrapped_tree_clear(PyObject * self, PyObject * args) {
    //virtual void clear() = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.clear"))
	return NULL;

      // Finally call the method
      rp->mTree->clear();
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  //--------------------------------------------------------------------------------

  PyObject *
  wrapped_tree_get_root(PyObject * self, PyObject * args) {
    //virtual Object * get_root() = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tree.get_root"))
	return NULL;

      // Finally call the method
      return object_reference_new(rp->mTree->get_root());
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }


  PyObject *
  wrapped_tree_get_parent(PyObject * self, PyObject * args) {
    //virtual Object * get_parent(Object * node) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.get_parent", & ref_node))
	return NULL;

      Object * node = 0;
      if (!convert_to_object(node, ref_node))
	return NULL;

      // Finally call the method
      return object_reference_new(rp->mTree->get_parent(node));
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  //--------------------------------------------------------------------------------
  
  PyObject *
  wrapped_tree_contains(PyObject * self, PyObject * args) {
    //virtual bool contains(Object * node) const = 0;	// is the item somewhere in the tree ?
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.contains", & ref_node))
	return NULL;

      Object * node = 0;
      if (!convert_to_object(node, ref_node))
	return NULL;

      // Finally call the method
      int result = rp->mTree->contains(node);
      return Py_BuildValue("i", result);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  //--------------------------------------------------------------------------------

  PyObject *
  wrapped_tree_set_image(PyObject * self, PyObject * args) {
    //virtual void set_image(Object * node, Drawing::Image * image) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      PyObject * ref_image = 0;
      if (!PyArg_ParseTuple(args, "OO:Tree.set_image", & ref_node, & ref_image))
	return NULL;

      Object * node = 0;
      Drawing::Image * image = 0;
      if (!convert_to_object(node, ref_node) ||
	  !convert_to_image(image, ref_image))
	return NULL;

      // Finally call the method
      rp->mTree->set_image(node, image);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  PyObject *
  wrapped_tree_set_label(PyObject * self, PyObject * args) {
    //virtual void set_label(Object * node, const string & label) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * ref_node = 0;
      const char * label = 0;
      if (!PyArg_ParseTuple(args, "Os:Tree.set_label", & ref_node, & label))
	return NULL;

      Object * node = 0;
      if (!convert_to_object(node, ref_node))
	return NULL;

      // Finally call the method
      rp->mTree->set_label(node, string(label));
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }
  
  //--------------------------------------------------------------------------------

  PyObject *
  wrapped_tree_point_to_node(PyObject * self, PyObject * args) {
    //virtual Object * point_to_node(const Drawing::Point & point) = 0;
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      int point_x = 0;
      int point_y = 0;
      if (!PyArg_ParseTuple(args, "(ii):Tree.point_to_node", & point_x, & point_y))
	return NULL;

      // Finally call the method
      return object_reference_new(rp->mTree->point_to_node(Drawing::Point(point_x, point_y)));
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  //--------------------------------------------------------------------------------

  class ProxyTreeMaintainer: public Toolkit::TreeMaintainer {
    PyObject * mObject;
  public:
    virtual bool can_relocate(Toolkit::Tree * tree, Object * node,
			      Object * old_parent, Object * new_parent) const {
      PyObject * ref_tree = tree_reference_new(tree);
      PyObject * ref_node = object_reference_new(node);
      PyObject * ref_old_parent = object_reference_new(old_parent);
      PyObject * ref_new_parent = object_reference_new(new_parent);
      PyObject * ref_result = PyObject_CallMethod(mObject, "can_relocate", "(OOOO)", 
						  ref_tree, ref_node, 
						  ref_old_parent, ref_new_parent);
      Py_DECREF(ref_tree);
      Py_DECREF(ref_node);
      Py_DECREF(ref_old_parent);
      Py_DECREF(ref_new_parent);
      bool result = false;
      if (ref_result == NULL) {
	ERROR("Error returned by 'can_relocate'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }

    virtual void relocated(Toolkit::Tree * tree, Object * node, Object * new_parent) {
      PyObject * ref_tree = tree_reference_new(tree);
      PyObject * ref_node = object_reference_new(node);
      PyObject * ref_new_parent = object_reference_new(new_parent);
      PyObject * ref_result = PyObject_CallMethod(mObject, "relocated", "(OOO)", 
						  ref_tree, ref_node, ref_new_parent);
      Py_DECREF(ref_tree);
      Py_DECREF(ref_node);
      Py_DECREF(ref_new_parent);
      if (ref_result == NULL) {
	ERROR("Error returned by 'relocated'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	Py_DECREF(ref_result);
      }
    }
    
    virtual void renamed(Toolkit::Tree * tree, Object * node, const string & name) {
      PyObject * ref_tree = tree_reference_new(tree);
      PyObject * ref_node = object_reference_new(node);
      PyObject * ref_result = PyObject_CallMethod(mObject, "renamed", "(OOs)", 
						  ref_tree, ref_node, name.c_str());
      Py_DECREF(ref_tree);
      Py_DECREF(ref_node);
      if (ref_result == NULL) {
	ERROR("Error returned by 'renamed'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	Py_DECREF(ref_result);
      }
    }
    
    virtual void selection(const set<Object *> & nodes) {

      // Copy the set into a python list
      int count = nodes.size();
      PyObject * ref_nodes = PyList_New(count);
      set<Object *>::const_iterator iter = nodes.begin();
      for (int i = 0; i < count; i++, ++iter)
	PyList_SetItem(ref_nodes, i, object_reference_new(* iter));

      PyObject * ref_result = PyObject_CallMethod(mObject, "selection", "(O)", ref_nodes);
      Py_DECREF(ref_nodes);
      if (ref_result == NULL) {
	ERROR("Error returned by 'selection'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	Py_DECREF(ref_result);
      }
    }

    ProxyTreeMaintainer(PyObject * object) : mObject(object) { 
      Py_INCREF(mObject);
    }
    virtual ~ProxyTreeMaintainer() {
      Py_DECREF(mObject);
    }
  };


  PyObject *
  wrapped_tree_set_maintainer(PyObject * self, PyObject * args) {
    //void set_maintainer(TreeMaintainer * maintainer);
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * object = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.set_maintainer", & object))
	return NULL;
      // Error checking?

      // Remove any existing proxy
      delete(rp->mTree->get_maintainer());

      // Add the new maintainer proxy
      if (object != Py_None) {
	ProxyTreeMaintainer * proxy = new ProxyTreeMaintainer(object);
	rp->mTree->set_maintainer(proxy);
      }
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  //--------------------------------------------------------------------------------

  class ProxyMouseListener : public Toolkit::MouseListener {
    PyObject * mObject;

  public:
    virtual bool mouse_entered(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "mouse_entered", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'entered'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }

    virtual bool mouse_exited(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "mouse_exited", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'exited'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }

    virtual bool mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "mouse_pressed", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'pressed'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }

    virtual bool mouse_released(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "released", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'released'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }
    virtual bool mouse_moved(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "mouse_moved", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'moved'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;

    }
    virtual bool mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
      PyObject * ref_event = mouse_event_reference_new(mouse_event);
      PyObject * ref_result = PyObject_CallMethod(mObject, "mouse_dragged", "(O)", ref_event);
      bool result = false;
      Py_DECREF(ref_event);
      if (ref_result == NULL) {
	ERROR("Error returned by 'dragged'");
	if (PyErr_Occurred())
	  PyErr_Print();
      }
      else {
	if (PyObject_IsTrue(ref_result))
	  result = true;
	Py_DECREF(ref_result);
      }
      return result;
    }

    PyObject * get() { return mObject; }
    ProxyMouseListener(PyObject * object) : mObject(object) { 
      Py_INCREF(mObject);

      // Check each of the functions to see if it is callable?      
    }
    virtual ~ProxyMouseListener() { 
      Py_DECREF(mObject); 
    }
  };
  
  //--------------------------------------------------------------------------------

  // List of maintainers registered
  list<ProxyMouseListener *> listener_list;

  
  PyObject *
  wrapped_tree_listener_add(PyObject * self, PyObject * args) {
    //void add(MouseListener * mouse_listener);
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * object = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.listener_add", & object))
	return NULL;
      // type check?

      ProxyMouseListener * proxy = new ProxyMouseListener(object);
      listener_list.push_back(proxy);
      rp->mTree->add(proxy);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }
  
  PyObject *
  wrapped_tree_listener_remove(PyObject * self, PyObject * args) {
    //void remove(const MouseListener * mouse_listener);
    assert(PyObject_TypeCheck(self, & toolkitTreeType));
    TreeReference * rp = (TreeReference *) self;
    try {
      // Extract the arguments
      PyObject * object = 0;
      if (!PyArg_ParseTuple(args, "O:Tree.listener_remove", & object))
	return NULL;

      // Lookup the proxy to remove
      for (list<ProxyMouseListener *>::iterator iter = listener_list.begin();
	   iter != listener_list.end(); iter++) {
	if (object == (* iter)->get()) {

	  // Unregister the listener, delete the object, remove from list
	  rp->mTree->remove(* iter);
	  delete(* iter);
	  listener_list.erase(iter);
	  break;
	}
      }
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  //--------------------------------------------------------------------------------

  struct PyMethodDef tree_methods[ ] = {
/*
    { "begin_transaction",	wrapped_tree_begin_transaction, METH_VARARGS },
    { "end_transaction",	wrapped_tree_end_transaction, METH_VARARGS },
*/
    { "show",			wrapped_tree_show,	      METH_VARARGS },
    { "hide",			wrapped_tree_hide,	      METH_VARARGS },

    { "add",			wrapped_tree_add,	      METH_VARARGS },
    { "remove",			wrapped_tree_remove,          METH_VARARGS },
    { "clear",			wrapped_tree_clear,           METH_VARARGS },
    { "get_root",		wrapped_tree_get_root,        METH_VARARGS },
    { "get_parent",		wrapped_tree_get_parent,      METH_VARARGS },
    { "contains",		wrapped_tree_contains,        METH_VARARGS },
    { "set_image",		wrapped_tree_set_image,       METH_VARARGS },
    { "set_label",		wrapped_tree_set_label,       METH_VARARGS },
    { "point_to_node",		wrapped_tree_point_to_node,   METH_VARARGS },

    { "set_maintainer",         wrapped_tree_set_maintainer,  METH_VARARGS },

    { "listener_add",		wrapped_tree_listener_add,    METH_VARARGS },
    { "listener_remove",	wrapped_tree_listener_remove, METH_VARARGS },
    
    { NULL, NULL }
  };



  PyObject *
  tree_getattr(PyObject * self, char * name) {

    // Look for a method
    return Py_FindMethod(tree_methods, self, name);
  }


  int
  tree_setattr(PyObject * self, char * name, PyObject * value) {
    PyErr_SetString(PyExc_NameError, "No attributes to be set");
    return -1;
  }


  void
  tree_dealloc(PyObject * self) {
    if (PyObject_TypeCheck(self, & toolkitTreeType)) {
      TreeReference * rp = (TreeReference *) self;
      if (rp->mTree) {
	reference_table.erase(rp->mTree);
	rp->mTree->unref();
      }
    }
    PyMem_DEL(self);
  }


}

//================================================================================

namespace Python {


  //--------------------------------------------------------------------------------

  PyTypeObject toolkitTreeType = {
    PyObject_HEAD_INIT(&PyType_Type)		// Initialisation
    0,						// ob_size	: Usually 0
    "toolkit.Tree",				// tp_name	: Type Name
    sizeof(TreeReference),			// tp_basicsize : Object size
    0,						// tp_itemsize  : Usually 0
    
    // Standard methods
    (destructor) tree_dealloc,			// tp_dealloc	: refcount = 0
    (printfunc) 0,				// tp_print	: print x
    (getattrfunc) tree_getattr,			// tp_getattr	: x.attr
    (setattrfunc) tree_setattr,			// tp_setattr	: a.attr = v
    (cmpfunc) 0,				// tp_compare	: x > y
    (reprfunc) 0,				// tp_repr	: repr(x)

    // Type categories
    0,						// tp_as_number
    0,						// tp_as_sequence
    0,						// tp_as_mapping

    (hashfunc) 0,				// tp_hash
    (ternaryfunc) 0,				// tp_call
    (reprfunc) 0,				// tp_str
    (getattrofunc) 0,				// tp_getatto
    (setattrofunc) 0,				// tp_setatto
    
    0,						// tp_as_buffer
    0,						// tp_flags
    0,						// tp_doc
    
    (traverseproc) 0,				// tp_traverse
    (inquiry) 0,				// tp_clear
    (richcmpfunc) 0,				// tp_richcompare
    0,						// tp_weaklistoffset
    (getiterfunc) 0,				// tp_iter
    (iternextfunc) 0,				// tp_iternext
    0, 						// tp_methods
    0, 						// tp_members
    0,						// tp_getset
    0,						// tp_base

  };


  //--------------------------------------------------------------------------------


  PyObject *
  tree_reference_new(Toolkit::Tree * tree) {

    PyObject * result = 0;
    ReferenceMap::iterator iter = reference_table.find(tree);
    if (iter == reference_table.end()) {

      // Create a new reference
      TreeReference * self = PyObject_NEW(TreeReference, & toolkitTreeType);
      if (! self)
	return NULL;
    
      tree->ref();		 // take joint ownership of the object
      self->mTree = tree;
      result = (PyObject *) self;
      reference_table[tree] = result;
    }
    else {
      TRACE(2, "Reusing an existing reference");
      // Use an existing reference
      result = iter->second;
      Py_INCREF(result);
    }
    return result;
  }

  PyObject *
  tree_reference_new(PyObject * self, PyObject * args) {
    TRACE(1, "Create a new Toolkit.Tree");
    PyErr_SetString(PyExc_TypeError, "don't know how to construct");
    return NULL;
  }

} // namespace Python
