// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "debug_change_listener.h"

#include "changes.h"

#include <algorithm>
#include <functional>
#include <iostream>
using namespace std;

namespace REF {

    DebugChangeListener::DebugChangeListener(ostream & ost) : Object(), ChangeListener(), mOs(ost) {
    }

    DebugChangeListener::~DebugChangeListener() {
    }

    void
    DebugChangeListener::changes_occurred(const list<ChangeEvent *> & changes) throw() {
	mOs << "Got: " << changes.size() << " changes" << endl;

	for_each(changes.begin(), changes.end(),
		 bind1st(mem_fun(&DebugChangeListener::process_change), this));

	mOs << "End of change" << endl << endl;
    }

    void
    DebugChangeListener::process_change(ChangeEvent * change_event) {
	ChangeAttributeChanged * ac = dynamic_cast<ChangeAttributeChanged *>(change_event);

	if(ac) {
	    const Attribute * old_value = ac->get_old_value();
	    const Attribute * new_value = ac->get_new_value();

	    string o, n;

	    if(old_value) {
		old_value->ref_get_value(o);
	    }
	    else {
		o = "<null>";
	    }

	    if(new_value) {
		new_value->ref_get_value(n);
	    }
	    else {
		n = "<null>";
	    }

	    mOs << "attribute changed: " << ac->get_attribute_name() << ", "
		<< "old value: \"" << o << "\", "
		<< "new value: \"" << n << "\""
		<< endl;
	    return;
	}

	ChangeRoleChanged * rc = dynamic_cast<ChangeRoleChanged *>(change_event);

	if(rc) {
	    mOs << "role changed: " << rc->get_role_name() << endl;
	    return;
	}

	ChangeRoleAdded * ra = dynamic_cast<ChangeRoleAdded *>(change_event);

	if(ra) {
	    mOs << "role added: " << ra->get_role_name() << endl;
	    return;
	}

	ChangeRoleRemoved * rr = dynamic_cast<ChangeRoleRemoved *>(change_event);

	if(rr) {
	    mOs << "role removed: " << rr->get_role_name() << endl;
	    return;
	}
    }

} // namespace REF
