// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "document.h"

#include "foundation/macros.h"

#include "mouse_listener.h"

#include <algorithm>
#include <functional>
#include <utility>
using namespace std;

namespace Toolkit {

    Document::Document() : Widget(), mDocumentMaintainer(0) {
    }

    Document::~Document() {
    }

    void
    Document::set_maintainer(DocumentMaintainer * maintainer) {
	mDocumentMaintainer = maintainer;
    }

    void
    Document::mouse_add(MouseListener * mouse_listener) {
	mMouseListeners.push_front(mouse_listener);
    }

    void
    Document::mouse_remove(const MouseListener * mouse_listener) {
	list<MouseListener *>::iterator iter = find(mMouseListeners.begin(), mMouseListeners.end(),
						    mouse_listener);

	if(iter == mMouseListeners.end()) {
	    cerr << "Tried to remove mouse listener but couldn't find it" << endl;
	}
	else {
	    mMouseListeners.erase(iter);
	}
    }

    bool
    Document::mouse_entered(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_entered), mouse_event));

	return iter != mMouseListeners.end();
    }

    bool
    Document::mouse_exited(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_exited), mouse_event));

	return iter != mMouseListeners.end();
    }

    bool
    Document::mouse_pressed(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_pressed), mouse_event));

	return iter != mMouseListeners.end();
    }

    bool
    Document::mouse_released(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_released), mouse_event));

	return iter != mMouseListeners.end();
    }

    bool
    Document::mouse_moved(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_moved), mouse_event));

	return iter != mMouseListeners.end();
    }

    bool
    Document::mouse_dragged(const MouseEvent * mouse_event) {
	list<MouseListener *>::const_iterator iter =
	    find_if(mMouseListeners.begin(), mMouseListeners.end(),
		    bind2nd(mem_fun(&MouseListener::mouse_dragged), mouse_event));

	return iter != mMouseListeners.end();
    }

} // namespace Toolkit
