// Dachshund							-*- C++ -*-
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_DOCUMENT_H
#define TOOLKIT_DOCUMENT_H

#include "widget.h"

#include <list>
#include <string>
using namespace std;

namespace Drawing {
  class Point;
  class Image;
} // namespace Drawing


namespace Toolkit {

  class DocumentMaintainer;
  class MouseListener;
  class MouseEvent;
  class Tag;
  class Mark;

  // FIXUP need some exceptions to deal with errors


  class Document : public virtual Widget {
  public:
    Document();
    ~Document();
    
    //------------------------------------------------------------
    // Functions dealing with the special marks
    //
    virtual Mark *	get_insert_mark() = 0;
    virtual Mark *	get_change_mark() = 0;
    virtual Mark *	get_selection_mark() = 0;


    //------------------------------------------------------------
    // Methods that implement an insertion type model for the document.

    virtual void insert_block_begin(Mark * location, Mark * block_mark, Tag * block) = 0;
    virtual void insert_block_end(Mark * location, Tag * block) = 0;

    virtual void insert_field_begin(Mark * location, Tag * field) = 0;
    virtual void insert_field_end(Mark * location, Tag * block) = 0;

    virtual void insert_field_text(Mark * location, Tag * field, const char * text) = 0;

    virtual void insert_text(Mark * location, const char * text) = 0;
    virtual void insert_image(Mark * location, Drawing::Image * image) = 0;
    virtual void insert_widget(Mark * location, Toolkit::Widget * widget) = 0;


    //------------------------------------------------------------
    // Collection of methods used to maintain the document.

    virtual void get_field_text(Mark * location, Tag * field, string & result) = 0;
    
    virtual void replace_field_text(Mark * location, Tag * field, const char * new_text) = 0;
    virtual void replace_field_image(Mark * location, Tag * field, Drawing::Image * image) = 0;
    virtual void replace_field_widget(Mark * location, Tag * field, Toolkit::Widget * widget) = 0;

    virtual void delete_field(Mark * location, Tag * field) = 0;
    virtual void delete_block(Mark * location, Tag * block) = 0;
    virtual void clear() = 0;

    //------------------------------------------------------------

    // Miscellaneous stuff for managing interactions.

    //------------------------------------------------------------

    //virtual Object * get_root() = 0;
    //virtual Object * get_parent(Object * node) = 0;


    //virtual bool contains(Object * node) const = 0;	// is the item somewhere in the document ?
    //virtual Object * point_to_node(const Drawing::Point & point) = 0;

    // Document Maintainer interface
    void set_maintainer(DocumentMaintainer * maintainer);
    DocumentMaintainer * get_maintainer() { return mDocumentMaintainer; }
    
    // Mouse interface
    void mouse_add(MouseListener * mouse_listener);
    void mouse_remove(const MouseListener * mouse_listener);

  protected:
    bool mouse_entered(const MouseEvent * mouse_event);
    bool mouse_exited(const MouseEvent * mouse_event);
    bool mouse_pressed(const MouseEvent * mouse_event);
    bool mouse_released(const MouseEvent * mouse_event);
    bool mouse_moved(const MouseEvent * mouse_event);
    bool mouse_dragged(const MouseEvent * mouse_event);

  private:
    DocumentMaintainer * mDocumentMaintainer;
    list<MouseListener *> mMouseListeners;
  };

} // namespace Toolkit

#endif // TOOLKIT_DOCUMENT_H
