// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_INPUT_EVENT_H
#define TOOLKIT_INPUT_EVENT_H

#include "gui_event.h"

namespace Toolkit {

    class Widget;

    enum MaskType {
	MT_NONE		= 0,
	MT_SHIFT	= 1 << 0,
	MT_CONTROL	= 1 << 1,
	MT_ALT		= 1 << 2,
	MT_BUTTON1	= 1 << 3,
	MT_BUTTON2	= 1 << 4,
	MT_BUTTON3	= 1 << 5,
    };

    class InputEvent : public GuiEvent {
    public:
	Widget * get_widget() const { return mWidget; }
	int get_mask() const { return mMask; }
	int get_timestamp() const { return mTimestamp; }

	bool is_shift_down() const { return mMask & MT_SHIFT; }
	bool is_ctrl_down() const { return mMask & MT_CONTROL; }
	bool is_alt_down() const { return mMask & MT_ALT; }
	bool is_button1_down() const { return mMask & MT_BUTTON1; }
	bool is_button2_down() const { return mMask & MT_BUTTON2; }
	bool is_button3_down() const { return mMask & MT_BUTTON3; }

    protected:
	InputEvent(Widget * widget, int mask, int timestamp);
	~InputEvent();

    private:
	Widget * mWidget;
	int mMask;
	int mTimestamp;
    };

} // namespace Toolkit

#endif // TOOLKIT_INPUT_EVENT_H
