// Dachshund							-*- C++ -*-
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_MARK_H
#define TOOLKIT_MARK_H

#include "foundation/object.h"

#include <string>

using namespace std;


// Components of a Document, see document.h

namespace Toolkit {

  class Document;
  class Tag;

  class Mark : public Object {
  public:
    Mark(const char * name);
    ~Mark();

    const char *	get_name() const { return mName.c_str(); } // used for debugging

    bool		is_block_mark() const { return mIsBlockMark; }
    void		set_is_block_mark(bool s) { mIsBlockMark = s; }

    void		set_associated_block(Tag * block);
    void		set_associated_level(int level);
    void		set_associated_object(Object * o);
    Tag *		get_associated_block() { return mAssocBlock; }
    int			get_associated_level() { return mAssocLevel; }
    Object *		get_associated_object() { return mAssocObject; }

    //----------------------------------------

    virtual void	relocate_to_mark(Mark * mark) = 0;
    virtual void	relocate_to_tag_end(Tag * tag) = 0;

    virtual bool	at_tag_start(Tag * tag) = 0;
    virtual bool	at_tag_end(Tag * tag) = 0;

    //----------------------------------------

  protected:
    string		mName;

    bool		mIsBlockMark;
    Tag *		mAssocBlock;
    int			mAssocLevel;
    Object *		mAssocObject;
  };



} // namespace Toolkit

#endif // TOOLKIT_MARK_H
