// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_MOUSE_EVENT_H
#define TOOLKIT_MOUSE_EVENT_H

#include "input_event.h"

#include "drawing/point.h"

namespace Toolkit {

    enum MouseEventType {
	MET_ENTERED,
	MET_EXITED,
	MET_MOVED,
	MET_DRAGGED,
	MET_PRESSED,
	MET_RELEASED,
	MET_LAST
    };

    class MouseEvent : public InputEvent {
    public:
	MouseEvent(MouseEventType type,
		const Drawing::Point & point,
		Widget * widget,
		int mask,
		int timestamp,
		int button = -1,
		int click_count = 0);
	~MouseEvent();

	MouseEventType get_type() const { return mType; }
	const Drawing::Point & get_point() const { return mPoint; }
	int get_button() const { return mButton; }
	int get_click_count() const { return mClickCount; }

    private:
	MouseEventType mType;
	Drawing::Point mPoint;
	int mButton;
	int mClickCount;
    };

} // namespace Toolkit

#endif // TOOLKIT_MOUSE_EVENT_H
