// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_SETTINGS_H
#define TOOLKIT_SETTINGS_H

#include "foundation/object.h"

#include <string>
#include <list>
using namespace std;

namespace Toolkit {

    class SettingsListener;

    class Settings : public Object {
    public:
	static void instance(Settings * settings);
	static Settings * instance();
	static void shutdown();

	virtual bool get_value(const string & key, string & value) const = 0;
	virtual bool get_value(const string & key, int & value) const = 0;
	virtual bool get_value(const string & key, bool & value) const = 0;
	virtual bool get_value(const string & key, double & value) const = 0;

	virtual void set_value(const string & key, const string & value) = 0;
	virtual void set_value(const string & key, int value) = 0;
	virtual void set_value(const string & key, bool value) = 0;
	virtual void set_value(const string & key, double value) = 0;

	void add(SettingsListener * settings_listener);
	void remove(SettingsListener * settings_listener);

    protected:
	Settings();
	~Settings();

	void value_changed(const string & key);

    private:
	list<SettingsListener *> mSettingsListeners;

	static Settings * mInstance;
    };

} // namespace Toolkit

#endif // TOOLKIT_SETTINGS_H
