// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_class.h"

#include "uml/foundation/data_types/name.h"
#include "uml/foundation/core/class.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <sstream>
using namespace std;

namespace UMLExt {

    FigClass::FigClass(UML::ClassPresentation * cp) : FigNode(cp) {
	mBaseFig = new Diagram::FigRect();
	mBaseFig->set_filled(true);
	mBaseFig->set_fill_color(get_fill_color());

	mChildren.push_front(mBaseFig);

	//

	mOutlineFig = new Diagram::FigRect();
	mOutlineFig->set_filled(false);
	mOutlineFig->set_line_color(get_line_color());

	mChildren.push_front(mOutlineFig);

	//

	mNameFig = new Diagram::FigText();

	mChildren.push_front(mNameFig);

	//

	mSeparator1Fig = new Diagram::FigHLine();
	mSeparator1Fig->set_line_color(get_line_color());

	mChildren.push_front(mSeparator1Fig);

	//

	mSeparator2Fig = new Diagram::FigHLine();
	mSeparator2Fig->set_line_color(get_line_color());

	mChildren.push_front(mSeparator2Fig);
    }

    FigClass::~FigClass() {
    }

    void
    FigClass::update_to_owner() {
        FigNode::update_to_owner();

        UML::ClassPresentation * cp = known_cast<UML::ClassPresentation *>(get_owner());

	UML::Class * c = known_cast<UML::Class *>(cp->get_subject());

	if(c->get_name()) {
	    mNameFig->set_text(c->get_name()->get_value());
	}
	else {
	    mNameFig->set_text("<unnamed>");
	}
    }

    void
    FigClass::layout() {
	mBaseFig->rebound(get_bounds());

	//

	mOutlineFig->rebound(get_bounds());

	//

	mNameFig->translate(Drawing::Vector(mBounds.get_x() + 1 - mNameFig->get_bounds().get_x(),
					    mBounds.get_y() + 1 - mNameFig->get_bounds().get_y()));
	mNameFig->set_width(mBounds.get_w() - 2);

	mNameHeight = mNameFig->get_bounds().get_h() + 4;

	mOperationsHeight = (mBounds.get_h() - mNameHeight) / 2;
	mAttributesHeight = mBounds.get_h() - mOperationsHeight - mNameHeight;

	mSeparator1Fig->translate(Drawing::Vector(mBounds.get_x() -
						  mSeparator1Fig->get_bounds().get_x(),
						  mBounds.get_y() + mNameHeight -
						  mSeparator1Fig->get_bounds().get_y()));
	mSeparator1Fig->set_width(mBounds.get_w());

	mSeparator2Fig->translate(Drawing::Vector(mBounds.get_x() -
						  mSeparator2Fig->get_bounds().get_x(),
						  mBounds.get_y() +
						  mNameHeight + mOperationsHeight -
						  mSeparator2Fig->get_bounds().get_y()));
	mSeparator2Fig->set_width(mBounds.get_w());
    }

} // namespace UMLExt
