// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "presentation_element.h"

#include "dependency_presentation.h"
#include "uml/foundation/core/model_element.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string PresentationElement::TYPE = "PresentationElement";
    
    PresentationElement::PresentationElement() {
	mSubject = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    PresentationElement::PresentationElement(const PresentationElement & presentation_element) {
	ASSERT_NOT_REACHED();
    }
    
    PresentationElement::~PresentationElement() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // MMM
    PresentationElement::internal_set_subject(ModelElement * subject) {
	ChangeEvent * change = change_role_changed(this, "subject",
						   subject, mSubject, false);
	log_change(change);
	
	change->unref();
	
	if(mSubject) {
	    mSubject->unref();
	}
	
	mSubject = subject;
	
	if(mSubject) {
	    mSubject->ref();
	}
    }
    
    void // LLL
    PresentationElement::set_subject(ModelElement * subject) {
	if(mSubject != subject) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(subject) {
		ASSERT(!ref_get_context() || subject->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &PresentationElement::set_subject,
						      subject, mSubject);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "subject", subject,
						       mSubject, false);
	    log_change(change);
	    change->unref();
	    
	    if(mSubject) {
		mSubject->internal_remove_presentation(this);
		mSubject->unref();
	    }
	    
	    mSubject = subject;
	    
	    if(mSubject) {
		mSubject->internal_add_presentation(this);
		mSubject->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // EEE
    PresentationElement::internal_add_supplier_dependency(DependencyPresentation * supplier_dependency) {
	supplier_dependency->ref();
	
	int index = mSupplierDependency.size();
	
	mSupplierDependency.push_back(supplier_dependency);
	
	ChangeEvent * change = change_role_added(this, "supplier_dependency",
						 supplier_dependency, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    PresentationElement::internal_remove_supplier_dependency(DependencyPresentation * supplier_dependency) {
	mSupplierDependency.remove(supplier_dependency);
	
	ChangeEvent * change = change_role_removed(this, "supplier_dependency",
						   supplier_dependency, -1, false);
	log_change(change);
	change->unref();
	
	supplier_dependency->unref();
    }
    
    void // BBB
    PresentationElement::set_supplier_dependency(const list<DependencyPresentation *> & supplier_dependency) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<DependencyPresentation *> before;
	set<DependencyPresentation *> after;
	
	before.insert(mSupplierDependency.begin(), mSupplierDependency.end());
	after.insert(supplier_dependency.begin(), supplier_dependency.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSupplierDependency.size());
	RETURN_IF_FAIL(after.size() == supplier_dependency.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<DependencyPresentation *> added;
	vector<DependencyPresentation *> removed;
	
	set_difference(supplier_dependency.begin(), supplier_dependency.end(),
		       mSupplierDependency.begin(), mSupplierDependency.end(),
		       back_inserter(added));
	
	set_difference(mSupplierDependency.begin(), mSupplierDependency.end(),
		       supplier_dependency.begin(), supplier_dependency.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &PresentationElement::set_supplier_dependency,
						  supplier_dependency, mSupplierDependency);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&DependencyPresentation::internal_set_supplier), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&DependencyPresentation::internal_set_supplier),
			 static_cast<PresentationElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSupplierDependency = supplier_dependency;
	
	// changes made
	
	for(vector<DependencyPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    DependencyPresentation * supplier_dependency = *iter;
	    ChangeEvent * change = change_role_added(this, "supplier_dependency",
						     supplier_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<DependencyPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    DependencyPresentation * supplier_dependency = *iter;
	    ChangeEvent * change = change_role_removed(this, "supplier_dependency",
						       supplier_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    PresentationElement::add_supplier_dependency(DependencyPresentation * supplier_dependency, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSupplierDependency.size()));
	
	if(index < 0) {
	    index = mSupplierDependency.size();
	}
	
	list<DependencyPresentation *>::iterator iter = mSupplierDependency.begin();
	
	for(int i = 0; i < static_cast<int>(mSupplierDependency.size()); ++i) {
	    RETURN_IF_FAIL(*iter != supplier_dependency);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&PresentationElement::add_supplier_dependency,
						&PresentationElement::remove_supplier_dependency,
						supplier_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "supplier_dependency", supplier_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier_dependency->ref();
	
	mSupplierDependency.insert(iter, supplier_dependency);
	supplier_dependency->internal_set_supplier(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    PresentationElement::remove_supplier_dependency(DependencyPresentation * supplier_dependency) {
	// work out the index
	
	int index = 0;
	
	list<DependencyPresentation *>::iterator iter = mSupplierDependency.begin();
	
	for(; iter != mSupplierDependency.end(); ++iter, ++index) {
	    if(*iter == supplier_dependency) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSupplierDependency.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&PresentationElement::remove_supplier_dependency,
						&PresentationElement::add_supplier_dependency,
						supplier_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "supplier_dependency", supplier_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier_dependency->internal_set_supplier(static_cast<PresentationElement *>(0));
	mSupplierDependency.erase(iter);
	
	supplier_dependency->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    PresentationElement::internal_add_client_dependency(DependencyPresentation * client_dependency) {
	client_dependency->ref();
	
	int index = mClientDependency.size();
	
	mClientDependency.push_back(client_dependency);
	
	ChangeEvent * change = change_role_added(this, "client_dependency",
						 client_dependency, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    PresentationElement::internal_remove_client_dependency(DependencyPresentation * client_dependency) {
	mClientDependency.remove(client_dependency);
	
	ChangeEvent * change = change_role_removed(this, "client_dependency",
						   client_dependency, -1, false);
	log_change(change);
	change->unref();
	
	client_dependency->unref();
    }
    
    void // BBB
    PresentationElement::set_client_dependency(const list<DependencyPresentation *> & client_dependency) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<DependencyPresentation *> before;
	set<DependencyPresentation *> after;
	
	before.insert(mClientDependency.begin(), mClientDependency.end());
	after.insert(client_dependency.begin(), client_dependency.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mClientDependency.size());
	RETURN_IF_FAIL(after.size() == client_dependency.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<DependencyPresentation *> added;
	vector<DependencyPresentation *> removed;
	
	set_difference(client_dependency.begin(), client_dependency.end(),
		       mClientDependency.begin(), mClientDependency.end(),
		       back_inserter(added));
	
	set_difference(mClientDependency.begin(), mClientDependency.end(),
		       client_dependency.begin(), client_dependency.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &PresentationElement::set_client_dependency,
						  client_dependency, mClientDependency);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&DependencyPresentation::internal_set_client), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&DependencyPresentation::internal_set_client),
			 static_cast<PresentationElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mClientDependency = client_dependency;
	
	// changes made
	
	for(vector<DependencyPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    DependencyPresentation * client_dependency = *iter;
	    ChangeEvent * change = change_role_added(this, "client_dependency",
						     client_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<DependencyPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    DependencyPresentation * client_dependency = *iter;
	    ChangeEvent * change = change_role_removed(this, "client_dependency",
						       client_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    PresentationElement::add_client_dependency(DependencyPresentation * client_dependency, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mClientDependency.size()));
	
	if(index < 0) {
	    index = mClientDependency.size();
	}
	
	list<DependencyPresentation *>::iterator iter = mClientDependency.begin();
	
	for(int i = 0; i < static_cast<int>(mClientDependency.size()); ++i) {
	    RETURN_IF_FAIL(*iter != client_dependency);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&PresentationElement::add_client_dependency,
						&PresentationElement::remove_client_dependency,
						client_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "client_dependency", client_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client_dependency->ref();
	
	mClientDependency.insert(iter, client_dependency);
	client_dependency->internal_set_client(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    PresentationElement::remove_client_dependency(DependencyPresentation * client_dependency) {
	// work out the index
	
	int index = 0;
	
	list<DependencyPresentation *>::iterator iter = mClientDependency.begin();
	
	for(; iter != mClientDependency.end(); ++iter, ++index) {
	    if(*iter == client_dependency) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mClientDependency.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&PresentationElement::remove_client_dependency,
						&PresentationElement::add_client_dependency,
						client_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "client_dependency", client_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client_dependency->internal_set_client(static_cast<PresentationElement *>(0));
	mClientDependency.erase(iter);
	
	client_dependency->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    PresentationElement::ref_set_context_impl(set<string> & visited,
					      REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    GraphElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    PresentationElement::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_presentation_element(this);
	    
	    GraphElement::ref_implant_impl(visited);
	}
    }
    
    void
    PresentationElement::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    GraphElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_presentation_element(this);
	}
    }
    
    bool
    PresentationElement::ref_add_role_impl(set<string> & visited,
					   const string & role_name,
					   REF::Class * role,
					   REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier_dependency") {
		DependencyPresentation * supplier_dependency = dynamic_cast<DependencyPresentation *>(role);
		
		if(!supplier_dependency) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_supplier_dependency(supplier_dependency, index);
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		DependencyPresentation * client_dependency = dynamic_cast<DependencyPresentation *>(role);
		
		if(!client_dependency) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_client_dependency(client_dependency, index);
		return true;
	    }
	    
	    if(GraphElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_remove_role_impl(set<string> & visited,
					      const string & role_name,
					      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_get_role_impl(set<string> & visited,
					   REF::Class * & role,
					   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "subject") {
		role = get_subject();
		return true;
	    }
	    
	    if(GraphElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_set_role_impl(set<string> & visited,
					   const string & role_name,
					   REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "subject") {
		ModelElement * subject = dynamic_cast<ModelElement *>(role);
		
		if(!subject) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_subject(subject);
		return true;
	    }
	    
	    if(GraphElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_create_class_impl(set<string> & visited,
					       REF::Class * & role,
					       const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "subject") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "supplier_dependency") {
		role = new DependencyPresentation();
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		role = new DependencyPresentation();
		return true;
	    }
	    
	    if(GraphElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_get_role_iterator_impl(set<string> & visited,
						    REF::Iterator * & iterator,
						    const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "subject") {
		iterator = REF::iterator<ModelElement>(get_subject());
		return true;
	    }
	    
	    if(role_name == "supplier_dependency") {
		iterator = REF::iterator<DependencyPresentation>(get_supplier_dependency());
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		iterator = REF::iterator<DependencyPresentation>(get_client_dependency());
		return true;
	    }
	    
	    if(GraphElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_get_attribute_impl(set<string> & visited,
						const REF::Attribute * & attribute,
						const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_set_attribute_impl(set<string> & visited,
						const string & attribute_name,
						const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_get_attribute_set_impl(set<string> & visited,
						    const vector<const REF::Attribute *> * & attribute_set,
						    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_set_attribute_set_impl(set<string> & visited,
						    const string & attribute_name,
						    const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_create_attribute_impl(set<string> & visited,
						   REF::Attribute * & attribute,
						   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GraphElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    PresentationElement::ref_is_a_type_of_impl(set<string> & visited,
					       const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "PresentationElement") {
		return true;
	    }
	    
	    if(GraphElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    PresentationElement::ref_get_type() const {
	return TYPE;
    }
    
    void
    PresentationElement::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    ModelElement * subject = 0;
	    set_subject(subject);
	    
	    list<DependencyPresentation *> supplier_dependency;
	    set_supplier_dependency(supplier_dependency);
	    
	    list<DependencyPresentation *> client_dependency;
	    set_client_dependency(client_dependency);
	    
	    GraphElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
