// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "association.h"

#include "association_end.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Association::TYPE = "Association";
    
    Association::Association() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Association::Association(const Association & association) {
	ASSERT_NOT_REACHED();
    }
    
    Association::~Association() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Association::internal_add_connection(AssociationEnd * connection) {
	connection->ref();
	
	int index = mConnection.size();
	
	mConnection.push_back(connection);
	
	ChangeEvent * change = change_role_added(this, "connection",
						 connection, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Association::internal_remove_connection(AssociationEnd * connection) {
	mConnection.remove(connection);
	
	ChangeEvent * change = change_role_removed(this, "connection",
						   connection, -1, true);
	log_change(change);
	change->unref();
	
	connection->unref();
    }
    
    void // BBB
    Association::set_connection(const list<AssociationEnd *> & connection) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<AssociationEnd *> before;
	set<AssociationEnd *> after;
	
	before.insert(mConnection.begin(), mConnection.end());
	after.insert(connection.begin(), connection.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mConnection.size());
	RETURN_IF_FAIL(after.size() == connection.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<AssociationEnd *> added;
	vector<AssociationEnd *> removed;
	
	set_difference(connection.begin(), connection.end(),
		       mConnection.begin(), mConnection.end(),
		       back_inserter(added));
	
	set_difference(mConnection.begin(), mConnection.end(),
		       connection.begin(), connection.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Association::set_connection,
						  connection, mConnection);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&AssociationEnd::internal_set_connector), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&AssociationEnd::internal_set_connector),
			 static_cast<Association *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mConnection = connection;
	
	// changes made
	
	for(vector<AssociationEnd *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    AssociationEnd * connection = *iter;
	    ChangeEvent * change = change_role_added(this, "connection",
						     connection, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<AssociationEnd *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    AssociationEnd * connection = *iter;
	    ChangeEvent * change = change_role_removed(this, "connection",
						       connection, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Association::add_connection(AssociationEnd * connection, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mConnection.size()));
	
	if(index < 0) {
	    index = mConnection.size();
	}
	
	list<AssociationEnd *>::iterator iter = mConnection.begin();
	
	for(int i = 0; i < static_cast<int>(mConnection.size()); ++i) {
	    RETURN_IF_FAIL(*iter != connection);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Association::add_connection,
						&Association::remove_connection,
						connection, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "connection", connection, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	connection->ref();
	
	mConnection.insert(iter, connection);
	connection->internal_set_connector(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Association::remove_connection(AssociationEnd * connection) {
	// work out the index
	
	int index = 0;
	
	list<AssociationEnd *>::iterator iter = mConnection.begin();
	
	for(; iter != mConnection.end(); ++iter, ++index) {
	    if(*iter == connection) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mConnection.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Association::remove_connection,
						&Association::add_connection,
						connection, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "connection", connection, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	connection->internal_set_connector(static_cast<Association *>(0));
	mConnection.erase(iter);
	
	connection->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Association::ref_set_context_impl(set<string> & visited,
				      REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_set_context_impl(visited, context);
	    Relationship::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Association::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_association(this);
	    
	    GeneralizableElement::ref_implant_impl(visited);
	    
	    Relationship::ref_implant_impl(visited);
	}
    }
    
    void
    Association::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_uproot_impl(visited);
	    
	    Relationship::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_association(this);
	}
    }
    
    bool
    Association::ref_add_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role,
				   REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connection") {
		AssociationEnd * connection = dynamic_cast<AssociationEnd *>(role);
		
		if(!connection) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_connection(connection, index);
		return true;
	    }
	    
	    if(GeneralizableElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    if(Relationship::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_remove_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    if(Relationship::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_get_role_impl(set<string> & visited,
				   REF::Class * & role,
				   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_set_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    if(Relationship::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_create_class_impl(set<string> & visited,
				       REF::Class * & role,
				       const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connection") {
		role = new AssociationEnd();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_get_role_iterator_impl(set<string> & visited,
					    REF::Iterator * & iterator,
					    const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connection") {
		iterator = REF::iterator<AssociationEnd>(get_connection());
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_get_attribute_impl(set<string> & visited,
					const REF::Attribute * & attribute,
					const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_set_attribute_impl(set<string> & visited,
					const string & attribute_name,
					const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    if(Relationship::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_get_attribute_set_impl(set<string> & visited,
					    const vector<const REF::Attribute *> * & attribute_set,
					    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_set_attribute_set_impl(set<string> & visited,
					    const string & attribute_name,
					    const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    if(Relationship::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_create_attribute_impl(set<string> & visited,
					   REF::Attribute * & attribute,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Association::ref_is_a_type_of_impl(set<string> & visited,
				       const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Association") {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    if(Relationship::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Association::ref_get_type() const {
	return TYPE;
    }
    
    void
    Association::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<AssociationEnd *> connection_list = get_connection();
	    for_each(connection_list.begin(), connection_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<AssociationEnd *> connection;
	    set_connection(connection);
	    
	    GeneralizableElement::ref_delete_impl(visited);
	    Relationship::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
