// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "association_end.h"

#include "association.h"
#include "attribute.h"
#include "classifier.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string AssociationEnd::TYPE = "AssociationEnd";
    
    AssociationEnd::AssociationEnd() {
	mConnector = 0;
	mType = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    AssociationEnd::AssociationEnd(const AssociationEnd & association_end) {
	ASSERT_NOT_REACHED();
    }
    
    AssociationEnd::~AssociationEnd() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // MMM
    AssociationEnd::internal_set_connector(Association * connector) {
	ChangeEvent * change = change_role_changed(this, "connector",
						   connector, mConnector, false);
	log_change(change);
	
	change->unref();
	
	if(mConnector) {
	    mConnector->unref();
	}
	
	mConnector = connector;
	
	if(mConnector) {
	    mConnector->ref();
	}
    }
    
    void // LLL
    AssociationEnd::set_connector(Association * connector) {
	if(mConnector != connector) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(connector) {
		ASSERT(!ref_get_context() || connector->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &AssociationEnd::set_connector,
						      connector, mConnector);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "connector", connector,
						       mConnector, false);
	    log_change(change);
	    change->unref();
	    
	    if(mConnector) {
		mConnector->internal_remove_connection(this);
		mConnector->unref();
	    }
	    
	    mConnector = connector;
	    
	    if(mConnector) {
		mConnector->internal_add_connection(this);
		mConnector->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // MMM
    AssociationEnd::internal_set_type(Classifier * type) {
	ChangeEvent * change = change_role_changed(this, "type",
						   type, mType, false);
	log_change(change);
	
	change->unref();
	
	if(mType) {
	    mType->unref();
	}
	
	mType = type;
	
	if(mType) {
	    mType->ref();
	}
    }
    
    void // LLL
    AssociationEnd::set_type(Classifier * type) {
	if(mType != type) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(type) {
		ASSERT(!type->ref_get_context() ||
		       type->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &AssociationEnd::set_type,
						      type, mType);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "type", type,
						       mType, false);
	    log_change(change);
	    change->unref();
	    
	    if(mType) {
		mType->internal_remove_association(this);
		mType->unref();
	    }
	    
	    mType = type;
	    
	    if(mType) {
		mType->internal_add_association(this);
		mType->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // EEE
    AssociationEnd::internal_add_qualifier(Attribute * qualifier) {
	qualifier->ref();
	
	int index = mQualifier.size();
	
	mQualifier.push_back(qualifier);
	
	ChangeEvent * change = change_role_added(this, "qualifier",
						 qualifier, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    AssociationEnd::internal_remove_qualifier(Attribute * qualifier) {
	mQualifier.remove(qualifier);
	
	ChangeEvent * change = change_role_removed(this, "qualifier",
						   qualifier, -1, true);
	log_change(change);
	change->unref();
	
	qualifier->unref();
    }
    
    void // BBB
    AssociationEnd::set_qualifier(const list<Attribute *> & qualifier) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Attribute *> before;
	set<Attribute *> after;
	
	before.insert(mQualifier.begin(), mQualifier.end());
	after.insert(qualifier.begin(), qualifier.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mQualifier.size());
	RETURN_IF_FAIL(after.size() == qualifier.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Attribute *> added;
	vector<Attribute *> removed;
	
	set_difference(qualifier.begin(), qualifier.end(),
		       mQualifier.begin(), mQualifier.end(),
		       back_inserter(added));
	
	set_difference(mQualifier.begin(), mQualifier.end(),
		       qualifier.begin(), qualifier.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &AssociationEnd::set_qualifier,
						  qualifier, mQualifier);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Attribute::internal_set_association_end), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Attribute::internal_set_association_end),
			 static_cast<AssociationEnd *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mQualifier = qualifier;
	
	// changes made
	
	for(vector<Attribute *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Attribute * qualifier = *iter;
	    ChangeEvent * change = change_role_added(this, "qualifier",
						     qualifier, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Attribute *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Attribute * qualifier = *iter;
	    ChangeEvent * change = change_role_removed(this, "qualifier",
						       qualifier, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    AssociationEnd::add_qualifier(Attribute * qualifier, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mQualifier.size()));
	
	if(index < 0) {
	    index = mQualifier.size();
	}
	
	list<Attribute *>::iterator iter = mQualifier.begin();
	
	for(int i = 0; i < static_cast<int>(mQualifier.size()); ++i) {
	    RETURN_IF_FAIL(*iter != qualifier);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&AssociationEnd::add_qualifier,
						&AssociationEnd::remove_qualifier,
						qualifier, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "qualifier", qualifier, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	qualifier->ref();
	
	mQualifier.insert(iter, qualifier);
	qualifier->internal_set_association_end(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    AssociationEnd::remove_qualifier(Attribute * qualifier) {
	// work out the index
	
	int index = 0;
	
	list<Attribute *>::iterator iter = mQualifier.begin();
	
	for(; iter != mQualifier.end(); ++iter, ++index) {
	    if(*iter == qualifier) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mQualifier.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&AssociationEnd::remove_qualifier,
						&AssociationEnd::add_qualifier,
						qualifier, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "qualifier", qualifier, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	qualifier->internal_set_association_end(static_cast<AssociationEnd *>(0));
	mQualifier.erase(iter);
	
	qualifier->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    AssociationEnd::ref_set_context_impl(set<string> & visited,
					 REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    AssociationEnd::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_association_end(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    AssociationEnd::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_association_end(this);
	}
    }
    
    bool
    AssociationEnd::ref_add_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role,
				      REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "qualifier") {
		Attribute * qualifier = dynamic_cast<Attribute *>(role);
		
		if(!qualifier) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_qualifier(qualifier, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_remove_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_get_role_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connector") {
		role = get_connector();
		return true;
	    }
	    
	    if(role_name == "type") {
		role = get_type();
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_set_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connector") {
		Association * connector = dynamic_cast<Association *>(role);
		
		if(!connector) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_connector(connector);
		return true;
	    }
	    
	    if(role_name == "type") {
		Classifier * type = dynamic_cast<Classifier *>(role);
		
		if(!type) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_type(type);
		return true;
	    }
	    
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_create_class_impl(set<string> & visited,
					  REF::Class * & role,
					  const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connector") {
		role = new Association();
		return true;
	    }
	    
	    if(role_name == "type") {
		role = new Classifier();
		return true;
	    }
	    
	    if(role_name == "qualifier") {
		role = new Attribute();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_get_role_iterator_impl(set<string> & visited,
					       REF::Iterator * & iterator,
					       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "connector") {
		iterator = REF::iterator<Association>(get_connector());
		return true;
	    }
	    
	    if(role_name == "type") {
		iterator = REF::iterator<Classifier>(get_type());
		return true;
	    }
	    
	    if(role_name == "qualifier") {
		iterator = REF::iterator<Attribute>(get_qualifier());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_get_attribute_impl(set<string> & visited,
					   const REF::Attribute * & attribute,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_set_attribute_impl(set<string> & visited,
					   const string & attribute_name,
					   const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_get_attribute_set_impl(set<string> & visited,
					       const vector<const REF::Attribute *> * & attribute_set,
					       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_set_attribute_set_impl(set<string> & visited,
					       const string & attribute_name,
					       const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_create_attribute_impl(set<string> & visited,
					      REF::Attribute * & attribute,
					      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    AssociationEnd::ref_is_a_type_of_impl(set<string> & visited,
					  const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "AssociationEnd") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    AssociationEnd::ref_get_type() const {
	return TYPE;
    }
    
    void
    AssociationEnd::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Attribute *> qualifier_list = get_qualifier();
	    for_each(qualifier_list.begin(), qualifier_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    Association * connector = 0;
	    set_connector(connector);
	    
	    Classifier * type = 0;
	    set_type(type);
	    
	    list<Attribute *> qualifier;
	    set_qualifier(qualifier);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
