// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "behavioral_feature.h"

#include "parameter.h"
#include "uml/foundation/data_types/boolean.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string BehavioralFeature::TYPE = "BehavioralFeature";
    
    BehavioralFeature::BehavioralFeature() {
	mIsQuery = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    BehavioralFeature::BehavioralFeature(const BehavioralFeature & behavioral_feature) {
	ASSERT_NOT_REACHED();
    }
    
    BehavioralFeature::~BehavioralFeature() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mIsQuery) {
	    mIsQuery->unref();
	}
    }
    
    void // AAA
    BehavioralFeature::set_is_query(const Boolean * is_query) {
	begin_transaction();
	
	//
	
	Boolean * old_value = mIsQuery ? new Boolean(*mIsQuery) : 0;
	Boolean * new_value = is_query ? new Boolean(*is_query) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &BehavioralFeature::set_is_query,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mIsQuery) {
	    mIsQuery->unref();
	}
	
	mIsQuery = new_value ? new Boolean(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "is_query", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // EEE
    BehavioralFeature::internal_add_parameter(Parameter * parameter) {
	parameter->ref();
	
	int index = mParameter.size();
	
	mParameter.push_back(parameter);
	
	ChangeEvent * change = change_role_added(this, "parameter",
						 parameter, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    BehavioralFeature::internal_remove_parameter(Parameter * parameter) {
	mParameter.remove(parameter);
	
	ChangeEvent * change = change_role_removed(this, "parameter",
						   parameter, -1, true);
	log_change(change);
	change->unref();
	
	parameter->unref();
    }
    
    void // BBB
    BehavioralFeature::set_parameter(const list<Parameter *> & parameter) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Parameter *> before;
	set<Parameter *> after;
	
	before.insert(mParameter.begin(), mParameter.end());
	after.insert(parameter.begin(), parameter.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mParameter.size());
	RETURN_IF_FAIL(after.size() == parameter.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Parameter *> added;
	vector<Parameter *> removed;
	
	set_difference(parameter.begin(), parameter.end(),
		       mParameter.begin(), mParameter.end(),
		       back_inserter(added));
	
	set_difference(mParameter.begin(), mParameter.end(),
		       parameter.begin(), parameter.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &BehavioralFeature::set_parameter,
						  parameter, mParameter);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Parameter::internal_set_behavioral_feature), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Parameter::internal_set_behavioral_feature),
			 static_cast<BehavioralFeature *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mParameter = parameter;
	
	// changes made
	
	for(vector<Parameter *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Parameter * parameter = *iter;
	    ChangeEvent * change = change_role_added(this, "parameter",
						     parameter, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Parameter *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Parameter * parameter = *iter;
	    ChangeEvent * change = change_role_removed(this, "parameter",
						       parameter, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    BehavioralFeature::add_parameter(Parameter * parameter, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mParameter.size()));
	
	if(index < 0) {
	    index = mParameter.size();
	}
	
	list<Parameter *>::iterator iter = mParameter.begin();
	
	for(int i = 0; i < static_cast<int>(mParameter.size()); ++i) {
	    RETURN_IF_FAIL(*iter != parameter);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&BehavioralFeature::add_parameter,
						&BehavioralFeature::remove_parameter,
						parameter, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "parameter", parameter, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	parameter->ref();
	
	mParameter.insert(iter, parameter);
	parameter->internal_set_behavioral_feature(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    BehavioralFeature::remove_parameter(Parameter * parameter) {
	// work out the index
	
	int index = 0;
	
	list<Parameter *>::iterator iter = mParameter.begin();
	
	for(; iter != mParameter.end(); ++iter, ++index) {
	    if(*iter == parameter) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mParameter.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&BehavioralFeature::remove_parameter,
						&BehavioralFeature::add_parameter,
						parameter, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "parameter", parameter, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	parameter->internal_set_behavioral_feature(static_cast<BehavioralFeature *>(0));
	mParameter.erase(iter);
	
	parameter->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    BehavioralFeature::ref_set_context_impl(set<string> & visited,
					    REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    Feature::ref_set_context_impl(visited, context);
	}
    }
    
    void
    BehavioralFeature::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_behavioral_feature(this);
	    
	    Feature::ref_implant_impl(visited);
	}
    }
    
    void
    BehavioralFeature::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    Feature::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_behavioral_feature(this);
	}
    }
    
    bool
    BehavioralFeature::ref_add_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role,
					 REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "parameter") {
		Parameter * parameter = dynamic_cast<Parameter *>(role);
		
		if(!parameter) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_parameter(parameter, index);
		return true;
	    }
	    
	    if(Feature::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_remove_role_impl(set<string> & visited,
					    const string & role_name,
					    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Feature::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_get_role_impl(set<string> & visited,
					 REF::Class * & role,
					 const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(Feature::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_set_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Feature::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_create_class_impl(set<string> & visited,
					     REF::Class * & role,
					     const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "parameter") {
		role = new Parameter();
		return true;
	    }
	    
	    if(Feature::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_get_role_iterator_impl(set<string> & visited,
						  REF::Iterator * & iterator,
						  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "parameter") {
		iterator = REF::iterator<Parameter>(get_parameter());
		return true;
	    }
	    
	    if(Feature::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_get_attribute_impl(set<string> & visited,
					      const REF::Attribute * & attribute,
					      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_query") {
		attribute = get_is_query();
		return true;
	    }
	    
	    if(Feature::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_set_attribute_impl(set<string> & visited,
					      const string & attribute_name,
					      const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_query") {
		const Boolean * is_query = dynamic_cast<const Boolean *>(attribute);
		
		if(!is_query) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_is_query(is_query);
		return true;
	    }
	    
	    if(Feature::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_get_attribute_set_impl(set<string> & visited,
						  const vector<const REF::Attribute *> * & attribute_set,
						  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Feature::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_set_attribute_set_impl(set<string> & visited,
						  const string & attribute_name,
						  const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(Feature::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_create_attribute_impl(set<string> & visited,
						 REF::Attribute * & attribute,
						 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_query") {
		attribute = new Boolean();
		return true;
	    }
	    
	    if(Feature::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    BehavioralFeature::ref_is_a_type_of_impl(set<string> & visited,
					     const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "BehavioralFeature") {
		return true;
	    }
	    
	    if(Feature::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    BehavioralFeature::ref_get_type() const {
	return TYPE;
    }
    
    void
    BehavioralFeature::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Parameter *> parameter_list = get_parameter();
	    for_each(parameter_list.begin(), parameter_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<Parameter *> parameter;
	    set_parameter(parameter);
	    
	    Feature::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
