// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "classifier.h"

#include "association_end.h"
#include "feature.h"
#include "parameter.h"
#include "structural_feature.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Classifier::TYPE = "Classifier";
    
    Classifier::Classifier() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Classifier::Classifier(const Classifier & classifier) {
	ASSERT_NOT_REACHED();
    }
    
    Classifier::~Classifier() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Classifier::internal_add_feature(Feature * feature) {
	feature->ref();
	
	int index = mFeature.size();
	
	mFeature.push_back(feature);
	
	ChangeEvent * change = change_role_added(this, "feature",
						 feature, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Classifier::internal_remove_feature(Feature * feature) {
	mFeature.remove(feature);
	
	ChangeEvent * change = change_role_removed(this, "feature",
						   feature, -1, true);
	log_change(change);
	change->unref();
	
	feature->unref();
    }
    
    void // BBB
    Classifier::set_feature(const list<Feature *> & feature) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Feature *> before;
	set<Feature *> after;
	
	before.insert(mFeature.begin(), mFeature.end());
	after.insert(feature.begin(), feature.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mFeature.size());
	RETURN_IF_FAIL(after.size() == feature.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Feature *> added;
	vector<Feature *> removed;
	
	set_difference(feature.begin(), feature.end(),
		       mFeature.begin(), mFeature.end(),
		       back_inserter(added));
	
	set_difference(mFeature.begin(), mFeature.end(),
		       feature.begin(), feature.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Classifier::set_feature,
						  feature, mFeature);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Feature::internal_set_owner), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Feature::internal_set_owner),
			 static_cast<Classifier *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mFeature = feature;
	
	// changes made
	
	for(vector<Feature *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Feature * feature = *iter;
	    ChangeEvent * change = change_role_added(this, "feature",
						     feature, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Feature *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Feature * feature = *iter;
	    ChangeEvent * change = change_role_removed(this, "feature",
						       feature, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Classifier::add_feature(Feature * feature, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mFeature.size()));
	
	if(index < 0) {
	    index = mFeature.size();
	}
	
	list<Feature *>::iterator iter = mFeature.begin();
	
	for(int i = 0; i < static_cast<int>(mFeature.size()); ++i) {
	    RETURN_IF_FAIL(*iter != feature);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::add_feature,
						&Classifier::remove_feature,
						feature, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "feature", feature, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	feature->ref();
	
	mFeature.insert(iter, feature);
	feature->internal_set_owner(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Classifier::remove_feature(Feature * feature) {
	// work out the index
	
	int index = 0;
	
	list<Feature *>::iterator iter = mFeature.begin();
	
	for(; iter != mFeature.end(); ++iter, ++index) {
	    if(*iter == feature) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mFeature.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::remove_feature,
						&Classifier::add_feature,
						feature, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "feature", feature, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	feature->internal_set_owner(static_cast<Classifier *>(0));
	mFeature.erase(iter);
	
	feature->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // BBB
    Classifier::set_typed_feature(const list<StructuralFeature *> & typed_feature) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<StructuralFeature *> before;
	set<StructuralFeature *> after;
	
	before.insert(mTypedFeature.begin(), mTypedFeature.end());
	after.insert(typed_feature.begin(), typed_feature.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mTypedFeature.size());
	RETURN_IF_FAIL(after.size() == typed_feature.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<StructuralFeature *> added;
	vector<StructuralFeature *> removed;
	
	set_difference(typed_feature.begin(), typed_feature.end(),
		       mTypedFeature.begin(), mTypedFeature.end(),
		       back_inserter(added));
	
	set_difference(mTypedFeature.begin(), mTypedFeature.end(),
		       typed_feature.begin(), typed_feature.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Classifier::set_typed_feature,
						  typed_feature, mTypedFeature);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&StructuralFeature::internal_set_type), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&StructuralFeature::internal_set_type),
			 static_cast<Classifier *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mTypedFeature = typed_feature;
	
	// changes made
	
	for(vector<StructuralFeature *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    StructuralFeature * typed_feature = *iter;
	    ChangeEvent * change = change_role_added(this, "typed_feature",
						     typed_feature, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<StructuralFeature *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    StructuralFeature * typed_feature = *iter;
	    ChangeEvent * change = change_role_removed(this, "typed_feature",
						       typed_feature, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Classifier::add_typed_feature(StructuralFeature * typed_feature, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mTypedFeature.size()));
	
	if(index < 0) {
	    index = mTypedFeature.size();
	}
	
	list<StructuralFeature *>::iterator iter = mTypedFeature.begin();
	
	for(int i = 0; i < static_cast<int>(mTypedFeature.size()); ++i) {
	    RETURN_IF_FAIL(*iter != typed_feature);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::add_typed_feature,
						&Classifier::remove_typed_feature,
						typed_feature, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "typed_feature", typed_feature, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_feature->ref();
	
	mTypedFeature.insert(iter, typed_feature);
	typed_feature->internal_set_type(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Classifier::remove_typed_feature(StructuralFeature * typed_feature) {
	// work out the index
	
	int index = 0;
	
	list<StructuralFeature *>::iterator iter = mTypedFeature.begin();
	
	for(; iter != mTypedFeature.end(); ++iter, ++index) {
	    if(*iter == typed_feature) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mTypedFeature.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::remove_typed_feature,
						&Classifier::add_typed_feature,
						typed_feature, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "typed_feature", typed_feature, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_feature->internal_set_type(static_cast<Classifier *>(0));
	mTypedFeature.erase(iter);
	
	typed_feature->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // BBB
    Classifier::set_typed_parameter(const list<Parameter *> & typed_parameter) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Parameter *> before;
	set<Parameter *> after;
	
	before.insert(mTypedParameter.begin(), mTypedParameter.end());
	after.insert(typed_parameter.begin(), typed_parameter.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mTypedParameter.size());
	RETURN_IF_FAIL(after.size() == typed_parameter.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Parameter *> added;
	vector<Parameter *> removed;
	
	set_difference(typed_parameter.begin(), typed_parameter.end(),
		       mTypedParameter.begin(), mTypedParameter.end(),
		       back_inserter(added));
	
	set_difference(mTypedParameter.begin(), mTypedParameter.end(),
		       typed_parameter.begin(), typed_parameter.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Classifier::set_typed_parameter,
						  typed_parameter, mTypedParameter);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Parameter::internal_set_type), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Parameter::internal_set_type),
			 static_cast<Classifier *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mTypedParameter = typed_parameter;
	
	// changes made
	
	for(vector<Parameter *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Parameter * typed_parameter = *iter;
	    ChangeEvent * change = change_role_added(this, "typed_parameter",
						     typed_parameter, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Parameter *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Parameter * typed_parameter = *iter;
	    ChangeEvent * change = change_role_removed(this, "typed_parameter",
						       typed_parameter, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Classifier::add_typed_parameter(Parameter * typed_parameter, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mTypedParameter.size()));
	
	if(index < 0) {
	    index = mTypedParameter.size();
	}
	
	list<Parameter *>::iterator iter = mTypedParameter.begin();
	
	for(int i = 0; i < static_cast<int>(mTypedParameter.size()); ++i) {
	    RETURN_IF_FAIL(*iter != typed_parameter);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::add_typed_parameter,
						&Classifier::remove_typed_parameter,
						typed_parameter, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "typed_parameter", typed_parameter, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_parameter->ref();
	
	mTypedParameter.insert(iter, typed_parameter);
	typed_parameter->internal_set_type(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Classifier::remove_typed_parameter(Parameter * typed_parameter) {
	// work out the index
	
	int index = 0;
	
	list<Parameter *>::iterator iter = mTypedParameter.begin();
	
	for(; iter != mTypedParameter.end(); ++iter, ++index) {
	    if(*iter == typed_parameter) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mTypedParameter.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::remove_typed_parameter,
						&Classifier::add_typed_parameter,
						typed_parameter, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "typed_parameter", typed_parameter, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_parameter->internal_set_type(static_cast<Classifier *>(0));
	mTypedParameter.erase(iter);
	
	typed_parameter->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    Classifier::internal_add_association(AssociationEnd * association) {
	association->ref();
	
	int index = mAssociation.size();
	
	mAssociation.push_back(association);
	
	ChangeEvent * change = change_role_added(this, "association",
						 association, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Classifier::internal_remove_association(AssociationEnd * association) {
	mAssociation.remove(association);
	
	ChangeEvent * change = change_role_removed(this, "association",
						   association, -1, false);
	log_change(change);
	change->unref();
	
	association->unref();
    }
    
    void // BBB
    Classifier::set_association(const list<AssociationEnd *> & association) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<AssociationEnd *> before;
	set<AssociationEnd *> after;
	
	before.insert(mAssociation.begin(), mAssociation.end());
	after.insert(association.begin(), association.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mAssociation.size());
	RETURN_IF_FAIL(after.size() == association.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<AssociationEnd *> added;
	vector<AssociationEnd *> removed;
	
	set_difference(association.begin(), association.end(),
		       mAssociation.begin(), mAssociation.end(),
		       back_inserter(added));
	
	set_difference(mAssociation.begin(), mAssociation.end(),
		       association.begin(), association.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Classifier::set_association,
						  association, mAssociation);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&AssociationEnd::internal_set_type), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&AssociationEnd::internal_set_type),
			 static_cast<Classifier *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mAssociation = association;
	
	// changes made
	
	for(vector<AssociationEnd *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    AssociationEnd * association = *iter;
	    ChangeEvent * change = change_role_added(this, "association",
						     association, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<AssociationEnd *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    AssociationEnd * association = *iter;
	    ChangeEvent * change = change_role_removed(this, "association",
						       association, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Classifier::add_association(AssociationEnd * association, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mAssociation.size()));
	
	if(index < 0) {
	    index = mAssociation.size();
	}
	
	list<AssociationEnd *>::iterator iter = mAssociation.begin();
	
	for(int i = 0; i < static_cast<int>(mAssociation.size()); ++i) {
	    RETURN_IF_FAIL(*iter != association);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::add_association,
						&Classifier::remove_association,
						association, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "association", association, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association->ref();
	
	mAssociation.insert(iter, association);
	association->internal_set_type(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Classifier::remove_association(AssociationEnd * association) {
	// work out the index
	
	int index = 0;
	
	list<AssociationEnd *>::iterator iter = mAssociation.begin();
	
	for(; iter != mAssociation.end(); ++iter, ++index) {
	    if(*iter == association) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mAssociation.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Classifier::remove_association,
						&Classifier::add_association,
						association, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "association", association, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association->internal_set_type(static_cast<Classifier *>(0));
	mAssociation.erase(iter);
	
	association->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Classifier::ref_set_context_impl(set<string> & visited,
				     REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    Namespace::ref_set_context_impl(visited, context);
	    GeneralizableElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Classifier::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_classifier(this);
	    
	    Namespace::ref_implant_impl(visited);
	    
	    GeneralizableElement::ref_implant_impl(visited);
	}
    }
    
    void
    Classifier::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    Namespace::ref_uproot_impl(visited);
	    
	    GeneralizableElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_classifier(this);
	}
    }
    
    bool
    Classifier::ref_add_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role,
				  REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "feature") {
		Feature * feature = dynamic_cast<Feature *>(role);
		
		if(!feature) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_feature(feature, index);
		return true;
	    }
	    
	    if(role_name == "typed_feature") {
		StructuralFeature * typed_feature = dynamic_cast<StructuralFeature *>(role);
		
		if(!typed_feature) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_typed_feature(typed_feature, index);
		return true;
	    }
	    
	    if(role_name == "typed_parameter") {
		Parameter * typed_parameter = dynamic_cast<Parameter *>(role);
		
		if(!typed_parameter) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_typed_parameter(typed_parameter, index);
		return true;
	    }
	    
	    if(role_name == "association") {
		AssociationEnd * association = dynamic_cast<AssociationEnd *>(role);
		
		if(!association) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_association(association, index);
		return true;
	    }
	    
	    if(Namespace::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_remove_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_get_role_impl(set<string> & visited,
				  REF::Class * & role,
				  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_set_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_create_class_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "feature") {
		role = new Feature();
		return true;
	    }
	    
	    if(role_name == "typed_feature") {
		role = new StructuralFeature();
		return true;
	    }
	    
	    if(role_name == "typed_parameter") {
		role = new Parameter();
		return true;
	    }
	    
	    if(role_name == "association") {
		role = new AssociationEnd();
		return true;
	    }
	    
	    if(Namespace::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_get_role_iterator_impl(set<string> & visited,
					   REF::Iterator * & iterator,
					   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "feature") {
		iterator = REF::iterator<Feature>(get_feature());
		return true;
	    }
	    
	    if(role_name == "typed_feature") {
		iterator = REF::iterator<StructuralFeature>(get_typed_feature());
		return true;
	    }
	    
	    if(role_name == "typed_parameter") {
		iterator = REF::iterator<Parameter>(get_typed_parameter());
		return true;
	    }
	    
	    if(role_name == "association") {
		iterator = REF::iterator<AssociationEnd>(get_association());
		return true;
	    }
	    
	    if(Namespace::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_get_attribute_impl(set<string> & visited,
				       const REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_set_attribute_impl(set<string> & visited,
				       const string & attribute_name,
				       const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_get_attribute_set_impl(set<string> & visited,
					   const vector<const REF::Attribute *> * & attribute_set,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_set_attribute_set_impl(set<string> & visited,
					   const string & attribute_name,
					   const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_create_attribute_impl(set<string> & visited,
					  REF::Attribute * & attribute,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Namespace::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Classifier::ref_is_a_type_of_impl(set<string> & visited,
				      const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Classifier") {
		return true;
	    }
	    
	    if(Namespace::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Classifier::ref_get_type() const {
	return TYPE;
    }
    
    void
    Classifier::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Feature *> feature_list = get_feature();
	    for_each(feature_list.begin(), feature_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<Feature *> feature;
	    set_feature(feature);
	    
	    list<StructuralFeature *> typed_feature;
	    set_typed_feature(typed_feature);
	    
	    list<Parameter *> typed_parameter;
	    set_typed_parameter(typed_parameter);
	    
	    list<AssociationEnd *> association;
	    set_association(association);
	    
	    Namespace::ref_delete_impl(visited);
	    GeneralizableElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
