// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "comment.h"

#include "model_element.h"
#include "uml/foundation/data_types/name.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Comment::TYPE = "Comment";
    
    Comment::Comment() {
	mBody = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Comment::Comment(const Comment & comment) {
	ASSERT_NOT_REACHED();
    }
    
    Comment::~Comment() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mBody) {
	    mBody->unref();
	}
    }
    
    void // AAA
    Comment::set_body(const Name * body) {
	begin_transaction();
	
	//
	
	Name * old_value = mBody ? new Name(*mBody) : 0;
	Name * new_value = body ? new Name(*body) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &Comment::set_body,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mBody) {
	    mBody->unref();
	}
	
	mBody = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "body", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // EEE
    Comment::internal_add_annotated_element(ModelElement * annotated_element) {
	annotated_element->ref();
	
	int index = mAnnotatedElement.size();
	
	mAnnotatedElement.push_back(annotated_element);
	
	ChangeEvent * change = change_role_added(this, "annotated_element",
						 annotated_element, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Comment::internal_remove_annotated_element(ModelElement * annotated_element) {
	mAnnotatedElement.remove(annotated_element);
	
	ChangeEvent * change = change_role_removed(this, "annotated_element",
						   annotated_element, -1, false);
	log_change(change);
	change->unref();
	
	annotated_element->unref();
    }
    
    void // BBB
    Comment::set_annotated_element(const list<ModelElement *> & annotated_element) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mAnnotatedElement.begin(), mAnnotatedElement.end());
	after.insert(annotated_element.begin(), annotated_element.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mAnnotatedElement.size());
	RETURN_IF_FAIL(after.size() == annotated_element.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(annotated_element.begin(), annotated_element.end(),
		       mAnnotatedElement.begin(), mAnnotatedElement.end(),
		       back_inserter(added));
	
	set_difference(mAnnotatedElement.begin(), mAnnotatedElement.end(),
		       annotated_element.begin(), annotated_element.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Comment::set_annotated_element,
						  annotated_element, mAnnotatedElement);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_comment), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_comment), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mAnnotatedElement = annotated_element;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * annotated_element = *iter;
	    ChangeEvent * change = change_role_added(this, "annotated_element",
						     annotated_element, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * annotated_element = *iter;
	    ChangeEvent * change = change_role_removed(this, "annotated_element",
						       annotated_element, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Comment::add_annotated_element(ModelElement * annotated_element, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mAnnotatedElement.size()));
	
	if(index < 0) {
	    index = mAnnotatedElement.size();
	}
	
	list<ModelElement *>::iterator iter = mAnnotatedElement.begin();
	
	for(int i = 0; i < static_cast<int>(mAnnotatedElement.size()); ++i) {
	    RETURN_IF_FAIL(*iter != annotated_element);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Comment::add_annotated_element,
						&Comment::remove_annotated_element,
						annotated_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "annotated_element", annotated_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	annotated_element->ref();
	
	mAnnotatedElement.insert(iter, annotated_element);
	annotated_element->internal_add_comment(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Comment::remove_annotated_element(ModelElement * annotated_element) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mAnnotatedElement.begin();
	
	for(; iter != mAnnotatedElement.end(); ++iter, ++index) {
	    if(*iter == annotated_element) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mAnnotatedElement.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Comment::remove_annotated_element,
						&Comment::add_annotated_element,
						annotated_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "annotated_element", annotated_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	annotated_element->internal_remove_comment(this);
	mAnnotatedElement.erase(iter);
	
	annotated_element->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Comment::ref_set_context_impl(set<string> & visited,
				  REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Comment::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_comment(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Comment::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_comment(this);
	}
    }
    
    bool
    Comment::ref_add_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role,
			       REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "annotated_element") {
		ModelElement * annotated_element = dynamic_cast<ModelElement *>(role);
		
		if(!annotated_element) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_annotated_element(annotated_element, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_remove_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_get_role_impl(set<string> & visited,
			       REF::Class * & role,
			       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_set_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_create_class_impl(set<string> & visited,
				   REF::Class * & role,
				   const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "annotated_element") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_get_role_iterator_impl(set<string> & visited,
					REF::Iterator * & iterator,
					const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "annotated_element") {
		iterator = REF::iterator<ModelElement>(get_annotated_element());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_get_attribute_impl(set<string> & visited,
				    const REF::Attribute * & attribute,
				    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "body") {
		attribute = get_body();
		return true;
	    }
	    
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_set_attribute_impl(set<string> & visited,
				    const string & attribute_name,
				    const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "body") {
		const Name * body = dynamic_cast<const Name *>(attribute);
		
		if(!body) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_body(body);
		return true;
	    }
	    
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_get_attribute_set_impl(set<string> & visited,
					const vector<const REF::Attribute *> * & attribute_set,
					const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_set_attribute_set_impl(set<string> & visited,
					const string & attribute_name,
					const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_create_attribute_impl(set<string> & visited,
				       REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "body") {
		attribute = new Name();
		return true;
	    }
	    
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Comment::ref_is_a_type_of_impl(set<string> & visited,
				   const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Comment") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Comment::ref_get_type() const {
	return TYPE;
    }
    
    void
    Comment::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<ModelElement *> annotated_element;
	    set_annotated_element(annotated_element);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
