// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "constraint.h"

#include "model_element.h"
#include "uml/foundation/extension_mechanisms/stereotype.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Constraint::TYPE = "Constraint";
    
    Constraint::Constraint() {
	mConstrainedStereotype = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Constraint::Constraint(const Constraint & constraint) {
	ASSERT_NOT_REACHED();
    }
    
    Constraint::~Constraint() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Constraint::internal_add_constrained_element(ModelElement * constrained_element) {
	constrained_element->ref();
	
	int index = mConstrainedElement.size();
	
	mConstrainedElement.push_back(constrained_element);
	
	ChangeEvent * change = change_role_added(this, "constrained_element",
						 constrained_element, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Constraint::internal_remove_constrained_element(ModelElement * constrained_element) {
	mConstrainedElement.remove(constrained_element);
	
	ChangeEvent * change = change_role_removed(this, "constrained_element",
						   constrained_element, -1, false);
	log_change(change);
	change->unref();
	
	constrained_element->unref();
    }
    
    void // BBB
    Constraint::set_constrained_element(const list<ModelElement *> & constrained_element) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mConstrainedElement.begin(), mConstrainedElement.end());
	after.insert(constrained_element.begin(), constrained_element.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mConstrainedElement.size());
	RETURN_IF_FAIL(after.size() == constrained_element.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(constrained_element.begin(), constrained_element.end(),
		       mConstrainedElement.begin(), mConstrainedElement.end(),
		       back_inserter(added));
	
	set_difference(mConstrainedElement.begin(), mConstrainedElement.end(),
		       constrained_element.begin(), constrained_element.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Constraint::set_constrained_element,
						  constrained_element, mConstrainedElement);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_constraint), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_constraint), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mConstrainedElement = constrained_element;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * constrained_element = *iter;
	    ChangeEvent * change = change_role_added(this, "constrained_element",
						     constrained_element, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * constrained_element = *iter;
	    ChangeEvent * change = change_role_removed(this, "constrained_element",
						       constrained_element, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Constraint::add_constrained_element(ModelElement * constrained_element, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mConstrainedElement.size()));
	
	if(index < 0) {
	    index = mConstrainedElement.size();
	}
	
	list<ModelElement *>::iterator iter = mConstrainedElement.begin();
	
	for(int i = 0; i < static_cast<int>(mConstrainedElement.size()); ++i) {
	    RETURN_IF_FAIL(*iter != constrained_element);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Constraint::add_constrained_element,
						&Constraint::remove_constrained_element,
						constrained_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "constrained_element", constrained_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	constrained_element->ref();
	
	mConstrainedElement.insert(iter, constrained_element);
	constrained_element->internal_add_constraint(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Constraint::remove_constrained_element(ModelElement * constrained_element) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mConstrainedElement.begin();
	
	for(; iter != mConstrainedElement.end(); ++iter, ++index) {
	    if(*iter == constrained_element) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mConstrainedElement.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Constraint::remove_constrained_element,
						&Constraint::add_constrained_element,
						constrained_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "constrained_element", constrained_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	constrained_element->internal_remove_constraint(this);
	mConstrainedElement.erase(iter);
	
	constrained_element->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // MMM
    Constraint::internal_set_constrained_stereotype(Stereotype * constrained_stereotype) {
	ChangeEvent * change = change_role_changed(this, "constrained_stereotype",
						   constrained_stereotype, mConstrainedStereotype, false);
	log_change(change);
	
	change->unref();
	
	if(mConstrainedStereotype) {
	    mConstrainedStereotype->unref();
	}
	
	mConstrainedStereotype = constrained_stereotype;
	
	if(mConstrainedStereotype) {
	    mConstrainedStereotype->ref();
	}
    }
    
    void // LLL
    Constraint::set_constrained_stereotype(Stereotype * constrained_stereotype) {
	if(mConstrainedStereotype != constrained_stereotype) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(constrained_stereotype) {
		ASSERT(!ref_get_context() || constrained_stereotype->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &Constraint::set_constrained_stereotype,
						      constrained_stereotype, mConstrainedStereotype);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "constrained_stereotype", constrained_stereotype,
						       mConstrainedStereotype, false);
	    log_change(change);
	    change->unref();
	    
	    if(mConstrainedStereotype) {
		mConstrainedStereotype->internal_remove_stereotype_constraint(this);
		mConstrainedStereotype->unref();
	    }
	    
	    mConstrainedStereotype = constrained_stereotype;
	    
	    if(mConstrainedStereotype) {
		mConstrainedStereotype->internal_add_stereotype_constraint(this);
		mConstrainedStereotype->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void
    Constraint::ref_set_context_impl(set<string> & visited,
				     REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Constraint::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_constraint(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Constraint::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_constraint(this);
	}
    }
    
    bool
    Constraint::ref_add_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role,
				  REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constrained_element") {
		ModelElement * constrained_element = dynamic_cast<ModelElement *>(role);
		
		if(!constrained_element) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_constrained_element(constrained_element, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_remove_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_get_role_impl(set<string> & visited,
				  REF::Class * & role,
				  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constrained_stereotype") {
		role = get_constrained_stereotype();
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_set_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constrained_stereotype") {
		Stereotype * constrained_stereotype = dynamic_cast<Stereotype *>(role);
		
		if(!constrained_stereotype) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_constrained_stereotype(constrained_stereotype);
		return true;
	    }
	    
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_create_class_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constrained_element") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "constrained_stereotype") {
		role = new Stereotype();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_get_role_iterator_impl(set<string> & visited,
					   REF::Iterator * & iterator,
					   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constrained_element") {
		iterator = REF::iterator<ModelElement>(get_constrained_element());
		return true;
	    }
	    
	    if(role_name == "constrained_stereotype") {
		iterator = REF::iterator<Stereotype>(get_constrained_stereotype());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_get_attribute_impl(set<string> & visited,
				       const REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_set_attribute_impl(set<string> & visited,
				       const string & attribute_name,
				       const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_get_attribute_set_impl(set<string> & visited,
					   const vector<const REF::Attribute *> * & attribute_set,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_set_attribute_set_impl(set<string> & visited,
					   const string & attribute_name,
					   const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_create_attribute_impl(set<string> & visited,
					  REF::Attribute * & attribute,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Constraint::ref_is_a_type_of_impl(set<string> & visited,
				      const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Constraint") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Constraint::ref_get_type() const {
	return TYPE;
    }
    
    void
    Constraint::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<ModelElement *> constrained_element;
	    set_constrained_element(constrained_element);
	    
	    Stereotype * constrained_stereotype = 0;
	    set_constrained_stereotype(constrained_stereotype);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
