// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "dependency.h"

#include "model_element.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Dependency::TYPE = "Dependency";
    
    Dependency::Dependency() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Dependency::Dependency(const Dependency & dependency) {
	ASSERT_NOT_REACHED();
    }
    
    Dependency::~Dependency() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Dependency::internal_add_supplier(ModelElement * supplier) {
	supplier->ref();
	
	int index = mSupplier.size();
	
	mSupplier.push_back(supplier);
	
	ChangeEvent * change = change_role_added(this, "supplier",
						 supplier, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Dependency::internal_remove_supplier(ModelElement * supplier) {
	mSupplier.remove(supplier);
	
	ChangeEvent * change = change_role_removed(this, "supplier",
						   supplier, -1, false);
	log_change(change);
	change->unref();
	
	supplier->unref();
    }
    
    void // BBB
    Dependency::set_supplier(const list<ModelElement *> & supplier) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mSupplier.begin(), mSupplier.end());
	after.insert(supplier.begin(), supplier.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSupplier.size());
	RETURN_IF_FAIL(after.size() == supplier.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(supplier.begin(), supplier.end(),
		       mSupplier.begin(), mSupplier.end(),
		       back_inserter(added));
	
	set_difference(mSupplier.begin(), mSupplier.end(),
		       supplier.begin(), supplier.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Dependency::set_supplier,
						  supplier, mSupplier);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_supplier_dependency), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_supplier_dependency), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSupplier = supplier;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * supplier = *iter;
	    ChangeEvent * change = change_role_added(this, "supplier",
						     supplier, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * supplier = *iter;
	    ChangeEvent * change = change_role_removed(this, "supplier",
						       supplier, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Dependency::add_supplier(ModelElement * supplier, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSupplier.size()));
	
	if(index < 0) {
	    index = mSupplier.size();
	}
	
	list<ModelElement *>::iterator iter = mSupplier.begin();
	
	for(int i = 0; i < static_cast<int>(mSupplier.size()); ++i) {
	    RETURN_IF_FAIL(*iter != supplier);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Dependency::add_supplier,
						&Dependency::remove_supplier,
						supplier, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "supplier", supplier, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier->ref();
	
	mSupplier.insert(iter, supplier);
	supplier->internal_add_supplier_dependency(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Dependency::remove_supplier(ModelElement * supplier) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mSupplier.begin();
	
	for(; iter != mSupplier.end(); ++iter, ++index) {
	    if(*iter == supplier) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSupplier.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Dependency::remove_supplier,
						&Dependency::add_supplier,
						supplier, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "supplier", supplier, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier->internal_remove_supplier_dependency(this);
	mSupplier.erase(iter);
	
	supplier->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    Dependency::internal_add_client(ModelElement * client) {
	client->ref();
	
	int index = mClient.size();
	
	mClient.push_back(client);
	
	ChangeEvent * change = change_role_added(this, "client",
						 client, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Dependency::internal_remove_client(ModelElement * client) {
	mClient.remove(client);
	
	ChangeEvent * change = change_role_removed(this, "client",
						   client, -1, false);
	log_change(change);
	change->unref();
	
	client->unref();
    }
    
    void // BBB
    Dependency::set_client(const list<ModelElement *> & client) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mClient.begin(), mClient.end());
	after.insert(client.begin(), client.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mClient.size());
	RETURN_IF_FAIL(after.size() == client.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(client.begin(), client.end(),
		       mClient.begin(), mClient.end(),
		       back_inserter(added));
	
	set_difference(mClient.begin(), mClient.end(),
		       client.begin(), client.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Dependency::set_client,
						  client, mClient);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_client_dependency), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_client_dependency), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mClient = client;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * client = *iter;
	    ChangeEvent * change = change_role_added(this, "client",
						     client, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * client = *iter;
	    ChangeEvent * change = change_role_removed(this, "client",
						       client, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Dependency::add_client(ModelElement * client, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mClient.size()));
	
	if(index < 0) {
	    index = mClient.size();
	}
	
	list<ModelElement *>::iterator iter = mClient.begin();
	
	for(int i = 0; i < static_cast<int>(mClient.size()); ++i) {
	    RETURN_IF_FAIL(*iter != client);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Dependency::add_client,
						&Dependency::remove_client,
						client, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "client", client, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client->ref();
	
	mClient.insert(iter, client);
	client->internal_add_client_dependency(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Dependency::remove_client(ModelElement * client) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mClient.begin();
	
	for(; iter != mClient.end(); ++iter, ++index) {
	    if(*iter == client) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mClient.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Dependency::remove_client,
						&Dependency::add_client,
						client, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "client", client, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client->internal_remove_client_dependency(this);
	mClient.erase(iter);
	
	client->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Dependency::ref_set_context_impl(set<string> & visited,
				     REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    Relationship::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Dependency::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_dependency(this);
	    
	    Relationship::ref_implant_impl(visited);
	}
    }
    
    void
    Dependency::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    Relationship::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_dependency(this);
	}
    }
    
    bool
    Dependency::ref_add_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role,
				  REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		ModelElement * supplier = dynamic_cast<ModelElement *>(role);
		
		if(!supplier) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_supplier(supplier, index);
		return true;
	    }
	    
	    if(role_name == "client") {
		ModelElement * client = dynamic_cast<ModelElement *>(role);
		
		if(!client) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_client(client, index);
		return true;
	    }
	    
	    if(Relationship::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_remove_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_get_role_impl(set<string> & visited,
				  REF::Class * & role,
				  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_set_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_create_class_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "client") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(Relationship::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_get_role_iterator_impl(set<string> & visited,
					   REF::Iterator * & iterator,
					   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		iterator = REF::iterator<ModelElement>(get_supplier());
		return true;
	    }
	    
	    if(role_name == "client") {
		iterator = REF::iterator<ModelElement>(get_client());
		return true;
	    }
	    
	    if(Relationship::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_get_attribute_impl(set<string> & visited,
				       const REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_set_attribute_impl(set<string> & visited,
				       const string & attribute_name,
				       const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_get_attribute_set_impl(set<string> & visited,
					   const vector<const REF::Attribute *> * & attribute_set,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_set_attribute_set_impl(set<string> & visited,
					   const string & attribute_name,
					   const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_create_attribute_impl(set<string> & visited,
					  REF::Attribute * & attribute,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Dependency::ref_is_a_type_of_impl(set<string> & visited,
				      const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Dependency") {
		return true;
	    }
	    
	    if(Relationship::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Dependency::ref_get_type() const {
	return TYPE;
    }
    
    void
    Dependency::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<ModelElement *> supplier;
	    set_supplier(supplier);
	    
	    list<ModelElement *> client;
	    set_client(client);
	    
	    Relationship::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
