// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "generalizable_element.h"

#include "generalization.h"
#include "uml/foundation/data_types/boolean.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string GeneralizableElement::TYPE = "GeneralizableElement";
    
    GeneralizableElement::GeneralizableElement() {
	mIsRoot = 0;
	mIsLeaf = 0;
	mIsAbstract = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    GeneralizableElement::GeneralizableElement(const GeneralizableElement & generalizable_element) {
	ASSERT_NOT_REACHED();
    }
    
    GeneralizableElement::~GeneralizableElement() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mIsRoot) {
	    mIsRoot->unref();
	}
	if(mIsLeaf) {
	    mIsLeaf->unref();
	}
	if(mIsAbstract) {
	    mIsAbstract->unref();
	}
    }
    
    void // AAA
    GeneralizableElement::set_is_root(const Boolean * is_root) {
	begin_transaction();
	
	//
	
	Boolean * old_value = mIsRoot ? new Boolean(*mIsRoot) : 0;
	Boolean * new_value = is_root ? new Boolean(*is_root) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &GeneralizableElement::set_is_root,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mIsRoot) {
	    mIsRoot->unref();
	}
	
	mIsRoot = new_value ? new Boolean(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "is_root", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // AAA
    GeneralizableElement::set_is_leaf(const Boolean * is_leaf) {
	begin_transaction();
	
	//
	
	Boolean * old_value = mIsLeaf ? new Boolean(*mIsLeaf) : 0;
	Boolean * new_value = is_leaf ? new Boolean(*is_leaf) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &GeneralizableElement::set_is_leaf,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mIsLeaf) {
	    mIsLeaf->unref();
	}
	
	mIsLeaf = new_value ? new Boolean(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "is_leaf", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // AAA
    GeneralizableElement::set_is_abstract(const Boolean * is_abstract) {
	begin_transaction();
	
	//
	
	Boolean * old_value = mIsAbstract ? new Boolean(*mIsAbstract) : 0;
	Boolean * new_value = is_abstract ? new Boolean(*is_abstract) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &GeneralizableElement::set_is_abstract,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mIsAbstract) {
	    mIsAbstract->unref();
	}
	
	mIsAbstract = new_value ? new Boolean(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "is_abstract", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // EEE
    GeneralizableElement::internal_add_generalization(Generalization * generalization) {
	generalization->ref();
	
	int index = mGeneralization.size();
	
	mGeneralization.push_back(generalization);
	
	ChangeEvent * change = change_role_added(this, "generalization",
						 generalization, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    GeneralizableElement::internal_remove_generalization(Generalization * generalization) {
	mGeneralization.remove(generalization);
	
	ChangeEvent * change = change_role_removed(this, "generalization",
						   generalization, -1, false);
	log_change(change);
	change->unref();
	
	generalization->unref();
    }
    
    void // BBB
    GeneralizableElement::set_generalization(const list<Generalization *> & generalization) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Generalization *> before;
	set<Generalization *> after;
	
	before.insert(mGeneralization.begin(), mGeneralization.end());
	after.insert(generalization.begin(), generalization.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mGeneralization.size());
	RETURN_IF_FAIL(after.size() == generalization.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Generalization *> added;
	vector<Generalization *> removed;
	
	set_difference(generalization.begin(), generalization.end(),
		       mGeneralization.begin(), mGeneralization.end(),
		       back_inserter(added));
	
	set_difference(mGeneralization.begin(), mGeneralization.end(),
		       generalization.begin(), generalization.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &GeneralizableElement::set_generalization,
						  generalization, mGeneralization);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Generalization::internal_set_child), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Generalization::internal_set_child),
			 static_cast<GeneralizableElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mGeneralization = generalization;
	
	// changes made
	
	for(vector<Generalization *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Generalization * generalization = *iter;
	    ChangeEvent * change = change_role_added(this, "generalization",
						     generalization, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Generalization *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Generalization * generalization = *iter;
	    ChangeEvent * change = change_role_removed(this, "generalization",
						       generalization, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    GeneralizableElement::add_generalization(Generalization * generalization, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mGeneralization.size()));
	
	if(index < 0) {
	    index = mGeneralization.size();
	}
	
	list<Generalization *>::iterator iter = mGeneralization.begin();
	
	for(int i = 0; i < static_cast<int>(mGeneralization.size()); ++i) {
	    RETURN_IF_FAIL(*iter != generalization);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&GeneralizableElement::add_generalization,
						&GeneralizableElement::remove_generalization,
						generalization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "generalization", generalization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	generalization->ref();
	
	mGeneralization.insert(iter, generalization);
	generalization->internal_set_child(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    GeneralizableElement::remove_generalization(Generalization * generalization) {
	// work out the index
	
	int index = 0;
	
	list<Generalization *>::iterator iter = mGeneralization.begin();
	
	for(; iter != mGeneralization.end(); ++iter, ++index) {
	    if(*iter == generalization) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mGeneralization.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&GeneralizableElement::remove_generalization,
						&GeneralizableElement::add_generalization,
						generalization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "generalization", generalization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	generalization->internal_set_child(static_cast<GeneralizableElement *>(0));
	mGeneralization.erase(iter);
	
	generalization->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    GeneralizableElement::internal_add_specialization(Generalization * specialization) {
	specialization->ref();
	
	int index = mSpecialization.size();
	
	mSpecialization.push_back(specialization);
	
	ChangeEvent * change = change_role_added(this, "specialization",
						 specialization, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    GeneralizableElement::internal_remove_specialization(Generalization * specialization) {
	mSpecialization.remove(specialization);
	
	ChangeEvent * change = change_role_removed(this, "specialization",
						   specialization, -1, false);
	log_change(change);
	change->unref();
	
	specialization->unref();
    }
    
    void // BBB
    GeneralizableElement::set_specialization(const list<Generalization *> & specialization) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Generalization *> before;
	set<Generalization *> after;
	
	before.insert(mSpecialization.begin(), mSpecialization.end());
	after.insert(specialization.begin(), specialization.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSpecialization.size());
	RETURN_IF_FAIL(after.size() == specialization.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Generalization *> added;
	vector<Generalization *> removed;
	
	set_difference(specialization.begin(), specialization.end(),
		       mSpecialization.begin(), mSpecialization.end(),
		       back_inserter(added));
	
	set_difference(mSpecialization.begin(), mSpecialization.end(),
		       specialization.begin(), specialization.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &GeneralizableElement::set_specialization,
						  specialization, mSpecialization);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Generalization::internal_set_parent), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Generalization::internal_set_parent),
			 static_cast<GeneralizableElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSpecialization = specialization;
	
	// changes made
	
	for(vector<Generalization *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Generalization * specialization = *iter;
	    ChangeEvent * change = change_role_added(this, "specialization",
						     specialization, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Generalization *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Generalization * specialization = *iter;
	    ChangeEvent * change = change_role_removed(this, "specialization",
						       specialization, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    GeneralizableElement::add_specialization(Generalization * specialization, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSpecialization.size()));
	
	if(index < 0) {
	    index = mSpecialization.size();
	}
	
	list<Generalization *>::iterator iter = mSpecialization.begin();
	
	for(int i = 0; i < static_cast<int>(mSpecialization.size()); ++i) {
	    RETURN_IF_FAIL(*iter != specialization);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&GeneralizableElement::add_specialization,
						&GeneralizableElement::remove_specialization,
						specialization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "specialization", specialization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specialization->ref();
	
	mSpecialization.insert(iter, specialization);
	specialization->internal_set_parent(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    GeneralizableElement::remove_specialization(Generalization * specialization) {
	// work out the index
	
	int index = 0;
	
	list<Generalization *>::iterator iter = mSpecialization.begin();
	
	for(; iter != mSpecialization.end(); ++iter, ++index) {
	    if(*iter == specialization) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSpecialization.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&GeneralizableElement::remove_specialization,
						&GeneralizableElement::add_specialization,
						specialization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "specialization", specialization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specialization->internal_set_parent(static_cast<GeneralizableElement *>(0));
	mSpecialization.erase(iter);
	
	specialization->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    GeneralizableElement::ref_set_context_impl(set<string> & visited,
					       REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    GeneralizableElement::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_generalizable_element(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    GeneralizableElement::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_generalizable_element(this);
	}
    }
    
    bool
    GeneralizableElement::ref_add_role_impl(set<string> & visited,
					    const string & role_name,
					    REF::Class * role,
					    REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "generalization") {
		Generalization * generalization = dynamic_cast<Generalization *>(role);
		
		if(!generalization) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_generalization(generalization, index);
		return true;
	    }
	    
	    if(role_name == "specialization") {
		Generalization * specialization = dynamic_cast<Generalization *>(role);
		
		if(!specialization) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_specialization(specialization, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_remove_role_impl(set<string> & visited,
					       const string & role_name,
					       REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_get_role_impl(set<string> & visited,
					    REF::Class * & role,
					    const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_set_role_impl(set<string> & visited,
					    const string & role_name,
					    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_create_class_impl(set<string> & visited,
						REF::Class * & role,
						const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "generalization") {
		role = new Generalization();
		return true;
	    }
	    
	    if(role_name == "specialization") {
		role = new Generalization();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_get_role_iterator_impl(set<string> & visited,
						     REF::Iterator * & iterator,
						     const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "generalization") {
		iterator = REF::iterator<Generalization>(get_generalization());
		return true;
	    }
	    
	    if(role_name == "specialization") {
		iterator = REF::iterator<Generalization>(get_specialization());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_get_attribute_impl(set<string> & visited,
						 const REF::Attribute * & attribute,
						 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_root") {
		attribute = get_is_root();
		return true;
	    }
	    
	    if(attribute_name == "is_leaf") {
		attribute = get_is_leaf();
		return true;
	    }
	    
	    if(attribute_name == "is_abstract") {
		attribute = get_is_abstract();
		return true;
	    }
	    
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_set_attribute_impl(set<string> & visited,
						 const string & attribute_name,
						 const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_root") {
		const Boolean * is_root = dynamic_cast<const Boolean *>(attribute);
		
		if(!is_root) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_is_root(is_root);
		return true;
	    }
	    
	    if(attribute_name == "is_leaf") {
		const Boolean * is_leaf = dynamic_cast<const Boolean *>(attribute);
		
		if(!is_leaf) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_is_leaf(is_leaf);
		return true;
	    }
	    
	    if(attribute_name == "is_abstract") {
		const Boolean * is_abstract = dynamic_cast<const Boolean *>(attribute);
		
		if(!is_abstract) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_is_abstract(is_abstract);
		return true;
	    }
	    
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_get_attribute_set_impl(set<string> & visited,
						     const vector<const REF::Attribute *> * & attribute_set,
						     const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_set_attribute_set_impl(set<string> & visited,
						     const string & attribute_name,
						     const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_create_attribute_impl(set<string> & visited,
						    REF::Attribute * & attribute,
						    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "is_root") {
		attribute = new Boolean();
		return true;
	    }
	    
	    if(attribute_name == "is_leaf") {
		attribute = new Boolean();
		return true;
	    }
	    
	    if(attribute_name == "is_abstract") {
		attribute = new Boolean();
		return true;
	    }
	    
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    GeneralizableElement::ref_is_a_type_of_impl(set<string> & visited,
						const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "GeneralizableElement") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    GeneralizableElement::ref_get_type() const {
	return TYPE;
    }
    
    void
    GeneralizableElement::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Generalization *> generalization;
	    set_generalization(generalization);
	    
	    list<Generalization *> specialization;
	    set_specialization(specialization);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
