// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "model_element.h"

#include "comment.h"
#include "constraint.h"
#include "dependency.h"
#include "namespace.h"
#include "uml/diagram/presentation_element.h"
#include "uml/foundation/data_types/boolean.h"
#include "uml/foundation/data_types/name.h"
#include "uml/foundation/extension_mechanisms/stereotype.h"
#include "uml/foundation/extension_mechanisms/tagged_value.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string ModelElement::TYPE = "ModelElement";
    
    ModelElement::ModelElement() {
	mNamespace = 0;
	mName = 0;
	mIsSpecification = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    ModelElement::ModelElement(const ModelElement & model_element) {
	ASSERT_NOT_REACHED();
    }
    
    ModelElement::~ModelElement() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mName) {
	    mName->unref();
	}
	if(mIsSpecification) {
	    mIsSpecification->unref();
	}
    }
    
    void // AAA
    ModelElement::set_name(const Name * name) {
	begin_transaction();
	
	//
	
	Name * old_value = mName ? new Name(*mName) : 0;
	Name * new_value = name ? new Name(*name) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &ModelElement::set_name,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mName) {
	    mName->unref();
	}
	
	mName = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "name", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // AAA
    ModelElement::set_is_specification(const Boolean * is_specification) {
	begin_transaction();
	
	//
	
	Boolean * old_value = mIsSpecification ? new Boolean(*mIsSpecification) : 0;
	Boolean * new_value = is_specification ? new Boolean(*is_specification) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &ModelElement::set_is_specification,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mIsSpecification) {
	    mIsSpecification->unref();
	}
	
	mIsSpecification = new_value ? new Boolean(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "is_specification", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_constraint(Constraint * constraint) {
	constraint->ref();
	
	int index = mConstraint.size();
	
	mConstraint.push_back(constraint);
	
	ChangeEvent * change = change_role_added(this, "constraint",
						 constraint, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_constraint(Constraint * constraint) {
	mConstraint.remove(constraint);
	
	ChangeEvent * change = change_role_removed(this, "constraint",
						   constraint, -1, false);
	log_change(change);
	change->unref();
	
	constraint->unref();
    }
    
    void // BBB
    ModelElement::set_constraint(const list<Constraint *> & constraint) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Constraint *> before;
	set<Constraint *> after;
	
	before.insert(mConstraint.begin(), mConstraint.end());
	after.insert(constraint.begin(), constraint.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mConstraint.size());
	RETURN_IF_FAIL(after.size() == constraint.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Constraint *> added;
	vector<Constraint *> removed;
	
	set_difference(constraint.begin(), constraint.end(),
		       mConstraint.begin(), mConstraint.end(),
		       back_inserter(added));
	
	set_difference(mConstraint.begin(), mConstraint.end(),
		       constraint.begin(), constraint.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_constraint,
						  constraint, mConstraint);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Constraint::internal_add_constrained_element), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Constraint::internal_remove_constrained_element), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mConstraint = constraint;
	
	// changes made
	
	for(vector<Constraint *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Constraint * constraint = *iter;
	    ChangeEvent * change = change_role_added(this, "constraint",
						     constraint, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Constraint *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Constraint * constraint = *iter;
	    ChangeEvent * change = change_role_removed(this, "constraint",
						       constraint, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_constraint(Constraint * constraint, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mConstraint.size()));
	
	if(index < 0) {
	    index = mConstraint.size();
	}
	
	list<Constraint *>::iterator iter = mConstraint.begin();
	
	for(int i = 0; i < static_cast<int>(mConstraint.size()); ++i) {
	    RETURN_IF_FAIL(*iter != constraint);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_constraint,
						&ModelElement::remove_constraint,
						constraint, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "constraint", constraint, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	constraint->ref();
	
	mConstraint.insert(iter, constraint);
	constraint->internal_add_constrained_element(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_constraint(Constraint * constraint) {
	// work out the index
	
	int index = 0;
	
	list<Constraint *>::iterator iter = mConstraint.begin();
	
	for(; iter != mConstraint.end(); ++iter, ++index) {
	    if(*iter == constraint) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mConstraint.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_constraint,
						&ModelElement::add_constraint,
						constraint, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "constraint", constraint, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	constraint->internal_remove_constrained_element(this);
	mConstraint.erase(iter);
	
	constraint->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // MMM
    ModelElement::internal_set_namespace(Namespace * namespace_) {
	ChangeEvent * change = change_role_changed(this, "namespace",
						   namespace_, mNamespace, false);
	log_change(change);
	
	change->unref();
	
	if(mNamespace) {
	    mNamespace->unref();
	}
	
	mNamespace = namespace_;
	
	if(mNamespace) {
	    mNamespace->ref();
	}
    }
    
    void // LLL
    ModelElement::set_namespace(Namespace * namespace_) {
	if(mNamespace != namespace_) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(namespace_) {
		ASSERT(!ref_get_context() || namespace_->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &ModelElement::set_namespace,
						      namespace_, mNamespace);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "namespace", namespace_,
						       mNamespace, false);
	    log_change(change);
	    change->unref();
	    
	    if(mNamespace) {
		mNamespace->internal_remove_owned_element(this);
		mNamespace->unref();
	    }
	    
	    mNamespace = namespace_;
	    
	    if(mNamespace) {
		mNamespace->internal_add_owned_element(this);
		mNamespace->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // EEE
    ModelElement::internal_add_supplier_dependency(Dependency * supplier_dependency) {
	supplier_dependency->ref();
	
	int index = mSupplierDependency.size();
	
	mSupplierDependency.push_back(supplier_dependency);
	
	ChangeEvent * change = change_role_added(this, "supplier_dependency",
						 supplier_dependency, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_supplier_dependency(Dependency * supplier_dependency) {
	mSupplierDependency.remove(supplier_dependency);
	
	ChangeEvent * change = change_role_removed(this, "supplier_dependency",
						   supplier_dependency, -1, false);
	log_change(change);
	change->unref();
	
	supplier_dependency->unref();
    }
    
    void // BBB
    ModelElement::set_supplier_dependency(const list<Dependency *> & supplier_dependency) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Dependency *> before;
	set<Dependency *> after;
	
	before.insert(mSupplierDependency.begin(), mSupplierDependency.end());
	after.insert(supplier_dependency.begin(), supplier_dependency.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSupplierDependency.size());
	RETURN_IF_FAIL(after.size() == supplier_dependency.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Dependency *> added;
	vector<Dependency *> removed;
	
	set_difference(supplier_dependency.begin(), supplier_dependency.end(),
		       mSupplierDependency.begin(), mSupplierDependency.end(),
		       back_inserter(added));
	
	set_difference(mSupplierDependency.begin(), mSupplierDependency.end(),
		       supplier_dependency.begin(), supplier_dependency.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_supplier_dependency,
						  supplier_dependency, mSupplierDependency);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Dependency::internal_add_supplier), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Dependency::internal_remove_supplier), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSupplierDependency = supplier_dependency;
	
	// changes made
	
	for(vector<Dependency *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Dependency * supplier_dependency = *iter;
	    ChangeEvent * change = change_role_added(this, "supplier_dependency",
						     supplier_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Dependency *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Dependency * supplier_dependency = *iter;
	    ChangeEvent * change = change_role_removed(this, "supplier_dependency",
						       supplier_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_supplier_dependency(Dependency * supplier_dependency, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSupplierDependency.size()));
	
	if(index < 0) {
	    index = mSupplierDependency.size();
	}
	
	list<Dependency *>::iterator iter = mSupplierDependency.begin();
	
	for(int i = 0; i < static_cast<int>(mSupplierDependency.size()); ++i) {
	    RETURN_IF_FAIL(*iter != supplier_dependency);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_supplier_dependency,
						&ModelElement::remove_supplier_dependency,
						supplier_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "supplier_dependency", supplier_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier_dependency->ref();
	
	mSupplierDependency.insert(iter, supplier_dependency);
	supplier_dependency->internal_add_supplier(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_supplier_dependency(Dependency * supplier_dependency) {
	// work out the index
	
	int index = 0;
	
	list<Dependency *>::iterator iter = mSupplierDependency.begin();
	
	for(; iter != mSupplierDependency.end(); ++iter, ++index) {
	    if(*iter == supplier_dependency) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSupplierDependency.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_supplier_dependency,
						&ModelElement::add_supplier_dependency,
						supplier_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "supplier_dependency", supplier_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	supplier_dependency->internal_remove_supplier(this);
	mSupplierDependency.erase(iter);
	
	supplier_dependency->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_client_dependency(Dependency * client_dependency) {
	client_dependency->ref();
	
	int index = mClientDependency.size();
	
	mClientDependency.push_back(client_dependency);
	
	ChangeEvent * change = change_role_added(this, "client_dependency",
						 client_dependency, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_client_dependency(Dependency * client_dependency) {
	mClientDependency.remove(client_dependency);
	
	ChangeEvent * change = change_role_removed(this, "client_dependency",
						   client_dependency, -1, false);
	log_change(change);
	change->unref();
	
	client_dependency->unref();
    }
    
    void // BBB
    ModelElement::set_client_dependency(const list<Dependency *> & client_dependency) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Dependency *> before;
	set<Dependency *> after;
	
	before.insert(mClientDependency.begin(), mClientDependency.end());
	after.insert(client_dependency.begin(), client_dependency.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mClientDependency.size());
	RETURN_IF_FAIL(after.size() == client_dependency.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Dependency *> added;
	vector<Dependency *> removed;
	
	set_difference(client_dependency.begin(), client_dependency.end(),
		       mClientDependency.begin(), mClientDependency.end(),
		       back_inserter(added));
	
	set_difference(mClientDependency.begin(), mClientDependency.end(),
		       client_dependency.begin(), client_dependency.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_client_dependency,
						  client_dependency, mClientDependency);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Dependency::internal_add_client), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Dependency::internal_remove_client), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mClientDependency = client_dependency;
	
	// changes made
	
	for(vector<Dependency *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Dependency * client_dependency = *iter;
	    ChangeEvent * change = change_role_added(this, "client_dependency",
						     client_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Dependency *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Dependency * client_dependency = *iter;
	    ChangeEvent * change = change_role_removed(this, "client_dependency",
						       client_dependency, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_client_dependency(Dependency * client_dependency, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mClientDependency.size()));
	
	if(index < 0) {
	    index = mClientDependency.size();
	}
	
	list<Dependency *>::iterator iter = mClientDependency.begin();
	
	for(int i = 0; i < static_cast<int>(mClientDependency.size()); ++i) {
	    RETURN_IF_FAIL(*iter != client_dependency);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_client_dependency,
						&ModelElement::remove_client_dependency,
						client_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "client_dependency", client_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client_dependency->ref();
	
	mClientDependency.insert(iter, client_dependency);
	client_dependency->internal_add_client(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_client_dependency(Dependency * client_dependency) {
	// work out the index
	
	int index = 0;
	
	list<Dependency *>::iterator iter = mClientDependency.begin();
	
	for(; iter != mClientDependency.end(); ++iter, ++index) {
	    if(*iter == client_dependency) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mClientDependency.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_client_dependency,
						&ModelElement::add_client_dependency,
						client_dependency, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "client_dependency", client_dependency, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	client_dependency->internal_remove_client(this);
	mClientDependency.erase(iter);
	
	client_dependency->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_stereotype(Stereotype * stereotype) {
	stereotype->ref();
	
	int index = mStereotype.size();
	
	mStereotype.push_back(stereotype);
	
	ChangeEvent * change = change_role_added(this, "stereotype",
						 stereotype, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_stereotype(Stereotype * stereotype) {
	mStereotype.remove(stereotype);
	
	ChangeEvent * change = change_role_removed(this, "stereotype",
						   stereotype, -1, false);
	log_change(change);
	change->unref();
	
	stereotype->unref();
    }
    
    void // BBB
    ModelElement::set_stereotype(const list<Stereotype *> & stereotype) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Stereotype *> before;
	set<Stereotype *> after;
	
	before.insert(mStereotype.begin(), mStereotype.end());
	after.insert(stereotype.begin(), stereotype.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mStereotype.size());
	RETURN_IF_FAIL(after.size() == stereotype.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Stereotype *> added;
	vector<Stereotype *> removed;
	
	set_difference(stereotype.begin(), stereotype.end(),
		       mStereotype.begin(), mStereotype.end(),
		       back_inserter(added));
	
	set_difference(mStereotype.begin(), mStereotype.end(),
		       stereotype.begin(), stereotype.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_stereotype,
						  stereotype, mStereotype);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Stereotype::internal_add_extended_element), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Stereotype::internal_remove_extended_element), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mStereotype = stereotype;
	
	// changes made
	
	for(vector<Stereotype *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Stereotype * stereotype = *iter;
	    ChangeEvent * change = change_role_added(this, "stereotype",
						     stereotype, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Stereotype *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Stereotype * stereotype = *iter;
	    ChangeEvent * change = change_role_removed(this, "stereotype",
						       stereotype, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_stereotype(Stereotype * stereotype, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mStereotype.size()));
	
	if(index < 0) {
	    index = mStereotype.size();
	}
	
	list<Stereotype *>::iterator iter = mStereotype.begin();
	
	for(int i = 0; i < static_cast<int>(mStereotype.size()); ++i) {
	    RETURN_IF_FAIL(*iter != stereotype);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_stereotype,
						&ModelElement::remove_stereotype,
						stereotype, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "stereotype", stereotype, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	stereotype->ref();
	
	mStereotype.insert(iter, stereotype);
	stereotype->internal_add_extended_element(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_stereotype(Stereotype * stereotype) {
	// work out the index
	
	int index = 0;
	
	list<Stereotype *>::iterator iter = mStereotype.begin();
	
	for(; iter != mStereotype.end(); ++iter, ++index) {
	    if(*iter == stereotype) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mStereotype.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_stereotype,
						&ModelElement::add_stereotype,
						stereotype, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "stereotype", stereotype, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	stereotype->internal_remove_extended_element(this);
	mStereotype.erase(iter);
	
	stereotype->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_tagged_value(TaggedValue * tagged_value) {
	tagged_value->ref();
	
	int index = mTaggedValue.size();
	
	mTaggedValue.push_back(tagged_value);
	
	ChangeEvent * change = change_role_added(this, "tagged_value",
						 tagged_value, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_tagged_value(TaggedValue * tagged_value) {
	mTaggedValue.remove(tagged_value);
	
	ChangeEvent * change = change_role_removed(this, "tagged_value",
						   tagged_value, -1, true);
	log_change(change);
	change->unref();
	
	tagged_value->unref();
    }
    
    void // BBB
    ModelElement::set_tagged_value(const list<TaggedValue *> & tagged_value) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<TaggedValue *> before;
	set<TaggedValue *> after;
	
	before.insert(mTaggedValue.begin(), mTaggedValue.end());
	after.insert(tagged_value.begin(), tagged_value.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mTaggedValue.size());
	RETURN_IF_FAIL(after.size() == tagged_value.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<TaggedValue *> added;
	vector<TaggedValue *> removed;
	
	set_difference(tagged_value.begin(), tagged_value.end(),
		       mTaggedValue.begin(), mTaggedValue.end(),
		       back_inserter(added));
	
	set_difference(mTaggedValue.begin(), mTaggedValue.end(),
		       tagged_value.begin(), tagged_value.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_tagged_value,
						  tagged_value, mTaggedValue);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_set_owner), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_set_owner),
			 static_cast<ModelElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mTaggedValue = tagged_value;
	
	// changes made
	
	for(vector<TaggedValue *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    TaggedValue * tagged_value = *iter;
	    ChangeEvent * change = change_role_added(this, "tagged_value",
						     tagged_value, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<TaggedValue *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    TaggedValue * tagged_value = *iter;
	    ChangeEvent * change = change_role_removed(this, "tagged_value",
						       tagged_value, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_tagged_value(TaggedValue * tagged_value, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mTaggedValue.size()));
	
	if(index < 0) {
	    index = mTaggedValue.size();
	}
	
	list<TaggedValue *>::iterator iter = mTaggedValue.begin();
	
	for(int i = 0; i < static_cast<int>(mTaggedValue.size()); ++i) {
	    RETURN_IF_FAIL(*iter != tagged_value);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_tagged_value,
						&ModelElement::remove_tagged_value,
						tagged_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "tagged_value", tagged_value, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	tagged_value->ref();
	
	mTaggedValue.insert(iter, tagged_value);
	tagged_value->internal_set_owner(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_tagged_value(TaggedValue * tagged_value) {
	// work out the index
	
	int index = 0;
	
	list<TaggedValue *>::iterator iter = mTaggedValue.begin();
	
	for(; iter != mTaggedValue.end(); ++iter, ++index) {
	    if(*iter == tagged_value) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mTaggedValue.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_tagged_value,
						&ModelElement::add_tagged_value,
						tagged_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "tagged_value", tagged_value, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	tagged_value->internal_set_owner(static_cast<ModelElement *>(0));
	mTaggedValue.erase(iter);
	
	tagged_value->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_reference_tag(TaggedValue * reference_tag) {
	reference_tag->ref();
	
	int index = mReferenceTag.size();
	
	mReferenceTag.push_back(reference_tag);
	
	ChangeEvent * change = change_role_added(this, "reference_tag",
						 reference_tag, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_reference_tag(TaggedValue * reference_tag) {
	mReferenceTag.remove(reference_tag);
	
	ChangeEvent * change = change_role_removed(this, "reference_tag",
						   reference_tag, -1, false);
	log_change(change);
	change->unref();
	
	reference_tag->unref();
    }
    
    void // BBB
    ModelElement::set_reference_tag(const list<TaggedValue *> & reference_tag) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<TaggedValue *> before;
	set<TaggedValue *> after;
	
	before.insert(mReferenceTag.begin(), mReferenceTag.end());
	after.insert(reference_tag.begin(), reference_tag.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mReferenceTag.size());
	RETURN_IF_FAIL(after.size() == reference_tag.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<TaggedValue *> added;
	vector<TaggedValue *> removed;
	
	set_difference(reference_tag.begin(), reference_tag.end(),
		       mReferenceTag.begin(), mReferenceTag.end(),
		       back_inserter(added));
	
	set_difference(mReferenceTag.begin(), mReferenceTag.end(),
		       reference_tag.begin(), reference_tag.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_reference_tag,
						  reference_tag, mReferenceTag);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_add_reference_value), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_remove_reference_value), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mReferenceTag = reference_tag;
	
	// changes made
	
	for(vector<TaggedValue *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    TaggedValue * reference_tag = *iter;
	    ChangeEvent * change = change_role_added(this, "reference_tag",
						     reference_tag, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<TaggedValue *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    TaggedValue * reference_tag = *iter;
	    ChangeEvent * change = change_role_removed(this, "reference_tag",
						       reference_tag, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_reference_tag(TaggedValue * reference_tag, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mReferenceTag.size()));
	
	if(index < 0) {
	    index = mReferenceTag.size();
	}
	
	list<TaggedValue *>::iterator iter = mReferenceTag.begin();
	
	for(int i = 0; i < static_cast<int>(mReferenceTag.size()); ++i) {
	    RETURN_IF_FAIL(*iter != reference_tag);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_reference_tag,
						&ModelElement::remove_reference_tag,
						reference_tag, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "reference_tag", reference_tag, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	reference_tag->ref();
	
	mReferenceTag.insert(iter, reference_tag);
	reference_tag->internal_add_reference_value(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_reference_tag(TaggedValue * reference_tag) {
	// work out the index
	
	int index = 0;
	
	list<TaggedValue *>::iterator iter = mReferenceTag.begin();
	
	for(; iter != mReferenceTag.end(); ++iter, ++index) {
	    if(*iter == reference_tag) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mReferenceTag.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_reference_tag,
						&ModelElement::add_reference_tag,
						reference_tag, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "reference_tag", reference_tag, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	reference_tag->internal_remove_reference_value(this);
	mReferenceTag.erase(iter);
	
	reference_tag->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_comment(Comment * comment) {
	comment->ref();
	
	int index = mComment.size();
	
	mComment.push_back(comment);
	
	ChangeEvent * change = change_role_added(this, "comment",
						 comment, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_comment(Comment * comment) {
	mComment.remove(comment);
	
	ChangeEvent * change = change_role_removed(this, "comment",
						   comment, -1, false);
	log_change(change);
	change->unref();
	
	comment->unref();
    }
    
    void // BBB
    ModelElement::set_comment(const list<Comment *> & comment) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Comment *> before;
	set<Comment *> after;
	
	before.insert(mComment.begin(), mComment.end());
	after.insert(comment.begin(), comment.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mComment.size());
	RETURN_IF_FAIL(after.size() == comment.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Comment *> added;
	vector<Comment *> removed;
	
	set_difference(comment.begin(), comment.end(),
		       mComment.begin(), mComment.end(),
		       back_inserter(added));
	
	set_difference(mComment.begin(), mComment.end(),
		       comment.begin(), comment.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_comment,
						  comment, mComment);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Comment::internal_add_annotated_element), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Comment::internal_remove_annotated_element), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mComment = comment;
	
	// changes made
	
	for(vector<Comment *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Comment * comment = *iter;
	    ChangeEvent * change = change_role_added(this, "comment",
						     comment, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Comment *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Comment * comment = *iter;
	    ChangeEvent * change = change_role_removed(this, "comment",
						       comment, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_comment(Comment * comment, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mComment.size()));
	
	if(index < 0) {
	    index = mComment.size();
	}
	
	list<Comment *>::iterator iter = mComment.begin();
	
	for(int i = 0; i < static_cast<int>(mComment.size()); ++i) {
	    RETURN_IF_FAIL(*iter != comment);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_comment,
						&ModelElement::remove_comment,
						comment, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "comment", comment, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	comment->ref();
	
	mComment.insert(iter, comment);
	comment->internal_add_annotated_element(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_comment(Comment * comment) {
	// work out the index
	
	int index = 0;
	
	list<Comment *>::iterator iter = mComment.begin();
	
	for(; iter != mComment.end(); ++iter, ++index) {
	    if(*iter == comment) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mComment.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_comment,
						&ModelElement::add_comment,
						comment, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "comment", comment, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	comment->internal_remove_annotated_element(this);
	mComment.erase(iter);
	
	comment->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ModelElement::internal_add_presentation(PresentationElement * presentation) {
	presentation->ref();
	
	int index = mPresentation.size();
	
	mPresentation.push_back(presentation);
	
	ChangeEvent * change = change_role_added(this, "presentation",
						 presentation, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ModelElement::internal_remove_presentation(PresentationElement * presentation) {
	mPresentation.remove(presentation);
	
	ChangeEvent * change = change_role_removed(this, "presentation",
						   presentation, -1, true);
	log_change(change);
	change->unref();
	
	presentation->unref();
    }
    
    void // BBB
    ModelElement::set_presentation(const list<PresentationElement *> & presentation) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<PresentationElement *> before;
	set<PresentationElement *> after;
	
	before.insert(mPresentation.begin(), mPresentation.end());
	after.insert(presentation.begin(), presentation.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mPresentation.size());
	RETURN_IF_FAIL(after.size() == presentation.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<PresentationElement *> added;
	vector<PresentationElement *> removed;
	
	set_difference(presentation.begin(), presentation.end(),
		       mPresentation.begin(), mPresentation.end(),
		       back_inserter(added));
	
	set_difference(mPresentation.begin(), mPresentation.end(),
		       presentation.begin(), presentation.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ModelElement::set_presentation,
						  presentation, mPresentation);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&PresentationElement::internal_set_subject), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&PresentationElement::internal_set_subject),
			 static_cast<ModelElement *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mPresentation = presentation;
	
	// changes made
	
	for(vector<PresentationElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    PresentationElement * presentation = *iter;
	    ChangeEvent * change = change_role_added(this, "presentation",
						     presentation, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<PresentationElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    PresentationElement * presentation = *iter;
	    ChangeEvent * change = change_role_removed(this, "presentation",
						       presentation, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ModelElement::add_presentation(PresentationElement * presentation, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mPresentation.size()));
	
	if(index < 0) {
	    index = mPresentation.size();
	}
	
	list<PresentationElement *>::iterator iter = mPresentation.begin();
	
	for(int i = 0; i < static_cast<int>(mPresentation.size()); ++i) {
	    RETURN_IF_FAIL(*iter != presentation);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::add_presentation,
						&ModelElement::remove_presentation,
						presentation, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "presentation", presentation, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	presentation->ref();
	
	mPresentation.insert(iter, presentation);
	presentation->internal_set_subject(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ModelElement::remove_presentation(PresentationElement * presentation) {
	// work out the index
	
	int index = 0;
	
	list<PresentationElement *>::iterator iter = mPresentation.begin();
	
	for(; iter != mPresentation.end(); ++iter, ++index) {
	    if(*iter == presentation) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mPresentation.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ModelElement::remove_presentation,
						&ModelElement::add_presentation,
						presentation, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "presentation", presentation, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	presentation->internal_set_subject(static_cast<ModelElement *>(0));
	mPresentation.erase(iter);
	
	presentation->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    ModelElement::ref_set_context_impl(set<string> & visited,
				       REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    Element::ref_set_context_impl(visited, context);
	}
    }
    
    void
    ModelElement::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_model_element(this);
	    
	    Element::ref_implant_impl(visited);
	}
    }
    
    void
    ModelElement::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    Element::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_model_element(this);
	}
    }
    
    bool
    ModelElement::ref_add_role_impl(set<string> & visited,
				    const string & role_name,
				    REF::Class * role,
				    REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constraint") {
		Constraint * constraint = dynamic_cast<Constraint *>(role);
		
		if(!constraint) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_constraint(constraint, index);
		return true;
	    }
	    
	    if(role_name == "supplier_dependency") {
		Dependency * supplier_dependency = dynamic_cast<Dependency *>(role);
		
		if(!supplier_dependency) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_supplier_dependency(supplier_dependency, index);
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		Dependency * client_dependency = dynamic_cast<Dependency *>(role);
		
		if(!client_dependency) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_client_dependency(client_dependency, index);
		return true;
	    }
	    
	    if(role_name == "stereotype") {
		Stereotype * stereotype = dynamic_cast<Stereotype *>(role);
		
		if(!stereotype) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_stereotype(stereotype, index);
		return true;
	    }
	    
	    if(role_name == "tagged_value") {
		TaggedValue * tagged_value = dynamic_cast<TaggedValue *>(role);
		
		if(!tagged_value) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_tagged_value(tagged_value, index);
		return true;
	    }
	    
	    if(role_name == "reference_tag") {
		TaggedValue * reference_tag = dynamic_cast<TaggedValue *>(role);
		
		if(!reference_tag) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_reference_tag(reference_tag, index);
		return true;
	    }
	    
	    if(role_name == "comment") {
		Comment * comment = dynamic_cast<Comment *>(role);
		
		if(!comment) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_comment(comment, index);
		return true;
	    }
	    
	    if(role_name == "presentation") {
		PresentationElement * presentation = dynamic_cast<PresentationElement *>(role);
		
		if(!presentation) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_presentation(presentation, index);
		return true;
	    }
	    
	    if(Element::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_remove_role_impl(set<string> & visited,
				       const string & role_name,
				       REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Element::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_get_role_impl(set<string> & visited,
				    REF::Class * & role,
				    const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "namespace") {
		role = get_namespace();
		return true;
	    }
	    
	    if(Element::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_set_role_impl(set<string> & visited,
				    const string & role_name,
				    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "namespace") {
		Namespace * namespace_ = dynamic_cast<Namespace *>(role);
		
		if(!namespace_) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_namespace(namespace_);
		return true;
	    }
	    
	    if(Element::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_create_class_impl(set<string> & visited,
					REF::Class * & role,
					const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constraint") {
		role = new Constraint();
		return true;
	    }
	    
	    if(role_name == "namespace") {
		role = new Namespace();
		return true;
	    }
	    
	    if(role_name == "supplier_dependency") {
		role = new Dependency();
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		role = new Dependency();
		return true;
	    }
	    
	    if(role_name == "stereotype") {
		role = new Stereotype();
		return true;
	    }
	    
	    if(role_name == "tagged_value") {
		role = new TaggedValue();
		return true;
	    }
	    
	    if(role_name == "reference_tag") {
		role = new TaggedValue();
		return true;
	    }
	    
	    if(role_name == "comment") {
		role = new Comment();
		return true;
	    }
	    
	    if(role_name == "presentation") {
		role = new PresentationElement();
		return true;
	    }
	    
	    if(Element::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_get_role_iterator_impl(set<string> & visited,
					     REF::Iterator * & iterator,
					     const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "constraint") {
		iterator = REF::iterator<Constraint>(get_constraint());
		return true;
	    }
	    
	    if(role_name == "namespace") {
		iterator = REF::iterator<Namespace>(get_namespace());
		return true;
	    }
	    
	    if(role_name == "supplier_dependency") {
		iterator = REF::iterator<Dependency>(get_supplier_dependency());
		return true;
	    }
	    
	    if(role_name == "client_dependency") {
		iterator = REF::iterator<Dependency>(get_client_dependency());
		return true;
	    }
	    
	    if(role_name == "stereotype") {
		iterator = REF::iterator<Stereotype>(get_stereotype());
		return true;
	    }
	    
	    if(role_name == "tagged_value") {
		iterator = REF::iterator<TaggedValue>(get_tagged_value());
		return true;
	    }
	    
	    if(role_name == "reference_tag") {
		iterator = REF::iterator<TaggedValue>(get_reference_tag());
		return true;
	    }
	    
	    if(role_name == "comment") {
		iterator = REF::iterator<Comment>(get_comment());
		return true;
	    }
	    
	    if(role_name == "presentation") {
		iterator = REF::iterator<PresentationElement>(get_presentation());
		return true;
	    }
	    
	    if(Element::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_get_attribute_impl(set<string> & visited,
					 const REF::Attribute * & attribute,
					 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		attribute = get_name();
		return true;
	    }
	    
	    if(attribute_name == "is_specification") {
		attribute = get_is_specification();
		return true;
	    }
	    
	    if(Element::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_set_attribute_impl(set<string> & visited,
					 const string & attribute_name,
					 const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		const Name * name = dynamic_cast<const Name *>(attribute);
		
		if(!name) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_name(name);
		return true;
	    }
	    
	    if(attribute_name == "is_specification") {
		const Boolean * is_specification = dynamic_cast<const Boolean *>(attribute);
		
		if(!is_specification) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_is_specification(is_specification);
		return true;
	    }
	    
	    if(Element::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_get_attribute_set_impl(set<string> & visited,
					     const vector<const REF::Attribute *> * & attribute_set,
					     const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Element::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_set_attribute_set_impl(set<string> & visited,
					     const string & attribute_name,
					     const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(Element::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_create_attribute_impl(set<string> & visited,
					    REF::Attribute * & attribute,
					    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		attribute = new Name();
		return true;
	    }
	    
	    if(attribute_name == "is_specification") {
		attribute = new Boolean();
		return true;
	    }
	    
	    if(Element::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ModelElement::ref_is_a_type_of_impl(set<string> & visited,
					const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "ModelElement") {
		return true;
	    }
	    
	    if(Element::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    ModelElement::ref_get_type() const {
	return TYPE;
    }
    
    void
    ModelElement::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<TaggedValue *> tagged_value_list = get_tagged_value();
	    for_each(tagged_value_list.begin(), tagged_value_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<PresentationElement *> presentation_list = get_presentation();
	    for_each(presentation_list.begin(), presentation_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<Constraint *> constraint;
	    set_constraint(constraint);
	    
	    Namespace * namespace_ = 0;
	    set_namespace(namespace_);
	    
	    list<Dependency *> supplier_dependency;
	    set_supplier_dependency(supplier_dependency);
	    
	    list<Dependency *> client_dependency;
	    set_client_dependency(client_dependency);
	    
	    list<Stereotype *> stereotype;
	    set_stereotype(stereotype);
	    
	    list<TaggedValue *> tagged_value;
	    set_tagged_value(tagged_value);
	    
	    list<TaggedValue *> reference_tag;
	    set_reference_tag(reference_tag);
	    
	    list<Comment *> comment;
	    set_comment(comment);
	    
	    list<PresentationElement *> presentation;
	    set_presentation(presentation);
	    
	    Element::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
