// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "namespace.h"

#include "model_element.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Namespace::TYPE = "Namespace";
    
    Namespace::Namespace() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Namespace::Namespace(const Namespace & namespace_) {
	ASSERT_NOT_REACHED();
    }
    
    Namespace::~Namespace() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Namespace::internal_add_owned_element(ModelElement * owned_element) {
	owned_element->ref();
	
	int index = mOwnedElement.size();
	
	mOwnedElement.push_back(owned_element);
	
	ChangeEvent * change = change_role_added(this, "owned_element",
						 owned_element, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Namespace::internal_remove_owned_element(ModelElement * owned_element) {
	mOwnedElement.remove(owned_element);
	
	ChangeEvent * change = change_role_removed(this, "owned_element",
						   owned_element, -1, true);
	log_change(change);
	change->unref();
	
	owned_element->unref();
    }
    
    void // BBB
    Namespace::set_owned_element(const list<ModelElement *> & owned_element) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mOwnedElement.begin(), mOwnedElement.end());
	after.insert(owned_element.begin(), owned_element.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mOwnedElement.size());
	RETURN_IF_FAIL(after.size() == owned_element.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(owned_element.begin(), owned_element.end(),
		       mOwnedElement.begin(), mOwnedElement.end(),
		       back_inserter(added));
	
	set_difference(mOwnedElement.begin(), mOwnedElement.end(),
		       owned_element.begin(), owned_element.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Namespace::set_owned_element,
						  owned_element, mOwnedElement);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_set_namespace), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_set_namespace),
			 static_cast<Namespace *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mOwnedElement = owned_element;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * owned_element = *iter;
	    ChangeEvent * change = change_role_added(this, "owned_element",
						     owned_element, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * owned_element = *iter;
	    ChangeEvent * change = change_role_removed(this, "owned_element",
						       owned_element, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Namespace::add_owned_element(ModelElement * owned_element, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mOwnedElement.size()));
	
	if(index < 0) {
	    index = mOwnedElement.size();
	}
	
	list<ModelElement *>::iterator iter = mOwnedElement.begin();
	
	for(int i = 0; i < static_cast<int>(mOwnedElement.size()); ++i) {
	    RETURN_IF_FAIL(*iter != owned_element);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Namespace::add_owned_element,
						&Namespace::remove_owned_element,
						owned_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "owned_element", owned_element, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	owned_element->ref();
	
	mOwnedElement.insert(iter, owned_element);
	owned_element->internal_set_namespace(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Namespace::remove_owned_element(ModelElement * owned_element) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mOwnedElement.begin();
	
	for(; iter != mOwnedElement.end(); ++iter, ++index) {
	    if(*iter == owned_element) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mOwnedElement.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Namespace::remove_owned_element,
						&Namespace::add_owned_element,
						owned_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "owned_element", owned_element, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	owned_element->internal_set_namespace(static_cast<Namespace *>(0));
	mOwnedElement.erase(iter);
	
	owned_element->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Namespace::ref_set_context_impl(set<string> & visited,
				    REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Namespace::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_namespace(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Namespace::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_namespace(this);
	}
    }
    
    bool
    Namespace::ref_add_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role,
				 REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owned_element") {
		ModelElement * owned_element = dynamic_cast<ModelElement *>(role);
		
		if(!owned_element) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_owned_element(owned_element, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_remove_role_impl(set<string> & visited,
				    const string & role_name,
				    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_get_role_impl(set<string> & visited,
				 REF::Class * & role,
				 const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_set_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_create_class_impl(set<string> & visited,
				     REF::Class * & role,
				     const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owned_element") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_get_role_iterator_impl(set<string> & visited,
					  REF::Iterator * & iterator,
					  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owned_element") {
		iterator = REF::iterator<ModelElement>(get_owned_element());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_get_attribute_impl(set<string> & visited,
				      const REF::Attribute * & attribute,
				      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_set_attribute_impl(set<string> & visited,
				      const string & attribute_name,
				      const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_get_attribute_set_impl(set<string> & visited,
					  const vector<const REF::Attribute *> * & attribute_set,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_set_attribute_set_impl(set<string> & visited,
					  const string & attribute_name,
					  const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_create_attribute_impl(set<string> & visited,
					 REF::Attribute * & attribute,
					 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Namespace::ref_is_a_type_of_impl(set<string> & visited,
				     const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Namespace") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Namespace::ref_get_type() const {
	return TYPE;
    }
    
    void
    Namespace::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<ModelElement *> owned_element_list = get_owned_element();
	    for_each(owned_element_list.begin(), owned_element_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<ModelElement *> owned_element;
	    set_owned_element(owned_element);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
