// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "tag_definition.h"

#include "stereotype.h"
#include "tagged_value.h"
#include "uml/foundation/data_types/name.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string TagDefinition::TYPE = "TagDefinition";
    
    TagDefinition::TagDefinition() {
	mOwner = 0;
	mTagType = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    TagDefinition::TagDefinition(const TagDefinition & tag_definition) {
	ASSERT_NOT_REACHED();
    }
    
    TagDefinition::~TagDefinition() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mTagType) {
	    mTagType->unref();
	}
    }
    
    void // AAA
    TagDefinition::set_tag_type(const Name * tag_type) {
	begin_transaction();
	
	//
	
	Name * old_value = mTagType ? new Name(*mTagType) : 0;
	Name * new_value = tag_type ? new Name(*tag_type) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &TagDefinition::set_tag_type,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mTagType) {
	    mTagType->unref();
	}
	
	mTagType = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "tag_type", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // MMM
    TagDefinition::internal_set_owner(Stereotype * owner) {
	ChangeEvent * change = change_role_changed(this, "owner",
						   owner, mOwner, false);
	log_change(change);
	
	change->unref();
	
	if(mOwner) {
	    mOwner->unref();
	}
	
	mOwner = owner;
	
	if(mOwner) {
	    mOwner->ref();
	}
    }
    
    void // LLL
    TagDefinition::set_owner(Stereotype * owner) {
	if(mOwner != owner) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(owner) {
		ASSERT(!ref_get_context() || owner->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &TagDefinition::set_owner,
						      owner, mOwner);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "owner", owner,
						       mOwner, false);
	    log_change(change);
	    change->unref();
	    
	    if(mOwner) {
		mOwner->internal_remove_defined_tag(this);
		mOwner->unref();
	    }
	    
	    mOwner = owner;
	    
	    if(mOwner) {
		mOwner->internal_add_defined_tag(this);
		mOwner->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // EEE
    TagDefinition::internal_add_typed_value(TaggedValue * typed_value) {
	typed_value->ref();
	
	int index = mTypedValue.size();
	
	mTypedValue.push_back(typed_value);
	
	ChangeEvent * change = change_role_added(this, "typed_value",
						 typed_value, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    TagDefinition::internal_remove_typed_value(TaggedValue * typed_value) {
	mTypedValue.remove(typed_value);
	
	ChangeEvent * change = change_role_removed(this, "typed_value",
						   typed_value, -1, false);
	log_change(change);
	change->unref();
	
	typed_value->unref();
    }
    
    void // BBB
    TagDefinition::set_typed_value(const list<TaggedValue *> & typed_value) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<TaggedValue *> before;
	set<TaggedValue *> after;
	
	before.insert(mTypedValue.begin(), mTypedValue.end());
	after.insert(typed_value.begin(), typed_value.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mTypedValue.size());
	RETURN_IF_FAIL(after.size() == typed_value.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<TaggedValue *> added;
	vector<TaggedValue *> removed;
	
	set_difference(typed_value.begin(), typed_value.end(),
		       mTypedValue.begin(), mTypedValue.end(),
		       back_inserter(added));
	
	set_difference(mTypedValue.begin(), mTypedValue.end(),
		       typed_value.begin(), typed_value.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &TagDefinition::set_typed_value,
						  typed_value, mTypedValue);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_set_type), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&TaggedValue::internal_set_type),
			 static_cast<TagDefinition *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mTypedValue = typed_value;
	
	// changes made
	
	for(vector<TaggedValue *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    TaggedValue * typed_value = *iter;
	    ChangeEvent * change = change_role_added(this, "typed_value",
						     typed_value, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<TaggedValue *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    TaggedValue * typed_value = *iter;
	    ChangeEvent * change = change_role_removed(this, "typed_value",
						       typed_value, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    TagDefinition::add_typed_value(TaggedValue * typed_value, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mTypedValue.size()));
	
	if(index < 0) {
	    index = mTypedValue.size();
	}
	
	list<TaggedValue *>::iterator iter = mTypedValue.begin();
	
	for(int i = 0; i < static_cast<int>(mTypedValue.size()); ++i) {
	    RETURN_IF_FAIL(*iter != typed_value);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&TagDefinition::add_typed_value,
						&TagDefinition::remove_typed_value,
						typed_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "typed_value", typed_value, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_value->ref();
	
	mTypedValue.insert(iter, typed_value);
	typed_value->internal_set_type(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    TagDefinition::remove_typed_value(TaggedValue * typed_value) {
	// work out the index
	
	int index = 0;
	
	list<TaggedValue *>::iterator iter = mTypedValue.begin();
	
	for(; iter != mTypedValue.end(); ++iter, ++index) {
	    if(*iter == typed_value) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mTypedValue.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&TagDefinition::remove_typed_value,
						&TagDefinition::add_typed_value,
						typed_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "typed_value", typed_value, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	typed_value->internal_set_type(static_cast<TagDefinition *>(0));
	mTypedValue.erase(iter);
	
	typed_value->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    TagDefinition::ref_set_context_impl(set<string> & visited,
					REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    TagDefinition::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_tag_definition(this);
	    
	    GeneralizableElement::ref_implant_impl(visited);
	}
    }
    
    void
    TagDefinition::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_tag_definition(this);
	}
    }
    
    bool
    TagDefinition::ref_add_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role,
				     REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "typed_value") {
		TaggedValue * typed_value = dynamic_cast<TaggedValue *>(role);
		
		if(!typed_value) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_typed_value(typed_value, index);
		return true;
	    }
	    
	    if(GeneralizableElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_remove_role_impl(set<string> & visited,
					const string & role_name,
					REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_get_role_impl(set<string> & visited,
				     REF::Class * & role,
				     const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = get_owner();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_set_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		Stereotype * owner = dynamic_cast<Stereotype *>(role);
		
		if(!owner) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_owner(owner);
		return true;
	    }
	    
	    if(GeneralizableElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_create_class_impl(set<string> & visited,
					 REF::Class * & role,
					 const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = new Stereotype();
		return true;
	    }
	    
	    if(role_name == "typed_value") {
		role = new TaggedValue();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_get_role_iterator_impl(set<string> & visited,
					      REF::Iterator * & iterator,
					      const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		iterator = REF::iterator<Stereotype>(get_owner());
		return true;
	    }
	    
	    if(role_name == "typed_value") {
		iterator = REF::iterator<TaggedValue>(get_typed_value());
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_get_attribute_impl(set<string> & visited,
					  const REF::Attribute * & attribute,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "tag_type") {
		attribute = get_tag_type();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_set_attribute_impl(set<string> & visited,
					  const string & attribute_name,
					  const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "tag_type") {
		const Name * tag_type = dynamic_cast<const Name *>(attribute);
		
		if(!tag_type) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_tag_type(tag_type);
		return true;
	    }
	    
	    if(GeneralizableElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_get_attribute_set_impl(set<string> & visited,
					      const vector<const REF::Attribute *> * & attribute_set,
					      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_set_attribute_set_impl(set<string> & visited,
					      const string & attribute_name,
					      const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_create_attribute_impl(set<string> & visited,
					     REF::Attribute * & attribute,
					     const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "tag_type") {
		attribute = new Name();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TagDefinition::ref_is_a_type_of_impl(set<string> & visited,
					 const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "TagDefinition") {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    TagDefinition::ref_get_type() const {
	return TYPE;
    }
    
    void
    TagDefinition::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    Stereotype * owner = 0;
	    set_owner(owner);
	    
	    list<TaggedValue *> typed_value;
	    set_typed_value(typed_value);
	    
	    GeneralizableElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
