// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "tagged_value.h"

#include "tag_definition.h"
#include "uml/foundation/core/model_element.h"
#include "uml/foundation/data_types/name.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string TaggedValue::TYPE = "TaggedValue";
    
    TaggedValue::TaggedValue() {
	mOwner = 0;
	mType = 0;
	mDataValue = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    TaggedValue::TaggedValue(const TaggedValue & tagged_value) {
	ASSERT_NOT_REACHED();
    }
    
    TaggedValue::~TaggedValue() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mDataValue) {
	    mDataValue->unref();
	}
    }
    
    void // AAA
    TaggedValue::set_data_value(const Name * data_value) {
	begin_transaction();
	
	//
	
	Name * old_value = mDataValue ? new Name(*mDataValue) : 0;
	Name * new_value = data_value ? new Name(*data_value) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &TaggedValue::set_data_value,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mDataValue) {
	    mDataValue->unref();
	}
	
	mDataValue = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "data_value", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // MMM
    TaggedValue::internal_set_owner(ModelElement * owner) {
	ChangeEvent * change = change_role_changed(this, "owner",
						   owner, mOwner, false);
	log_change(change);
	
	change->unref();
	
	if(mOwner) {
	    mOwner->unref();
	}
	
	mOwner = owner;
	
	if(mOwner) {
	    mOwner->ref();
	}
    }
    
    void // LLL
    TaggedValue::set_owner(ModelElement * owner) {
	if(mOwner != owner) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(owner) {
		ASSERT(!ref_get_context() || owner->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &TaggedValue::set_owner,
						      owner, mOwner);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "owner", owner,
						       mOwner, false);
	    log_change(change);
	    change->unref();
	    
	    if(mOwner) {
		mOwner->internal_remove_tagged_value(this);
		mOwner->unref();
	    }
	    
	    mOwner = owner;
	    
	    if(mOwner) {
		mOwner->internal_add_tagged_value(this);
		mOwner->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // EEE
    TaggedValue::internal_add_reference_value(ModelElement * reference_value) {
	reference_value->ref();
	
	int index = mReferenceValue.size();
	
	mReferenceValue.push_back(reference_value);
	
	ChangeEvent * change = change_role_added(this, "reference_value",
						 reference_value, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    TaggedValue::internal_remove_reference_value(ModelElement * reference_value) {
	mReferenceValue.remove(reference_value);
	
	ChangeEvent * change = change_role_removed(this, "reference_value",
						   reference_value, -1, false);
	log_change(change);
	change->unref();
	
	reference_value->unref();
    }
    
    void // BBB
    TaggedValue::set_reference_value(const list<ModelElement *> & reference_value) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mReferenceValue.begin(), mReferenceValue.end());
	after.insert(reference_value.begin(), reference_value.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mReferenceValue.size());
	RETURN_IF_FAIL(after.size() == reference_value.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(reference_value.begin(), reference_value.end(),
		       mReferenceValue.begin(), mReferenceValue.end(),
		       back_inserter(added));
	
	set_difference(mReferenceValue.begin(), mReferenceValue.end(),
		       reference_value.begin(), reference_value.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &TaggedValue::set_reference_value,
						  reference_value, mReferenceValue);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_reference_tag), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_reference_tag), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mReferenceValue = reference_value;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * reference_value = *iter;
	    ChangeEvent * change = change_role_added(this, "reference_value",
						     reference_value, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * reference_value = *iter;
	    ChangeEvent * change = change_role_removed(this, "reference_value",
						       reference_value, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    TaggedValue::add_reference_value(ModelElement * reference_value, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mReferenceValue.size()));
	
	if(index < 0) {
	    index = mReferenceValue.size();
	}
	
	list<ModelElement *>::iterator iter = mReferenceValue.begin();
	
	for(int i = 0; i < static_cast<int>(mReferenceValue.size()); ++i) {
	    RETURN_IF_FAIL(*iter != reference_value);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&TaggedValue::add_reference_value,
						&TaggedValue::remove_reference_value,
						reference_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "reference_value", reference_value, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	reference_value->ref();
	
	mReferenceValue.insert(iter, reference_value);
	reference_value->internal_add_reference_tag(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    TaggedValue::remove_reference_value(ModelElement * reference_value) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mReferenceValue.begin();
	
	for(; iter != mReferenceValue.end(); ++iter, ++index) {
	    if(*iter == reference_value) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mReferenceValue.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&TaggedValue::remove_reference_value,
						&TaggedValue::add_reference_value,
						reference_value, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "reference_value", reference_value, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	reference_value->internal_remove_reference_tag(this);
	mReferenceValue.erase(iter);
	
	reference_value->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // MMM
    TaggedValue::internal_set_type(TagDefinition * type) {
	ChangeEvent * change = change_role_changed(this, "type",
						   type, mType, false);
	log_change(change);
	
	change->unref();
	
	if(mType) {
	    mType->unref();
	}
	
	mType = type;
	
	if(mType) {
	    mType->ref();
	}
    }
    
    void // LLL
    TaggedValue::set_type(TagDefinition * type) {
	if(mType != type) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(type) {
		ASSERT(!type->ref_get_context() ||
		       type->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &TaggedValue::set_type,
						      type, mType);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "type", type,
						       mType, false);
	    log_change(change);
	    change->unref();
	    
	    if(mType) {
		mType->internal_remove_typed_value(this);
		mType->unref();
	    }
	    
	    mType = type;
	    
	    if(mType) {
		mType->internal_add_typed_value(this);
		mType->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void
    TaggedValue::ref_set_context_impl(set<string> & visited,
				      REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    TaggedValue::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_tagged_value(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    TaggedValue::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_tagged_value(this);
	}
    }
    
    bool
    TaggedValue::ref_add_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role,
				   REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "reference_value") {
		ModelElement * reference_value = dynamic_cast<ModelElement *>(role);
		
		if(!reference_value) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_reference_value(reference_value, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_remove_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_get_role_impl(set<string> & visited,
				   REF::Class * & role,
				   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = get_owner();
		return true;
	    }
	    
	    if(role_name == "type") {
		role = get_type();
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_set_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		ModelElement * owner = dynamic_cast<ModelElement *>(role);
		
		if(!owner) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_owner(owner);
		return true;
	    }
	    
	    if(role_name == "type") {
		TagDefinition * type = dynamic_cast<TagDefinition *>(role);
		
		if(!type) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_type(type);
		return true;
	    }
	    
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_create_class_impl(set<string> & visited,
				       REF::Class * & role,
				       const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "reference_value") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "type") {
		role = new TagDefinition();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_get_role_iterator_impl(set<string> & visited,
					    REF::Iterator * & iterator,
					    const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		iterator = REF::iterator<ModelElement>(get_owner());
		return true;
	    }
	    
	    if(role_name == "reference_value") {
		iterator = REF::iterator<ModelElement>(get_reference_value());
		return true;
	    }
	    
	    if(role_name == "type") {
		iterator = REF::iterator<TagDefinition>(get_type());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_get_attribute_impl(set<string> & visited,
					const REF::Attribute * & attribute,
					const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "data_value") {
		attribute = get_data_value();
		return true;
	    }
	    
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_set_attribute_impl(set<string> & visited,
					const string & attribute_name,
					const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "data_value") {
		const Name * data_value = dynamic_cast<const Name *>(attribute);
		
		if(!data_value) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_data_value(data_value);
		return true;
	    }
	    
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_get_attribute_set_impl(set<string> & visited,
					    const vector<const REF::Attribute *> * & attribute_set,
					    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_set_attribute_set_impl(set<string> & visited,
					    const string & attribute_name,
					    const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_create_attribute_impl(set<string> & visited,
					   REF::Attribute * & attribute,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "data_value") {
		attribute = new Name();
		return true;
	    }
	    
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    TaggedValue::ref_is_a_type_of_impl(set<string> & visited,
				       const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "TaggedValue") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    TaggedValue::ref_get_type() const {
	return TYPE;
    }
    
    void
    TaggedValue::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    ModelElement * owner = 0;
	    set_owner(owner);
	    
	    list<ModelElement *> reference_value;
	    set_reference_value(reference_value);
	    
	    TagDefinition * type = 0;
	    set_type(type);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
