// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef XML_NODE_H
#define XML_NODE_H

#include "foundation/object.h"

#include <list>
#include <string>
using namespace std;

#include <iosfwd>
using namespace std;

namespace XML {

    class Property;

    class Node : public Object {
    public:
	Node();
	~Node();

	const string & get_name() const { return mName; }
	void set_name(const string & name);

	const string & get_content() const { return mContent; }
	void set_content(const string & content);

	const list<Property *> & get_properties() { return mProperties; }
	void add_property(Property * property, int index = -1);
	void remove_property(Property * property);
	void set_properties(const list<Property *> & properties);

	const list<Node *> & get_children() { return mChildren; }
	void add_child(Node * node, int index = -1);
	void remove_child(Node * node);
	void set_children(const list<Node *> & children);

	friend ostream & operator << (ostream & ost, const Node & node);

    private:
	string mName;
	string mContent;
	list<Node *> mChildren;
	list<Property *> mProperties;
    };

    ostream & operator << (ostream & ost, const Node & node);

} // namespace XML

#endif // XML_NODE_H
