// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "sax_builder.h"

#include "foundation/macros.h"

#include "node.h"
#include "property.h"

#include <functional>
#include <iterator>
using namespace std;

namespace XML {

    namespace {

	Property * make_property(const pair<string, string> & name_value) {
	    return new Property(name_value.first, name_value.second);
	}

    } // namespace {anonymous}

    SAXBuilder::SAXBuilder() : SAXHandler(),
	mRoot(0) {
    }

    SAXBuilder::~SAXBuilder() {
	if(mRoot) {
	    mRoot->unref();
	}
    }

    void
    SAXBuilder::warning(SAXParser * parser, const string & msg) {
	TRACE(1, "XML warning");
    }

    void
    SAXBuilder::error(SAXParser * parser, const string & msg) {
	TRACE(1, "XML error");
    }

    void
    SAXBuilder::fatal(SAXParser * parser, const string & msg) {
	TRACE(1, "XML fatal");
    }

    void
    SAXBuilder::comment(SAXParser * parser, const string & comment) {
    }

    void
    SAXBuilder::start_document(SAXParser * parser) {
    }

    void
    SAXBuilder::end_document(SAXParser * parser) {
    }

    void
    SAXBuilder::start_element(SAXParser * parser,
			      const string & name, const Properties & properties) {
	mCharacters.resize(0);

	Node * node = new Node();

	node->set_name(name);

	list<Property *> props;

	transform(properties.begin(), properties.end(),
		  back_inserter(props), &make_property);

	node->set_properties(props);

	if(mStack.empty()) {
	    ASSERT(!mRoot);

	    if(mRoot) {
		mRoot->unref();
	    }

	    mRoot = node;
	}
	else {
	    mStack.top()->add_child(node);
	    node->unref();
	}

	mStack.push(node);
    }

    void
    SAXBuilder::end_element(SAXParser * parser, const string & name) {
	ASSERT(!mStack.empty());

	if(!mCharacters.empty()) {
	    Node * node = mStack.top();

	    node->set_content(mCharacters);
	}

	mStack.pop();
    }

    void
    SAXBuilder::characters(SAXParser * parser, const string & data) {
	mCharacters += data;
    }

    void
    SAXBuilder::reference(SAXParser * parser, const string & name) {
    }

    void
    SAXBuilder::cdata_block(SAXParser * parser, const string & data) {
    }

} // namespace XML
