#ifndef XML_SAX_HANDLER_H
#define XML_SAX_HANDLER_H

#include "foundation/object.h"

#include "foundation/hash_functions.h"
#include "foundation/hash.h"

namespace XML {

    class SAXParser;

    class Properties : public hash_map<string, string> {
    public:
	const string & get_required_property(const string & name) const;
	bool get_implied_property(const string & name, string & value) const;
	const string & get_implied_property_default(const string & name,
						    const string & def_value) const;
	bool has_property(const string & name) const;
    };

    class SAXHandler : public Object {
    public:
	SAXHandler();
	~SAXHandler();

	virtual void warning(SAXParser * parser, const string & msg) = 0;
	virtual void error(SAXParser * parser, const string & msg) = 0;
	virtual void fatal(SAXParser * parser, const string & msg) = 0;

	virtual void comment(SAXParser * parser, const string & comment) = 0;
	virtual void start_document(SAXParser * parser) = 0;
	virtual void end_document(SAXParser * parser) = 0;
	virtual void start_element(SAXParser * parser, const string & name,
				   const Properties & properties) = 0;
	virtual void end_element(SAXParser * parser, const string & name) = 0;
	virtual void characters(SAXParser * parser, const string & data) = 0;
	virtual void reference(SAXParser * parser, const string & name) = 0;
	virtual void cdata_block(SAXParser * parser, const string & data) = 0;
    };

} // namespace XML

#endif // XML_SAX_HANDLER_H
